@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)ant.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)ant.c	4.02 97/04/16 xlockmore";
 
 #endif
 
@@ -13,6 +13,7 @@ static const char sccsid[] = "@(#)ant.c	4.00 97/01/01 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
+ * 16-Apr-97: -neighbors 3 and 8 added
  * 01-Jan-97: Updated ant.c to handle more kinds of ants.  Thanks to
  *            J Austin David <Austin.David@tlogic.com>.  Check it out in
  *            java at http://havoc.gtf.gatech.edu/austin  He thought up the
@@ -37,9 +38,11 @@ static const char sccsid[] = "@(#)ant.c	4.00 97/01/01 xlockmore";
 /*-
   Species Grid     Number of Neigbors
   ------- ----     ------------------
-  Ants    Square   4 (or 8)          <- 8 is not implemented
+  Ants    Square   4 or 8
   Bees    Hexagon  6
-  ?       Triangle (3, 9? or 12)     <- 3, 9 and 12 are not implemented
+  Bees    Triangle 3 (9? or 12) <- 9 and 12 are not implemented
+
+  Neighbors 6 and neighbors 3 produce the same Turk ants.
 */
 
 #define DEF_TRUCHET  "False"
@@ -73,6 +76,7 @@ ModeSpecOpt ant_opts =
 #define MINGRIDSIZE 24
 #define MINSIZE 1
 #define ANGLES 360
+#define NEIGHBORKINDS 3
 
 static unsigned char patterns[COLORS - 1][PATTERNSIZE] =
 {
@@ -89,7 +93,9 @@ static unsigned char patterns[COLORS - 1][PATTERNSIZE] =
 };
 
 typedef struct {
-	unsigned char color, direction, next;
+	unsigned char color;
+	short       direction;
+	unsigned char next;
 } statestruct;
 
 typedef struct {
@@ -118,6 +124,7 @@ typedef struct {
 	GC          stippledGC;
 	Pixmap      pixmaps[COLORS - 1];
 	XPoint      hexagonList[7];
+	XPoint      triangleList[2][4];
 } antfarmstruct;
 
 static XPoint hexagonUnit[7] =
@@ -131,6 +138,24 @@ static XPoint hexagonUnit[7] =
 	{1, -1}
 };
 
+static XPoint triangleUnit[2][4] =
+{
+	{
+		{0, 0},
+		{1, -1},
+		{0, 2},
+		{-1, -1}},
+	{
+		{0, 0},
+		{-1, 1},
+		{0, -2},
+		{1, 1}}
+};
+
+static int  initVal[NEIGHBORKINDS] =
+{3, 4, 6};			/* Neighborhoods, 8 just makes a mess */
+
+
 /* Relative ant moves */
 #define FS 0			/* Step */
 #define RS 1			/* Turn right, then step */
@@ -172,6 +197,10 @@ static unsigned char tables[][3 * COLORS * STATES + 2] =
 	},
 #else
   /* First 2 numbers are the size (ncolors, nstates) */
+	{			/* LADDER BUILDER */
+		4, 1,
+		1, SR, 0, 2, SL, 0, 3, RS, 0, 0, LS, 0
+	},
 	{			/* SPIRALING PATTERN */
 		2, 2,
 		1, LS, 0, 0, FS, 1,
@@ -182,10 +211,6 @@ static unsigned char tables[][3 * COLORS * STATES + 2] =
 		1, LS, 0, 0, FS, 1,
 		0, RS, 0, 1, RS, 0
 	},
-	{			/* LADDER BUILDER */
-		4, 1,
-		1, SR, 0, 2, SL, 0, 3, RS, 0, 0, LS, 0
-	}
 #endif
 };
 
@@ -209,10 +234,29 @@ fillcell(ModeInfo * mi, int col, int row, int stipple)
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi),
 				     (stipple) ? ap->stippledGC : MI_GC(mi),
 			      ap->hexagonList, 6, Convex, CoordModePrevious);
-	} else {
+	} else if (ap->neighbors == 4 || ap->neighbors == 8) {
 		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),
 			       (stipple) ? ap->stippledGC : MI_GC(mi),
 		ap->xb + ap->xs * col, ap->yb + ap->ys * row, ap->xs, ap->ys);
+	} else {		/* TRI */
+		int         orient = (col + row) % 2;	/* O left 1 right */
+
+		ap->triangleList[orient][0].x = ap->xb + col * ap->xs;
+		ap->triangleList[orient][0].y = ap->yb + row * ap->ys;
+		if (ap->xs <= 3 || ap->ys <= 3)
+			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),
+				       (stipple) ? ap->stippledGC : MI_GC(mi), ((orient) ? -1 : 1) +
+				       ap->triangleList[orient][0].x, ap->triangleList[orient][0].y,
+							 1, 1);
+		else {
+			if (orient)
+				ap->triangleList[orient][0].x += (ap->xs / 2 - 1);
+			else
+				ap->triangleList[orient][0].x -= (ap->xs / 2 - 1);
+			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi),
+				     (stipple) ? ap->stippledGC : MI_GC(mi),
+				     ap->triangleList[orient], 3, Convex, CoordModePrevious);
+		}
 	}
 }
 
@@ -243,7 +287,7 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 					 (int) ((double) ap->xs * 1.5), (int) ((double) ap->ys * 1.5),
 				  ((555 - (side * 60)) % 360) * 64, 90 * 64);
 		}
-	} else {
+	} else if (ap->neighbors == 4) {
 		if (truchetstate) {
 			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
 				 ap->xb + ap->xs * col - ap->xs / 2,
@@ -267,6 +311,34 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 				 ap->xs, ap->ys,
 				 91 * 64, 88 * 64);
 		}
+	} else if (ap->neighbors == 3) {
+		int         orient = (col + row) % 2;	/* O left 1 right */
+		int         side, ang;
+		XPoint      tri;
+
+		tri.x = ap->xb + col * ap->xs;
+		tri.y = ap->yb + row * ap->ys;
+		if (orient) {
+			tri.x += (ap->xs / 2 - 2);
+		} else {
+			tri.x -= (ap->xs / 2 + 2);
+		}
+		for (side = 0; side < 3; side++) {
+			if (side > 0) {
+				tri.x += ap->triangleList[orient][side].x;
+				tri.y += ap->triangleList[orient][side].y;
+			}
+			if (truchetstate == side % 3) {
+				if (orient)
+					ang = (518 - side * 120) % 360;		/* Right */
+				else
+					ang = (690 - side * 120) % 360;		/* Left */
+				XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				  tri.x - ap->xs / 2, tri.y - 3 * ap->ys / 4,
+					 ap->xs, 3 * ap->ys / 2,
+					 ang * 64, 45 * 64);
+			}
+		}
 	}
 }
 
@@ -314,7 +386,6 @@ drawtruchet(ModeInfo * mi, int col, int row,
 static void
 draw_anant(ModeInfo * mi, int col, int row)
 {
-
 	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
 	fillcell(mi, col, row, 0);
 #if 0				/* Can not see eyes */
@@ -376,8 +447,42 @@ RandomSoup(mi)
 
 #endif
 
+static short
+fromTableDirection(unsigned char dir, int neighbors)
+{
+	switch (dir) {
+		case FS:
+			return 0;
+		case RS:
+			return (ANGLES / neighbors);
+		case HRS:
+			return (ANGLES / (2 * neighbors));
+		case BS:
+			return (ANGLES / 2);
+		case HLS:
+			return (ANGLES - ANGLES / (2 * neighbors));
+		case LS:
+			return (ANGLES - ANGLES / neighbors);
+		case SF:
+			return ANGLES;
+		case SR:
+			return (ANGLES + ANGLES / neighbors);
+		case SHR:
+			return (ANGLES + ANGLES / (2 * neighbors));
+		case SB:
+			return (3 * ANGLES / 2);
+		case SHL:
+			return (2 * ANGLES - ANGLES / (2 * neighbors));
+		case SL:
+			return (2 * ANGLES - ANGLES / neighbors);
+		default:
+			warning("%s: wrong direction %d\n", dir);
+	}
+	return -1;
+}
+
 static void
-GetTable(ModeInfo * mi, int i)
+getTable(ModeInfo * mi, int i)
 {
 	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 	int         j, total;
@@ -392,14 +497,14 @@ GetTable(ModeInfo * mi, int i)
 			      i, ap->ncolors, ap->nstates);
 	for (j = 0; j < total; j++) {
 		ap->machine[j].color = *patptr++;
-		ap->machine[j].direction = *patptr++;
+		ap->machine[j].direction = fromTableDirection(*patptr++, ap->neighbors);
 		ap->machine[j].next = *patptr++;
 	}
 	ap->truchet = False;
 }
 
 static void
-GetTurk(ModeInfo * mi, int i)
+getTurk(ModeInfo * mi, int i)
 {
 	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 	int         power2, j, number, total;
@@ -415,12 +520,16 @@ GetTurk(ModeInfo * mi, int i)
 	total = ap->ncolors * ap->nstates;
 	for (j = 0; j < total; j++) {
 		ap->machine[j].color = (j + 1) % total;
-		ap->machine[j].direction = (power2 & number) ? RS : LS;
+		ap->machine[j].direction = (power2 & number) ?
+			fromTableDirection(RS, ap->neighbors) :
+			fromTableDirection(LS, ap->neighbors);
 		ap->machine[j].next = 0;
 		power2 >>= 1;
 	}
-	if (truchet)
-		ap->truchet = 1;
+	if (ap->neighbors != 3 && ap->neighbors != 4 && ap->neighbors != 6)
+		ap->truchet = False;
+	else if (truchet)
+		ap->truchet = True;
 	if (MI_WIN_IS_VERBOSE(mi))
 		(void) printf("Turk's number %d, colors %d\n", number, ap->ncolors);
 }
@@ -463,12 +572,20 @@ init_ant(ModeInfo * mi)
 	ap->width = MI_WIN_WIDTH(mi);
 	ap->height = MI_WIN_HEIGHT(mi);
 
-	if (neighbors == 4)
-		ap->neighbors = 4;
-	else if (neighbors == 6)
-		ap->neighbors = 6;
+	if (neighbors == 8)
+		ap->neighbors = 8;
 	else
-		ap->neighbors = (LRAND() & 1) ? 4 : 6;
+		for (i = 0; i < NEIGHBORKINDS; i++) {
+			if (neighbors == initVal[i]) {
+				ap->neighbors = initVal[i];
+				break;
+			}
+			if (i == NEIGHBORKINDS - 1) {
+				i = NRAND(NEIGHBORKINDS);
+				ap->neighbors = initVal[i];
+				break;
+			}
+		}
 
 	if (ap->neighbors == 6) {
 		int         nccols, ncrows;
@@ -493,13 +610,13 @@ init_ant(ModeInfo * mi)
 		ncrows = MAX(ap->height / ap->ys - 1, 2);
 		ap->ncols = nccols / 2;
 		ap->nrows = 2 * (ncrows / 4);
-		ap->xb = (ap->width - ap->xs * nccols) / 2 + ap->xs;
-		ap->yb = (ap->height - ap->ys * ncrows) / 2 + ap->ys;
+		ap->xb = (ap->width - ap->xs * nccols) / 2 + ap->xs / 2;
+		ap->yb = (ap->height - ap->ys * (ncrows / 2) * 2) / 2 + ap->ys;
 		for (i = 0; i < 7; i++) {
 			ap->hexagonList[i].x = (ap->xs - 1) * hexagonUnit[i].x;
 			ap->hexagonList[i].y = (ap->ys - 1) * hexagonUnit[i].y / 2;
 		}
-	} else {
+	} else if (ap->neighbors == 4 && ap->neighbors == 8) {
 		if (size < -MINSIZE)
 			ap->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(ap->width, ap->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
@@ -516,15 +633,47 @@ init_ant(ModeInfo * mi)
 		ap->nrows = MAX(ap->height / ap->ys, 2);
 		ap->xb = (ap->width - ap->xs * ap->ncols) / 2;
 		ap->yb = (ap->height - ap->ys * ap->nrows) / 2;
+	} else {		/* TRI */
+		int         orient;
+
+		if (ap->width < 2)
+			ap->width = 2;
+		if (ap->height < 2)
+			ap->height = 2;
+		if (size < -MINSIZE)
+			ap->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE) {
+			if (!size)
+				ap->ys = MAX(MINSIZE, MIN(ap->width, ap->height) / MINGRIDSIZE);
+			else
+				ap->ys = MINSIZE;
+		} else
+			ap->ys = MIN(size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+					       MINGRIDSIZE));
+		ap->xs = 3 * ap->ys / 2;
+		ap->ncols = (MAX(ap->width / ap->xs - 1, 2) / 2) * 2;
+		ap->nrows = (MAX(ap->height / ap->ys - 1, 2) / 2) * 2;
+		ap->xb = (ap->width - ap->xs * ap->ncols) / 2 - ap->xs / 2;
+		ap->yb = (ap->height - ap->ys * ap->nrows) / 2 + ap->ys;
+		for (orient = 0; orient < 2; orient++) {
+			for (i = 0; i < 4; i++) {
+				ap->triangleList[orient][i].x =
+					(ap->xs - 2) * triangleUnit[orient][i].x;
+				ap->triangleList[orient][i].y =
+					(ap->ys - 2) * triangleUnit[orient][i].y;
+			}
+		}
 	}
 	XClearWindow(display, MI_WINDOW(mi));
 
-	if (!NRAND(COLORS))
-		GetTable(mi, (int) (NRAND(NTABLES)));
-	else
-		GetTurk(mi, (int) (NRAND(COLORS - 1)));
+	/* Exclude odd # of neighbors, stepping forward not defined */
+	if (!NRAND(COLORS) && ((ap->neighbors + 1) % 2)) {
+		getTable(mi, (int) (NRAND(NTABLES)));
+	} else
+		getTurk(mi, (int) (NRAND(COLORS - 1)));
 	if (MI_NPIXELS(mi) > 2)
-		for (i = 0; i < (int) ap->ncolors; i++)
+		for (i = 0; i < (int) ap->ncolors - 1; i++)
 			ap->colors[i] = (NRAND(MI_NPIXELS(mi)) +
 			     i * MI_NPIXELS(mi)) / ((int) (ap->ncolors - 1));
 	if (ap->ants == NULL)
@@ -574,23 +723,26 @@ draw_ant(ModeInfo * mi)
 		/* Find direction of Bees or Ants. */
 		/* Translate relative direction to actual direction */
 		old_dir = anant->direction;
-		chg_dir = (ap->neighbors == 4 && (status->direction % 6) > 1) ?
-			(unsigned char) ((int) ((status->direction % 6) + 1) / 2) :
-			status->direction % 6;
-		chg_dir = (ANGLES - chg_dir * (ANGLES / ap->neighbors)) % ANGLES;
+		chg_dir = (2 * ANGLES - status->direction) % ANGLES;
 		anant->direction = (chg_dir + old_dir) % ANGLES;
 		if (ap->truchet) {
-			int         a, b;
+			int         a = 0, b;
 
-			if (ap->neighbors == 4) {
+			if (ap->neighbors == 6) {
+				a = (old_dir / 60) % 3;
+				b = (anant->direction / 60) % 3;
+				a = (a + b + 1) % 3;
+				drawtruchet(mi, anant->col, anant->row, status->color, a);
+			} else if (ap->neighbors == 4) {
 				a = old_dir / 180;
 				b = anant->direction / 180;
 				a = ((a && !b) || (b && !a));
 				drawtruchet(mi, anant->col, anant->row, status->color, a);
-			} else {
-				a = (old_dir / 60) % 3;
-				b = (anant->direction / 60) % 3;
-				a = (a + b + 1) % 3;
+			} else if (ap->neighbors == 3) {
+				if (chg_dir == 240)
+					a = (anant->direction / 120 + 2) % 3;
+				else
+					a = (anant->direction / 120 + 1) % 3;
 				drawtruchet(mi, anant->col, anant->row, status->color, a);
 			}
 			ap->truchet_state[tape_pos] = a + 1;
@@ -599,7 +751,7 @@ draw_ant(ModeInfo * mi)
 
 		/* If edge than wrap it */
 		if (ap->neighbors == 6) {
-			switch ((status->direction < 6) ? anant->direction : old_dir) {
+			switch ((status->direction < ANGLES) ? anant->direction : old_dir) {
 				case 0:
 					anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
 					break;
@@ -629,23 +781,69 @@ draw_ant(ModeInfo * mi)
 				default:
 					warning("%s: wrong direction %d\n", anant->direction);
 			}
-		} else {
-			switch ((status->direction < 6) ? anant->direction : old_dir) {
+		} else if (ap->neighbors == 4 || ap->neighbors == 8) {
+			switch ((status->direction < ANGLES) ? anant->direction : old_dir) {
 				case 0:
 					anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
 					break;
+				case 45:
+					anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
+					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
+					break;
 				case 90:
 					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
 					break;
+				case 135:
+					anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
+					anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
+					break;
 				case 180:
 					anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
 					break;
+				case 225:
+					anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
+					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+					break;
 				case 270:
 					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
 					break;
+				case 315:
+					anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
+					anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+					break;
 				default:
 					warning("%s: wrong direction %d\n", anant->direction);
 			}
+		} else {	/* TRI */
+			if ((anant->col + anant->row) % 2) {	/* right */
+				switch ((status->direction < ANGLES) ? anant->direction : old_dir) {
+					case 0:
+						anant->col = (!anant->col) ? ap->ncols - 1 : anant->col - 1;
+						break;
+					case 120:
+						anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
+						break;
+					case 240:
+						anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+						break;
+					default:
+						warning("%s: wrong direction %d\n", anant->direction);
+				}
+			} else {	/* left */
+				switch ((status->direction < ANGLES) ? anant->direction : old_dir) {
+					case 0:
+						anant->col = (anant->col + 1 == ap->ncols) ? 0 : anant->col + 1;
+						break;
+					case 120:
+						anant->row = (anant->row + 1 == ap->nrows) ? 0 : anant->row + 1;
+						break;
+					case 240:
+						anant->row = (!anant->row) ? ap->nrows - 1 : anant->row - 1;
+						break;
+					default:
+						warning("%s: wrong direction %d\n", anant->direction);
+				}
+			}
 		}
 		draw_anant(mi, anant->col, anant->row);
 	}
