@@ -1,6 +1,6 @@
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bug.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)bug.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -371,7 +371,10 @@ makebacteria(ModeInfo * mi,
 	/* Make bacteria but if can't, don't loop forever */
 	while (i < n && j < 2 * n) {
 		hrow = NRAND(height) + starty;
-		hcol = NRAND(width - (!(hrow & 1))) + startx;
+		if (width - (!(hrow & 1)) > 0)
+			hcol = NRAND(width - (!(hrow & 1))) + startx;
+		else
+			hcol = startx;
 		colrow = hcol + hrow * bp->nhcols;
 		j++;
 		if (!bp->arr[colrow] && !bp->bacteria[colrow]) {
@@ -481,7 +484,10 @@ init_bug(ModeInfo * mi)
 	i = 0, j = 0;
 	while (i < bp->n && j < 2 * bp->n) {
 		hrow = NRAND(bp->nhrows);
-		hcol = NRAND(bp->nhcols - (!(hrow & 1)));
+		if (bp->nhcols - (!(hrow & 1)) > 0)
+			hcol = NRAND(bp->nhcols - (!(hrow & 1)));
+		else
+			hcol = 0;
 		colrow = hcol + hrow * bp->nhcols;
 		j++;
 		if (!bp->arr[colrow] && !neighbor(bp, hcol, hrow)) {
