@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)clock.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)clock.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -67,7 +67,7 @@ typedef struct {
 
 typedef struct {
 	oclock     *oclocks;
-	long        d_time;
+	time_t      d_time;
 	int         clock_count;
 	int         width, height;
 	int         nclocks;
