@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)color.c	3.9 96/05/25 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)color.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -21,7 +21,7 @@ static char sccsid[] = "@(#)color.c	3.9 96/05/25 xlockmore";
  * Create an HSB ramp.
  *
  * Revision History:
- * Changes of David Bagley <bagleyd@hertz.njit.edu>
+ * Changes of David Bagley <bagleyd@bigfoot.com>
  * 22-Jun-94: Modified for VMS
  *            <Anthony.D.Clarke@Support.Hatfield.Raytheon.bae.eurokom.ie>
  * Changes of Patrick J. Naughton
@@ -30,9 +30,6 @@ static char sccsid[] = "@(#)color.c	3.9 96/05/25 xlockmore";
  * 01-Sep-88: Written.
  */
 
-#include <sys/types.h>
-#include <math.h>
-
 static void
 hsb2rgb(double H, double S, double B,
 	unsigned char *r, unsigned char *g, unsigned char *b)
@@ -43,7 +40,7 @@ hsb2rgb(double H, double S, double B,
 
 	H -= floor(H);		/* remove anything over 1 */
 	H *= 6.0;
-	i = (int) floor(H);		/* 0..5 */
+	i = (int) floor(H);	/* 0..5 */
 	f = H - (float) i;	/* f = fractional part of H */
 	bb = 255.0 * B;
 	p = (unsigned char) (bb * (1.0 - S));
@@ -129,7 +126,7 @@ allocPixel(Display * display, Colormap cmap, char *name, char *def)
 
 void
 fixColormap(Display * display, Window window, int screen, float saturation,
-	    Bool install, Bool inroot, Bool inwindow, Bool verbose)
+	    Bool mono, Bool install, Bool inroot, Bool inwindow, Bool verbose)
 {
 	Screen     *scr = ScreenOfDisplay(display, screen);
 	Colormap    cmap, dcmap = DefaultColormapOfScreen(scr);
@@ -141,6 +138,12 @@ fixColormap(Display * display, Window window, int screen, float saturation,
 	static float lastsat[MAXSCREENS];
 	extern perscreen Scr[MAXSCREENS];
 
+	if (mono || CellsOfScreen(scr) <= 2) {
+		Scr[screen].pixels[0] = WhitePixelOfScreen(scr);
+		Scr[screen].pixels[1] = BlackPixelOfScreen(scr);
+		Scr[screen].npixels = 2;
+		return;
+	}
 	if (first) {
 		first = 0;
 		for (i = 0; i < MAXSCREENS; i++)
@@ -200,17 +203,19 @@ fixColormap(Display * display, Window window, int screen, float saturation,
 		cmap = Scr[screen].cmap;
 
 	if (install && !inroot) {
-		/* (void) XGetWindowAttributes(display, window, &wa); if(cmap !=
-		   wa.colormap) */
-		setColormap(display, window, cmap, inwindow);
+#if 0
+		(void) XGetWindowAttributes(display, window, &xgwa);
+		if (cmap != xgwa.colormap)
+#endif
+			setColormap(display, window, cmap, inwindow);
 	}
 	Scr[screen].cmap = cmap;
 }
 
 void
-setColormap(Display * display, Window window, Colormap map, Bool inwindow)
+setColormap(Display * display, Window window, Colormap cmap, Bool inwindow)
 {
-	XSetWindowColormap(display, window, map);
+	XSetWindowColormap(display, window, cmap);
 	/* Now, here we have a problem.  When we are running full-screen, the
 	   window's override_redirect attribute is on.  So, the window manager
 	   never gets the ColormapNotify event that gets generated on the
@@ -221,5 +226,62 @@ setColormap(Display * display, Window window, Colormap map, Bool inwindow)
 	   un-doing any XInstallColormap() performed by a client (which is why
 	   this does not work right under Fvwm). */
 	if (!inwindow)
-		XInstallColormap(display, map);
+		XInstallColormap(display, cmap);
 }
+
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+void
+reserveColors(ModeInfo * mi, Colormap cmap,
+	      unsigned long *blackpix, unsigned long *whitepix)
+{
+	Display    *display = MI_DISPLAY(mi);
+	unsigned long black, white;
+	XColor      whitecolor, blackcolor;
+
+	black = MI_WIN_BLACK_PIXEL(mi);
+	white = MI_WIN_WHITE_PIXEL(mi);
+
+	blackcolor.flags = DoRed | DoGreen | DoBlue;
+	blackcolor.pixel = black;
+	blackcolor.red = 0;
+	blackcolor.green = 0;
+	blackcolor.blue = 0;
+	whitecolor.flags = DoRed | DoGreen | DoBlue;
+	whitecolor.pixel = white;
+	whitecolor.red = 0xFFFF;
+	whitecolor.green = 0xFFFF;
+	whitecolor.blue = 0xFFFF;
+
+	/* If they fail what should I do? */
+	(void) XAllocColor(display, cmap, &blackcolor);
+	(void) XAllocColor(display, cmap, &whitecolor);
+#if 0
+	/* This stuff is supposed to reserve the fg and bg
+	   colors but does not work. */
+	{
+		unsigned long fg, bg;
+		XColor      fgcolor, bgcolor;
+		int         screen = MI_SCREEN(mi);
+
+		fg = MI_FG_COLOR(mi);
+		bg = MI_BG_COLOR(mi);
+
+		fgcolor.pixel = fg;
+		bgcolor.pixel = bg;
+		XQueryColor(display, MI_COLORMAP(mi), &fgcolor);
+		XQueryColor(display, MI_COLORMAP(mi), &bgcolor);
+
+		if ((fgcolor.pixel != blackcolor.pixel) &&
+		    (fgcolor.pixel != whitecolor.pixel))
+			(void) XAllocColor(display, cmap, &fgcolor);
+		if ((bgcolor.pixel != blackcolor.pixel) &&
+		    (bgcolor.pixel != whitecolor.pixel) &&
+		    (bgcolor.pixel != fgcolor.pixel))
+			(void) XAllocColor(display, cmap, &bgcolor);
+	}
+#endif
+	*blackpix = blackcolor.pixel;
+	*whitepix = whitecolor.pixel;
+}
+
+#endif
