@@ -10,3 +10,6 @@ banner -w 25 SYSV | ../config/xlockflag.pl > flag-sysv.h
 banner -w 25 ULTRIX | ../config/xlockflag.pl > flag-ultrix.h
 banner -w 25 UNIX | ../config/xlockflag.pl > flag-unix.h
 banner -w 25 VMS | ../config/xlockflag.pl > flag-vms.h
+banner -w 25 DEBIAN | ../config/xlockflag.pl > flag-debian.h
+banner -w 25 SLACKWARE | ../config/xlockflag.pl > flag-slackware.h
+banner -w 25 SuSE | ../config/xlockflag.pl > flag-suse.h
