@@ -111,7 +111,7 @@ typedef struct {		/* info about a pair of eyes */
 	int         xoff, yoff;
 	int         rectw, recth;
 	int         painted;
-	time_t      time_to_die;
+	unsigned long time_to_die;
 	unsigned long eyelid_pixel, eyeball_pixel, pupil_pixel;
 	Pixmap      pixmap;
 	XRectangle  bbox;
@@ -579,8 +579,8 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 		cycs = 1;
 	if (cycs > MAX_CYCLES)
 		cycs = MAX_CYCLES;
-	e->time_to_die = LIFE_MIN + NRAND(LIFE_RANGE);
-	e->time_to_die *= cycs;	/* multiply life by cycles */
+	e->time_to_die = (unsigned long) LIFE_MIN + NRAND(LIFE_RANGE);
+	e->time_to_die *= (unsigned long) cycs;		/* multiply life by cycles */
 	e->time_to_die += seconds();	/* add the time now */
 
 	e->pupil_pixel = black_pixel;	/* pupil is always black */
@@ -674,7 +674,7 @@ paint_eyes(ModeInfo * mi, Eyes * e, Fly * f, Eyes * eyes, int num_eyes)
 	Window      window = MI_WINDOW(mi);
 	GC          gc = ep->eyeGC;
 	Bool        iconic = MI_WIN_IS_ICONIC(mi);
-	long        now = seconds();
+	unsigned long now = seconds();
 	int         focusx = (f->x + (f->width / 2)) - e->x;
 	int         focusy = (f->y + (f->height / 2)) - e->y;
 	Pixmap      pix = e->pixmap;
