@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)flame.c	3.9 96/05/25 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)flame.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -22,14 +22,13 @@ static char sccsid[] = "@(#)flame.c	3.9 96/05/25 xlockmore";
  */
 
 #include "xlock.h"
-#include <math.h>
 
-#define MAXBATCH  10
 #define MAXLEV    4
 #define MAXKINDS  9
+#define MAXBATCH  12
 
 ModeSpecOpt flame_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	double      f[2][3][MAXLEV];	/* three non-homogeneous transforms */
@@ -43,11 +42,10 @@ typedef struct {
 	int         total_points;
 	int         pixcol;
 	int         cycles;
-	Window      win;
 	XPoint      pts[MAXBATCH];
 } flamestruct;
 
-static flamestruct flames[MAXSCREENS];
+static flamestruct *flames = NULL;
 
 static short
 halfrandom(int mv)
@@ -65,58 +63,34 @@ halfrandom(int mv)
 	return r % mv;
 }
 
-void
-init_flame(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	flamestruct *fs = &flames[screen];
-
-	fs->width = MI_WIN_WIDTH(mi);
-	fs->height = MI_WIN_HEIGHT(mi);
-
-	fs->max_levels = MI_BATCHCOUNT(mi);
-	fs->win = win;
-	fs->cycles = MI_CYCLES(mi);
-
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, fs->width, fs->height);
-
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		fs->pixcol = halfrandom(Scr[screen].npixels);
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[fs->pixcol]);
-	} else
-		XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
-
-	fs->variation = LRAND() % MAXKINDS;
-}
-
 static      Bool
-recurse(flamestruct * fs, register double x, register double y, register int l)
+recurse(ModeInfo * mi, flamestruct * fp,
+	register double x, register double y, register int l)
 {
 	int         i;
 	double      nx, ny;
 
-	if (l == fs->max_levels) {
-		fs->total_points++;
-		if (fs->total_points > fs->cycles)	/* how long each fractal runs */
+	if (l == fp->max_levels) {
+		fp->total_points++;
+		if (fp->total_points > fp->cycles)	/* how long each fractal runs */
 			return False;
 
 		if (x > -1.0 && x < 1.0 && y > -1.0 && y < 1.0) {
-			fs->pts[fs->num_points].x = (int) ((fs->width / 2) * (x + 1.0));
-			fs->pts[fs->num_points].y = (int) ((fs->height / 2) * (y + 1.0));
-			fs->num_points++;
-			if (fs->num_points > MAXBATCH) {	/* point buffer size */
-				XDrawPoints(dsp, fs->win, Scr[screen].gc, fs->pts,
-					    fs->num_points, CoordModeOrigin);
-				fs->num_points = 0;
+			fp->pts[fp->num_points].x = (int) ((fp->width / 2) * (x + 1.0));
+			fp->pts[fp->num_points].y = (int) ((fp->height / 2) * (y + 1.0));
+			fp->num_points++;
+			if (fp->num_points >= MAXBATCH) {	/* point buffer size */
+				XDrawPoints(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi), fp->pts,
+					    fp->num_points, CoordModeOrigin);
+				fp->num_points = 0;
 			}
 		}
 	} else {
-		for (i = 0; i < fs->snum; i++) {
-			nx = fs->f[0][0][i] * x + fs->f[0][1][i] * y + fs->f[0][2][i];
-			ny = fs->f[1][0][i] * x + fs->f[1][1][i] * y + fs->f[1][2][i];
-			if (i < fs->anum) {
-				switch (fs->variation) {
+		for (i = 0; i < fp->snum; i++) {
+			nx = fp->f[0][0][i] * x + fp->f[0][1][i] * y + fp->f[0][2][i];
+			ny = fp->f[1][0][i] * x + fp->f[1][1][i] * y + fp->f[1][2][i];
+			if (i < fp->anum) {
+				switch (fp->variation) {
 					case 0:	/* sinusoidal */
 						nx = sin(nx);
 						ny = sin(ny);
@@ -142,8 +116,11 @@ recurse(flamestruct * fs, register double x, register double y, register int l)
 							double      c2 = cos(r);
 							double      t = nx;
 
+							if (nx > 1e4 || nx < -1e4 || ny > 1e4 || ny < -1e4)
+								ny = 1e4;
+							else
+								ny = c2 * t + c1 * ny;
 							nx = c1 * nx - c2 * ny;
-							ny = c2 * t + c1 * ny;
 						}
 						break;
 					case 4:	/* horseshoe */
@@ -161,7 +138,10 @@ recurse(flamestruct * fs, register double x, register double y, register int l)
 						{
 							double      t = atan2(nx, ny) / M_PI;
 
-							ny = sqrt(nx * nx + ny * ny) - 1.0;
+							if (nx > 1e4 || nx < -1e4 || ny > 1e4 || ny < -1e4)
+								ny = 1e4;
+							else
+								ny = sqrt(nx * nx + ny * ny) - 1.0;
 							nx = t;
 						}
 						break;
@@ -216,52 +196,96 @@ recurse(flamestruct * fs, register double x, register double y, register int l)
 						ny = sin(ny);
 				}
 			}
-			if (!recurse(fs, nx, ny, l + 1))
+			if (!recurse(mi, fp, nx, ny, l + 1))
 				return False;
 		}
 	}
 	return True;
 }
 
+void
+init_flame(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	flamestruct *fp;
+
+	if (flames == NULL) {
+		if ((flames = (flamestruct *) calloc(MI_NUM_SCREENS(mi),
+					      sizeof (flamestruct))) == NULL)
+			return;
+	}
+	fp = &flames[MI_SCREEN(mi)];
+
+	fp->width = MI_WIN_WIDTH(mi);
+	fp->height = MI_WIN_HEIGHT(mi);
+
+	fp->max_levels = MI_BATCHCOUNT(mi);
+	fp->cycles = MI_CYCLES(mi);
+
+	XClearWindow(display, MI_WINDOW(mi));
+
+	if (MI_NPIXELS(mi) > 2) {
+		fp->pixcol = halfrandom(MI_NPIXELS(mi));
+		XSetForeground(display, gc, MI_PIXEL(mi, fp->pixcol));
+	} else
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+
+	fp->variation = NRAND(MAXKINDS);
+}
 
 void
 draw_flame(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	flamestruct *fs = &flames[screen];
+	Display    *display = MI_DISPLAY(mi);
+	flamestruct *fp = &flames[MI_SCREEN(mi)];
 	int         i, j, k;
 	static      alt = 0;
 
-	if (!(fs->cur_level++ % fs->max_levels)) {
-		XClearWindow(dsp, fs->win);
+	if (!(fp->cur_level++ % fp->max_levels)) {
+		XClearWindow(display, MI_WINDOW(mi));
 		alt = !alt;
 	} else {
-		if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-			XSetForeground(dsp, Scr[screen].gc,
-				       Scr[screen].pixels[fs->pixcol]);
-			if (--fs->pixcol < 0)
-				fs->pixcol = Scr[screen].npixels - 1;
+		if (MI_NPIXELS(mi) > 2) {
+			XSetForeground(display, MI_GC(mi), MI_PIXEL(mi, fp->pixcol));
+			if (--fp->pixcol < 0)
+				fp->pixcol = MI_NPIXELS(mi) - 1;
 		}
 	}
 
 	/* number of functions */
-	fs->snum = 2 + (fs->cur_level % (MAXLEV - 1));
+	fp->snum = 2 + (fp->cur_level % (MAXLEV - 1));
 
 	/* how many of them are of alternate form */
 	if (alt)
-		fs->anum = 0;
+		fp->anum = 0;
 	else
-		fs->anum = halfrandom(fs->snum) + 2;
+		fp->anum = halfrandom(fp->snum) + 2;
 
 	/* 6 coefs per function */
-	for (k = 0; k < fs->snum; k++) {
+	for (k = 0; k < fp->snum; k++) {
 		for (i = 0; i < 2; i++)
 			for (j = 0; j < 3; j++)
-				fs->f[i][j][k] = ((double) (LRAND() & 1023) / 512.0 - 1.0);
+				fp->f[i][j][k] = ((double) (LRAND() & 1023) / 512.0 - 1.0);
 	}
-	fs->num_points = 0;
-	fs->total_points = 0;
-	(void) recurse(fs, 0.0, 0.0, 0);
-	XDrawPoints(dsp, win, Scr[screen].gc,
-		    fs->pts, fs->num_points, CoordModeOrigin);
+	fp->num_points = 0;
+	fp->total_points = 0;
+	(void) recurse(mi, fp, 0.0, 0.0, 0);
+	XDrawPoints(display, MI_WINDOW(mi), MI_GC(mi),
+		    fp->pts, fp->num_points, CoordModeOrigin);
+}
+
+void
+release_flame(ModeInfo * mi)
+{
+	if (flames != NULL) {
+		(void) free((void *) flames);
+		flames = NULL;
+	}
+}
+
+void
+refresh_flame(ModeInfo * mi)
+{
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 }
