@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)forest.c	3.9 96/05/25 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)forest.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -21,7 +21,8 @@ static char sccsid[] = "@(#)forest.c	3.9 96/05/25 xlockmore";
  */
 
 #include "xlock.h"
-#include <math.h>
+
+#define MINTREES   1
 
 #define MINHEIGHT  20		/* Tree height range */
 #define MAXHEIGHT  40
@@ -37,26 +38,26 @@ static char sccsid[] = "@(#)forest.c	3.9 96/05/25 xlockmore";
 #define COLORSPEED  2		/* Color increment */
 
 /* degree to radian */
-#define DEGTORAD(x) (((float)(x)) * 0.017453293)
+#define DEGTORAD(x) (((float)(x)) * M_PI / 180.0)
 
-#define PIONTWO 1.5707963
-
-#define MAX_RAND(max) (LRAND() % (max))
-#define RANGE_RAND(min,max) ((min) + LRAND() % ((max) - (min)))
+#define RANGE_RAND(min,max) ((min) + NRAND((max) - (min)))
 
 ModeSpecOpt forest_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	int         width;
 	int         height;
 	int         time;	/* up time */
+	int         ntrees;
 } foreststruct;
 
-static foreststruct forests[MAXSCREENS];
+static foreststruct *forests = NULL;
 
 static void
-draw_tree(ModeInfo * mi, short int x, short int y, short int len, float a, float as, short int c, short int level)
+draw_tree(ModeInfo * mi,
+	  short int x, short int y, short int len,
+	  float a, float as, short int c, short int level)
 				/* Father's end */
 				/* Length */
 				/* color */
@@ -64,36 +65,38 @@ draw_tree(ModeInfo * mi, short int x, short int y, short int len, float a, float
 				/* Father's angle */
 				/* Father's angle step */
 {
-	Window      win = MI_WINDOW(mi);
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
 	short       x_1, y_1, x_2, y_2;
 	float       a1, a2;
 
 	/* left */
 
-	a1 = a + as + DEGTORAD(MAX_RAND(2 * RANDANGLE) - RANDANGLE);
+	a1 = a + as + DEGTORAD(NRAND(2 * RANDANGLE) - RANDANGLE);
 
 	x_1 = x + (short) (COSF(a1) * ((float) len));
 	y_1 = y + (short) (SINF(a1) * ((float) len));
 
 	/* right */
 
-	a2 = a - as + DEGTORAD(MAX_RAND(2 * RANDANGLE) - RANDANGLE);
+	a2 = a - as + DEGTORAD(NRAND(2 * RANDANGLE) - RANDANGLE);
 
 	x_2 = x + (short) (COSF(a2) * ((float) len));
 	y_2 = y + (short) (SINF(a2) * ((float) len));
 
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[c]);
-		c = (c + COLORSPEED) % Scr[screen].npixels;
+	if (MI_NPIXELS(mi) > 2) {
+		XSetForeground(display, gc, MI_PIXEL(mi, c));
+		c = (c + COLORSPEED) % MI_NPIXELS(mi);
 	} else
-		XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
 
-	XDrawLine(dsp, win, Scr[screen].gc, x, y, x_1, y_1);
-	XDrawLine(dsp, win, Scr[screen].gc, x, y, x_2, y_2);
+	XDrawLine(display, window, gc, x, y, x_1, y_1);
+	XDrawLine(display, window, gc, x, y, x_2, y_2);
 
 	if (level < 2) {
-		XDrawLine(dsp, win, Scr[screen].gc, x + 1, y, x_1 + 1, y_1);
-		XDrawLine(dsp, win, Scr[screen].gc, x + 1, y, x_2 + 1, y_2);
+		XDrawLine(display, window, gc, x + 1, y, x_1 + 1, y_1);
+		XDrawLine(display, window, gc, x + 1, y, x_2 + 1, y_2);
 	}
 	len = (len * REDUCE * 10) / 1000;
 
@@ -106,48 +109,79 @@ draw_tree(ModeInfo * mi, short int x, short int y, short int len, float a, float
 void
 init_forest(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	foreststruct *fp = &forests[screen];
+	foreststruct *fp;
+
+	if (forests == NULL) {
+		if ((forests = (foreststruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (foreststruct))) == NULL)
+			return;
+	}
+	fp = &forests[MI_SCREEN(mi)];
 
 	fp->width = MI_WIN_WIDTH(mi);
 	fp->height = MI_WIN_HEIGHT(mi);
 	fp->time = 0;
 
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, fp->width, fp->height);
+	fp->ntrees = MI_BATCHCOUNT(mi);
+	if (fp->ntrees < -MINTREES)
+		fp->ntrees = NRAND(-fp->ntrees - MINTREES + 1) + MINTREES;
+	else if (fp->ntrees < MINTREES)
+		fp->ntrees = MINTREES;
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 }
 
 void
 draw_forest(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	foreststruct *fp = &forests[MI_SCREEN(mi)];
 	short       x, y, x_2, y_2, len, c = 0;
 	float       a, as;
-	foreststruct *fp = &forests[screen];
 
-	if (fp->time < MI_BATCHCOUNT(mi)) {
+	if (fp->time < fp->ntrees) {
 
 		x = RANGE_RAND(0, fp->width);
 		y = RANGE_RAND(0, fp->height + MAXHEIGHT);
-		a = -PIONTWO + DEGTORAD(MAX_RAND(2 * RANDANGLE) - RANDANGLE);
+		a = -M_PI / 2.0 + DEGTORAD(NRAND(2 * RANDANGLE) - RANDANGLE);
 		as = DEGTORAD(RANGE_RAND(MINANGLE, MAXANGLE));
 		len = ((RANGE_RAND(MINHEIGHT, MAXHEIGHT) * (fp->width / 20)) / 50) + 2;
 
-		if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-			c = MAX_RAND(Scr[screen].npixels);
-			XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[c]);
-			c = (c + COLORSPEED) % Scr[screen].npixels;
+		if (MI_NPIXELS(mi) > 2) {
+			c = NRAND(MI_NPIXELS(mi));
+			XSetForeground(display, gc, MI_PIXEL(mi, c));
+			c = (c + COLORSPEED) % MI_NPIXELS(mi);
 		} else
-			XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+			XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
 
 		x_2 = x + (short) (COSF(a) * ((float) len));
 		y_2 = y + (short) (SINF(a) * ((float) len));
 
-		XDrawLine(dsp, win, Scr[screen].gc, x, y, x_2, y_2);
-		XDrawLine(dsp, win, Scr[screen].gc, x + 1, y, x_2 + 1, y_2);
+		XDrawLine(display, MI_WINDOW(mi), gc, x, y, x_2, y_2);
+		XDrawLine(display, MI_WINDOW(mi), gc, x + 1, y, x_2 + 1, y_2);
 
 		draw_tree(mi, x_2, y_2, (len * REDUCE) / 100, a, as, c, 1);
 	}
 	if (++fp->time > MI_CYCLES(mi))
 		init_forest(mi);
 }
+
+void
+release_forest(ModeInfo * mi)
+{
+	if (forests != NULL) {
+		(void) free((void *) forests);
+		forests = NULL;
+	}
+}
+
+void
+refresh_forest(ModeInfo * mi)
+{
+	foreststruct *fp = &forests[MI_SCREEN(mi)];
+
+	if (fp->time < fp->ntrees)
+		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	else
+		init_forest(mi);
+}
