@@ -1,5 +1,6 @@
+
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)galaxy.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)galaxy.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -19,6 +20,10 @@ static const char sccsid[] = "@(#)galaxy.c	4.00 97/01/01 xlockmore";
  * 30-Sep-94: Initial port by Hubert Feyer
  * 10-Oct-94: Add colors by Hubert Feyer
  * 23-Oct-94: Modified by David Bagley <bagleyd@bigfoot.com>
+ * 7-Apr-97: Modified by Dave Mitchell <davem@magnet.com>
+ *		random star sizes
+ *		colors change depending on velocity
+ * 18-Apr-97: Memory leak fixed by Tom Schmidt <tschmidt@micron.com>
  */
 
 #include "xlock.h"
@@ -43,6 +48,7 @@ static const char sccsid[] = "@(#)galaxy.c	4.00 97/01/01 xlockmore";
 #define GALAXYRANGESIZE  0.1
 #define GALAXYMINSIZE  0.1
 #define QCONS    0.001
+#define EPSILON 0.0000001
 
 #define COLORBASE  8
   /* Colors for stars start here */
@@ -51,13 +57,34 @@ static const char sccsid[] = "@(#)galaxy.c	4.00 97/01/01 xlockmore";
 #define drawStar(x,y,size) if(size<=1) XDrawPoint(display,window,gc,x,y);\
   else XFillArc(display,window,gc,x,y,size,size,0,23040)
 
+static Bool tracks;
+
+#define DEF_TRACKS "False"
+
+static XrmOptionDescRec opts[] =
+{
+	{"-tracks", ".galaxy.tracks", XrmoptionNoArg, (caddr_t) "on"},
+	{"+tracks", ".galaxy.tracks", XrmoptionNoArg, (caddr_t) "off"}
+};
+
+static argtype vars[] =
+{
+	{(caddr_t *) & tracks, "tracks", "Tracks", DEF_TRACKS, t_Bool}
+};
+
+static OptionStruct desc[] =
+{
+	{"-/+tracks", "turn on/off star tracks"}
+};
+
 ModeSpecOpt galaxy_opts =
-{0, NULL, 0, NULL, NULL};
+{2, opts, 1, vars, desc};
 
 typedef struct {
 	double      pos[3], vel[3];
 	int         px, py;
 	int         color;
+	int         size;
 } Star;
 
 typedef struct {
@@ -79,7 +106,7 @@ typedef struct {
 	double      scale;	/* Scale */
 	int         midx;	/* Middle of screen, x */
 	int         midy;	/* Middle of screen, y */
-	double      size, star_size;	/* */
+	double      size;	/* */
 	double      diff[3];	/* */
 	Galaxy     *galaxies;	/* the Whole Universe */
 	int         ngalaxies;	/* # galaxies */
@@ -111,17 +138,19 @@ static void
 startover(ModeInfo * mi)
 {
 	unistruct  *gp = &universes[MI_SCREEN(mi)];
+	int         size = MI_SIZE(mi);
 	int         i, j;	/* more tmp */
 	double      w1, w2;	/* more tmp */
 	double      d, v, w, h;	/* yet more tmp */
 
 	gp->step = 0;
 
-	gp->ngalaxies = MI_BATCHCOUNT(mi);
-	if (gp->ngalaxies < -MINGALAXIES) {
+  if (MI_BATCHCOUNT(mi) < -MINGALAXIES)
 		free_galaxies(gp);
+	gp->ngalaxies = MI_BATCHCOUNT(mi);
+	if (gp->ngalaxies < -MINGALAXIES)
 		gp->ngalaxies = NRAND(-gp->ngalaxies - MINGALAXIES + 1) + MINGALAXIES;
-	} else if (gp->ngalaxies < MINGALAXIES)
+	else if (gp->ngalaxies < MINGALAXIES)
 		gp->ngalaxies = MINGALAXIES;
 	if (gp->galaxies == NULL)
 		gp->galaxies = (Galaxy *) calloc(gp->ngalaxies, sizeof (Galaxy));
@@ -140,8 +169,6 @@ startover(ModeInfo * mi)
 			gt->stars = NULL;
 		}
 		gt->nstars = (NRAND(MAX_STARS / 2)) + MAX_STARS / 2;
-/* PURIFY 3.2 on Solaris2 and PURIFY 4.0 on SunOS4 reports a memory leak on
-   the next line */
 		gt->stars = (Star *) malloc(gt->nstars * sizeof (Star));
 		w1 = 2.0 * M_PI * FLOATRAND;
 		w2 = 2.0 * M_PI * FLOATRAND;
@@ -200,10 +227,15 @@ startover(ModeInfo * mi)
 			st->vel[2] = -gp->mat[0][2] * v * sinw + gp->mat[1][2] * v * cosw +
 				gt->vel[2];
 
-			st->color = COLORSTEP * gt->galcol + j % COLORSTEP;
-
 			st->px = 0;
 			st->py = 0;
+
+			if (size < -MINSIZE)
+				st->size = NRAND(-size - MINSIZE + 1) + MINSIZE;
+			else if (size < MINSIZE)
+				st->size = MINSIZE;
+			else
+				st->size = size;
 		}
 	}
 
@@ -224,7 +256,6 @@ void
 init_galaxy(ModeInfo * mi)
 {
 	unistruct  *gp;
-	int         size = MI_SIZE(mi);
 
 	if (universes == NULL) {
 		if ((universes = (unistruct *) calloc(MI_NUM_SCREENS(mi),
@@ -241,15 +272,9 @@ init_galaxy(ModeInfo * mi)
 	gp->clip.right = MI_WIN_WIDTH(mi);
 	gp->clip.bottom = MI_WIN_HEIGHT(mi);
 
-	gp->scale = (double) (gp->clip.right) / 4.0;
+	gp->scale = (double) (gp->clip.right + gp->clip.bottom) / 8.0;
 	gp->midx = gp->clip.right / 2;
 	gp->midy = gp->clip.bottom / 2;
-	if (size < -MINSIZE)
-		gp->star_size = NRAND(-size - MINSIZE + 1) + MINSIZE;
-	else if (size < MINSIZE)
-		gp->star_size = MINSIZE;
-	else
-		gp->star_size = size;
 	startover(mi);
 }
 
@@ -277,6 +302,8 @@ draw_galaxy(ModeInfo * mi)
 				gp->diff[2] = gtk->pos[2] - st->pos[2];
 				d = gp->diff[0] * gp->diff[0] + gp->diff[1] * gp->diff[1] +
 					gp->diff[2] * gp->diff[2];
+				if (d < EPSILON)
+					d = EPSILON;
 				d = gt->mass / (d * sqrt(d)) * gp->f_deltat * QCONS;
 				gp->diff[0] *= d;
 				gp->diff[1] *= d;
@@ -285,16 +312,20 @@ draw_galaxy(ModeInfo * mi)
 				st->vel[1] += gp->diff[1];
 				st->vel[2] += gp->diff[2];
 			}
+
+			st->color = COLORSTEP * gt->galcol + ((int) ((st->vel[0] * st->vel[0] +
+								      st->vel[1] * st->vel[1] + st->vel[2] * st->vel[2]) / 3.0)) % COLORSTEP;
+
 			st->pos[0] += st->vel[0] * gp->f_deltat;
 			st->pos[1] += st->vel[1] * gp->f_deltat;
 			st->pos[2] += st->vel[2] * gp->f_deltat;
 
 			if (st->px >= gp->clip.left &&
-			    st->px <= gp->clip.right - gp->star_size &&
+			    st->px <= gp->clip.right - st->size &&
 			    st->py >= gp->clip.top &&
-			    st->py <= gp->clip.bottom - gp->star_size) {
+			    st->py <= gp->clip.bottom - st->size) {
 				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
-				drawStar(st->px, st->py, gp->star_size);
+				drawStar(st->px, st->py, st->size);
 			}
 			st->px = (int) (st->pos[0] * gp->scale) + gp->midx;
 			st->py = (int) (st->pos[1] * gp->scale) + gp->midy;
@@ -321,14 +352,17 @@ draw_galaxy(ModeInfo * mi)
 
 
 			if (st->px >= gp->clip.left &&
-			    st->px <= gp->clip.right - gp->star_size &&
+			    st->px <= gp->clip.right - st->size &&
 			    st->py >= gp->clip.top &&
-			    st->py <= gp->clip.bottom - gp->star_size) {
+			    st->py <= gp->clip.bottom - st->size) {
 				if (MI_NPIXELS(mi) > 2)
 					XSetForeground(display, gc, MI_PIXEL(mi, st->color));
 				else
 					XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
-				drawStar(st->px, st->py, gp->star_size);
+				if (tracks)
+					drawStar(st->px + 1, st->py, st->size);
+				else
+					drawStar(st->px, st->py, st->size);
 			}
 		}
 
@@ -340,6 +374,8 @@ draw_galaxy(ModeInfo * mi)
 			gp->diff[2] = gtk->pos[2] - gt->pos[2];
 			d = gp->diff[0] * gp->diff[0] + gp->diff[1] * gp->diff[1] +
 				gp->diff[2] * gp->diff[2];
+			if (d < EPSILON)
+				d = EPSILON;
 			d = gt->mass * gt->mass / (d * sqrt(d)) * gp->f_deltat * QCONS;
 			gp->diff[0] *= d;
 			gp->diff[1] *= d;
