@@ -85,10 +85,8 @@ ModeSpecOpt grav_opts =
 {4, opts, 2, vars, desc};
 
 typedef struct {
-	double
-	            P[DIMENSIONS], V[DIMENSIONS], A[DIMENSIONS];
-	int
-	            xi, yi, ri;
+	double        P[DIMENSIONS], V[DIMENSIONS], A[DIMENSIONS];
+	int           xi, yi, ri;
 	unsigned long colors;
 } planetstruct;
 
@@ -119,9 +117,9 @@ init_planet(ModeInfo * mi, planetstruct * planet)
 	POS(Z) = FLOATRAND(-ZR, ZR);
 
 	if (POS(Z) > -ALMOST) {
-		planet->xi = (unsigned int)
+		planet->xi = (int)
 			((double) gp->width * (HALF + POS(X) / (POS(Z) + DIST)));
-		planet->yi = (unsigned int)
+		planet->yi = (int)
 			((double) gp->height * (HALF + POS(Y) / (POS(Z) + DIST)));
 	} else
 		planet->xi = planet->yi = -1;
@@ -172,9 +170,9 @@ draw_planet(ModeInfo * mi, planetstruct * planet)
 	gp->y = planet->yi;
 
 	if (POS(Z) > -ALMOST) {
-		planet->xi = (unsigned int)
+		planet->xi = (int)
 			((double) gp->width * (HALF + POS(X) / (POS(Z) + DIST)));
-		planet->yi = (unsigned int)
+		planet->yi = (int)
 			((double) gp->height * (HALF + POS(Y) / (POS(Z) + DIST)));
 	} else
 		planet->xi = planet->yi = -1;
