@@ -1,12 +1,13 @@
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)helix.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)helix.c	4.02 97/04/01 xlockmore";
 
 #endif
 
 /*-
  * String art
  *
+ * 06-Apr-97: new ellipse code from Dan Stromberg <strombrg@nis.acs.uci.edu>
  * 11-Aug-95: found some typos, looks more interesting now
  * 08-Aug-95: speed up thanks to (Heath A. Kehoe hakehoe@icaen.uiowa.edu)
  * 17-Jun-95: removed sleep statements
@@ -29,9 +30,26 @@ static const char sccsid[] = "@(#)helix.c	4.00 97/01/01 xlockmore";
 #include "xlock.h"
 
 #define ANGLES 360
+#define DEF_ELLIPSE "False"
+
+static Bool ellipse;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-ellipse", ".helix.ellipse", XrmoptionNoArg, (caddr_t) "on"},
+	{"+ellipse", ".helix.ellipse", XrmoptionNoArg, (caddr_t) "off"}
+};
+static argtype vars[] =
+{
+	{(caddr_t *) & ellipse, "ellipse", "Ellipse", DEF_ELLIPSE, t_Bool}
+};
+static OptionStruct desc[] =
+{
+	{"-/+ellipse", "turn on/off ellipse format"}
+};
 
 ModeSpecOpt helix_opts =
-{0, NULL, 0, NULL, NULL};
+{2, opts, 1, vars, desc};
 
 static double cos_array[ANGLES], sin_array[ANGLES];
 typedef struct {
@@ -43,6 +61,12 @@ typedef struct {
 	int         radius1, radius2, d_angle, factor1, factor2, factor3,
 	            factor4;
 	int         redraw;
+
+	Bool        ellipse;
+	int         d_angle_offset, dir;
+	int         offset;
+	int         density;
+	int         count;
 } helixstruct;
 
 static helixstruct *helixes = NULL;
@@ -81,7 +105,7 @@ helix(ModeInfo * mi, int radius1, int radius2, int d_angle,
 	for (i = 0; i < limit; i++) {
 		int         tmp;
 
-#define pmod(x,y) (tmp=(x%y),(tmp>=0?tmp:tmp+y))
+#define pmod(x,y) (tmp=((x)%(y)),(tmp>=0?tmp:tmp+y))
 		x_1 = hp->xmid + (int) (((double) radius1) *
 				 sin_array[pmod((angle * factor1), ANGLES)]);
 		y_1 = hp->ymid + (int) (((double) radius2) *
@@ -106,6 +130,41 @@ helix(ModeInfo * mi, int radius1, int radius2, int d_angle,
 	}
 }
 
+static void
+trig(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	helixstruct *hp = &helixes[MI_SCREEN(mi)];
+	int         x1, y1, x2, y2;
+	int         tmp, angle;
+
+
+#define pmod(x,y) (tmp=((x)%(y)),(tmp>=0?tmp:tmp+y))
+
+	while (ABS(hp->d_angle) <= ANGLES) {
+		angle = hp->d_angle + hp->d_angle_offset;
+		x1 = (int) (sin_array[pmod(angle * hp->factor1, ANGLES)] * hp->xmid) +
+			hp->xmid;
+		y1 = (int) (cos_array[pmod(angle * hp->factor1, ANGLES)] * hp->ymid) +
+			hp->ymid;
+		x2 = (int) (sin_array[pmod(angle * hp->factor2 + hp->offset, ANGLES)] *
+			    hp->xmid) + hp->xmid;
+		y2 = (int) (cos_array[pmod(angle * hp->factor2 + hp->offset, ANGLES)] *
+			    hp->ymid) + hp->ymid;
+		XDrawLine(display, MI_WINDOW(mi), gc, x1, y1, x2, y2);
+		if (MI_NPIXELS(mi) > 2) {
+			XSetForeground(display, gc, MI_PIXEL(mi, hp->color));
+			if (++hp->color >= MI_NPIXELS(mi))
+				hp->color = 0;
+		}
+		tmp = (int) ANGLES / (2 * hp->density * hp->factor1 * hp->factor2);
+		if (tmp == 0)	/* Do not want it getting stuck... */
+			tmp = 1;	/* Would not need if floating point */
+		hp->d_angle += hp->dir * tmp;
+	}
+}
+
 static void
 random_helix(ModeInfo * mi)
 {
@@ -157,6 +216,31 @@ random_helix(ModeInfo * mi)
 	      hp->factor1, hp->factor2, hp->factor3, hp->factor4);
 }
 
+static void
+random_trig(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	helixstruct *hp = &helixes[MI_SCREEN(mi)];
+
+	hp->d_angle = 0;
+	hp->factor1 = NRAND(8) + 1;
+	do
+		hp->factor2 = NRAND(8) + 1;
+	while (hp->factor1 == hp->factor2);
+	hp->dir = (LRAND() & 1) ? 1 : -1;
+	hp->d_angle_offset = NRAND(ANGLES);
+	hp->offset = (NRAND(ANGLES / 4 - 1) + 1) / 4;
+	hp->density = 1 << (NRAND(4) + 4);	/* Higher density, higher ANGLES */
+	if (MI_NPIXELS(mi) > 2) {
+		XSetForeground(display, gc, MI_PIXEL(mi, hp->color));
+		if (++hp->color >= MI_NPIXELS(mi))
+			hp->color = 0;
+	} else
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	trig(mi);
+}
+
 void
 init_helix(ModeInfo * mi)
 {
@@ -171,13 +255,6 @@ init_helix(ModeInfo * mi)
 	}
 	hp = &helixes[MI_SCREEN(mi)];
 
-	hp->width = MI_WIN_WIDTH(mi);
-	hp->height = MI_WIN_HEIGHT(mi);
-	hp->xmid = hp->width / 2;
-	hp->ymid = hp->height / 2;
-	hp->redraw = 0;
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
-
 	if (first) {
 		first = 0;
 		for (i = 0; i < ANGLES; i++) {
@@ -185,10 +262,25 @@ init_helix(ModeInfo * mi)
 			sin_array[i] = sin((((double) i) / (double) (ANGLES / 2)) * M_PI);;
 		}
 	}
+	hp->ellipse = ellipse;
+	if (MI_WIN_IS_FULLRANDOM(mi))
+		hp->ellipse = (Bool) (!NRAND(5));	/* 1:5 chance of running ellipse stuff */
+
+	hp->width = MI_WIN_WIDTH(mi);
+	hp->height = MI_WIN_HEIGHT(mi);
+	hp->xmid = hp->width / 2;
+	hp->ymid = hp->height / 2;
+	hp->redraw = 0;
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
 	if (MI_NPIXELS(mi) > 2)
 		hp->color = NRAND(MI_NPIXELS(mi));
 	hp->time = 0;
-	random_helix(mi);
+
+	if (hp->ellipse) {
+		random_trig(mi);
+	} else
+		random_helix(mi);
 }
 
 void
@@ -199,8 +291,12 @@ draw_helix(ModeInfo * mi)
 	if (++hp->time > MI_CYCLES(mi))
 		init_helix(mi);
 	if (hp->redraw) {
-		helix(mi, hp->radius1, hp->radius2, hp->d_angle,
-		      hp->factor1, hp->factor2, hp->factor3, hp->factor4);
+		if (hp->ellipse) {
+			trig(mi);
+		} else {
+			helix(mi, hp->radius1, hp->radius2, hp->d_angle,
+			 hp->factor1, hp->factor2, hp->factor3, hp->factor4);
+		}
 		hp->redraw = 0;
 	}
 }
