@@ -1,6 +1,6 @@
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)hop.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)hop.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -35,13 +35,41 @@ static const char sccsid[] = "@(#)hop.c	4.00 97/01/01 xlockmore";
 
 #include "xlock.h"
 
-#define SQRT 0
-#define SIN 1
-#define JONG 2
-#define OPS 3
+#define DEF_JONG "False"
+#define DEF_SINE "False"
+
+static Bool jong;
+static Bool sine;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-jong", ".hop.jong", XrmoptionNoArg, (caddr_t) "on"},
+	{"+jong", ".hop.jong", XrmoptionNoArg, (caddr_t) "off"},
+	{"-sine", ".hop.sine", XrmoptionNoArg, (caddr_t) "on"},
+	{"+sine", ".hop.sine", XrmoptionNoArg, (caddr_t) "off"}
+};
+static argtype vars[] =
+{
+	{(caddr_t *) & jong, "jong", "Jong", DEF_JONG, t_Bool},
+	{(caddr_t *) & sine, "sine", "Sine", DEF_SINE, t_Bool}
+};
+static OptionStruct desc[] =
+{
+	{"-/+jong", "turn on/off jong format"},
+	{"-/+sine", "turn on/off sine format"}
+};
 
 ModeSpecOpt hop_opts =
-{0, NULL, 0, NULL, NULL};
+{4, opts, 2, vars, desc};
+
+#define SQRT 0
+#define JONG 1
+#define SINE 2
+#ifdef OFFENDING
+#define OPS 2			/* Sine might be too close to a Swastika for some... */
+#else
+#define OPS 3
+#endif
 
 typedef struct {
 	int         centerx;
@@ -80,17 +108,18 @@ init_hop(ModeInfo * mi)
 	hp->centerx = MI_WIN_WIDTH(mi) / 2;
 	hp->centery = MI_WIN_HEIGHT(mi) / 2;
 	/* Make the other operations less common since they are less interesting */
-	if (NRAND(6))
+	if (MI_WIN_IS_FULLRANDOM(mi)) {
+		if (NRAND(6))
+			hp->op = SQRT;
+		else
+			hp->op = NRAND(OPS);
+	} else {
 		hp->op = SQRT;
-	else
-#ifdef OFFENDING
-		/* Sine hop may be too close to a Nazi cross to suit some.
-		   Compile with -DOFFENDING to weed this out.
-		 */
-		hp->op = NRAND(OPS - 2) + 2;
-#else
-		hp->op = NRAND(OPS - 1) + 1;
-#endif
+		if (jong)
+			hp->op = JONG;
+		else if (sine)
+			hp->op = SINE;
+	}
 	switch (hp->op) {
 		case SQRT:
 			range = sqrt((double) hp->centerx * hp->centerx +
@@ -102,15 +131,15 @@ init_hop(ModeInfo * mi)
 			if (LRAND() & 1)
 				hp->c = 0.0;
 			break;
-		case SIN:
-			hp->a = M_PI + ((LRAND() / MAXRAND) * 2.0 - 1.0) * 0.7;
-			break;
 		case JONG:
 			hp->a = (LRAND() / MAXRAND) * 2.0 * M_PI - M_PI;
 			hp->b = (LRAND() / MAXRAND) * 2.0 * M_PI - M_PI;
 			hp->c = (LRAND() / MAXRAND) * 2.0 * M_PI - M_PI;
 			hp->d = (LRAND() / MAXRAND) * 2.0 * M_PI - M_PI;
 			break;
+		case SINE:
+			hp->a = M_PI + ((LRAND() / MAXRAND) * 2.0 - 1.0) * 0.7;
+			break;
 	}
 	if (MI_NPIXELS(mi) > 2)
 		hp->pix = NRAND(MI_NPIXELS(mi));
@@ -154,20 +183,23 @@ draw_hop(ModeInfo * mi)
 				xp->x = hp->centerx + (int) (hp->i + hp->j);
 				xp->y = hp->centery - (int) (hp->i - hp->j);
 				break;
-			case SIN:
+			case JONG:
+				if (hp->centerx > 0)
+					oldi = hp->i + 4 * hp->inc / hp->centerx;
+				else
+					oldi = hp->i;
+				hp->j = sin(hp->c * hp->i) - cos(hp->d * hp->j);
+				hp->i = sin(hp->a * oldj) - cos(hp->b * oldi);
+				xp->x = hp->centerx + (int) (hp->centerx * (hp->i + hp->j) / 4.0);
+				xp->y = hp->centery - (int) (hp->centery * (hp->i - hp->j) / 4.0);
+				break;
+			case SINE:
 				oldi = hp->i + hp->inc;
 				hp->j = hp->a - hp->i;
 				hp->i = oldj - sin(oldi);
 				xp->x = hp->centerx + (int) (hp->i + hp->j);
 				xp->y = hp->centery - (int) (hp->i - hp->j);
 				break;
-			case JONG:
-				oldi = hp->i + 4 * hp->inc / hp->centerx;
-				hp->j = sin(hp->c * hp->i) - cos(hp->d * hp->j);
-				hp->i = sin(hp->a * oldj) - cos(hp->b * oldi);
-				xp->x = hp->centerx + (int) (hp->centerx * (hp->i + hp->j) / 4.0);
-				xp->y = hp->centery - (int) (hp->centery * (hp->i - hp->j) / 4.0);
-				break;
 		}
 		xp++;
 	}
