@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)hyper.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)hyper.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -10,7 +10,7 @@ static char sccsid[] = "@(#)hyper.c	3.11 96/09/20 xlockmore";
  *
  * Copyright (c) 1992 by Jamie Zawinski
  *
- * 2-Sep-93: xlock version (David Bagley <bagleyd@hertz.njit.edu>)
+ * 2-Sep-93: xlock version (David Bagley <bagleyd@bigfoot.com>)
  * 1992:     xscreensaver version (Jamie Zawinski <jwz@netscape.com>)
  */
 
@@ -26,7 +26,6 @@ static char sccsid[] = "@(#)hyper.c	3.11 96/09/20 xlockmore";
  * implied warranty.
  */
 
-#include <math.h>
 #include "xlock.h"
 
 #define COMBOS 6
@@ -128,6 +127,7 @@ init_hyper(ModeInfo * mi)
 	}
 	hp = &hypers[MI_SCREEN(mi)];
 
+	hp->redrawing = False;
 	hp->width = MI_WIN_WIDTH(mi);
 	hp->height = MI_WIN_HEIGHT(mi);
 	hp->unit_pixels = hp->width < hp->height ? hp->width : hp->height;
