@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)life1d.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)life1d.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -103,7 +103,7 @@ typedef struct {
 	int         k, r;
 	long        code;
 	int         repeating;
-	unsigned char *nextstate;
+	char       *nextstate;
 	int         colors[MAXSTATES];
 	Pixmap      pixmaps[MAXSTATES];
 	GC          stippledGC;
@@ -280,7 +280,7 @@ static int  totalistic_rules[][3] =
 #define TOTALISTICRULES (sizeof totalistic_rules / sizeof totalistic_rules[0])
 
 #if 0
-static unsigned char lcau21_rules[][8] =
+static char lcau21_rules[][9] =
 {
 	"00010010",		/* 18 Hollow enlarging triangles */
 	"00110110",		/* 54 Hollow triangles */
@@ -294,7 +294,7 @@ static unsigned char lcau21_rules[][8] =
 #define LCAU2RULES (sizeof lcau21_rules / sizeof lcau21_rules[0])
 #endif
 
-static unsigned char lcau31_rules[][27] =
+static char lcau31_rules[][28] =
 {
 	"000222111000111222222111000",	/* equal thirds */
 	"002220210110110211110002200",	/* threads on triangles */
@@ -324,7 +324,7 @@ static unsigned char lcau31_rules[][27] =
 
 #define LCAU3RULES (sizeof lcau31_rules / sizeof lcau31_rules[0])
 
-static unsigned char lcau41_rules[][64] =
+static char lcau41_rules[][65] =
 {
 	"0000000313131323232312121213232323231313131101010202020202010101",	/* skewed triangle */
 	"0000020202000000020232220203300000000101020030000100322121003020",	/* slow glider - copies bar */
@@ -442,21 +442,21 @@ GetRule(life1dstruct * lp, int i)
 
 		pow = power(lp->k, sum_size - 1);	/* Should be less than max long */
 		for (j = 0; j < sum_size; j++) {
-			lp->nextstate[sum_size - 1 - j] = (unsigned char) (code / pow);
-			code -= (long) lp->nextstate[sum_size - 1 - j] * pow;
+			lp->nextstate[sum_size - 1 - j] = (char) (code / pow);
+			code -= ((long) lp->nextstate[sum_size - 1 - j]) * pow;
 			pow /= (long) lp->k;
 		}
 	} else {
 		lp->r = 1;
 #ifdef LCAU2RULES
-		if (i < LCAU2RULES) {
+		if (i < (int) LCAU2RULES) {
 			lp->k = 2;
 			sum_size = power(lp->k, 2 * lp->r + 1);
 			for (j = 0; j < sum_size; j++)
 				lp->nextstate[sum_size - 1 - j] = lcau21_rules[i][j] - '0';
 		} else if (i < LCAU2RULES + LCAU3RULES)
 #else
-		if (i < LCAU3RULES)
+		if (i < (int) LCAU3RULES)
 #endif
 		{
 			lp->k = 3;
@@ -535,8 +535,7 @@ init_life1d(ModeInfo * mi)
 		maxsum_size = power(maxstates, (2 * maxradius + 1));
 	}
 	if (lp->nextstate == NULL)
-		lp->nextstate = (unsigned char *) malloc(maxsum_size *
-						     sizeof (unsigned char));
+		lp->nextstate = (char *) malloc(maxsum_size * sizeof (char));
 
 	if (!logo.data) {
 		logo.width = CELL_WIDTH;
@@ -656,7 +655,7 @@ draw_life1d(ModeInfo * mi)
 					pow *= lp->k;
 				}
 			}
-			lp->newcells[col] = lp->nextstate[sum];
+			lp->newcells[col] = (unsigned char) lp->nextstate[sum];
 		}
 		(void) memcpy((char *) (lp->oldcells + maxradius),
 			  (char *) lp->newcells, lp->ncols + 2 * lp->border);
