@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static char sccsid[] = "@(#)life3d.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)life3d.c	4.02 97/04/18 xlockmore";
 
 #endif
 
@@ -12,6 +12,7 @@ static char sccsid[] = "@(#)life3d.c	4.00 97/01/01 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
+ * 18-Apr-97: Memory leak fixed by Tom Schmidt <tschmidt@micron.com>
  * 12-Mar-95: added LIFE_5766 compile-time option
  * 12-Feb-95: shooting gliders added
  * 07-Dec-94: used life.c and a DOS version of 3dlife
@@ -1274,8 +1275,7 @@ BaseOffset(life3dstruct * lp,
 	*o = (x & 3) + ((y & 3) << 2) + ((z & 3) << 4);
 
 	if (!lp->base[*b])
-/* PURIFY 3.2 on Solaris2 and PURIFY 4.0 on SunOS4 reports a memory leak on
-   the next line. I think its faked out */
+/* PURIFY 3.0a on SunOS4 reports a 384 byte memory leak on the next line */
 		lp->base[*b] = (unsigned char *) calloc(64, sizeof (unsigned char));
 }
 
@@ -1358,6 +1358,13 @@ static void
 End3D(life3dstruct * lp)
 {
 	CellList   *ptr;
+	int i;
+
+	for (i = 0; i < BASESIZE; i++)
+		if (lp->base[i] != NULL) {
+			(void) free((void *) lp->base[i]);
+			lp->base[i] = NULL;
+		}
 
 	while (lp->ptrhead != NULL) {
 		SetMem(lp, lp->ptrhead->x, lp->ptrhead->y, lp->ptrhead->z, OFF);
@@ -2028,7 +2035,7 @@ release_life3d(ModeInfo * mi)
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			life3dstruct *lp = &life3ds[screen];
 
-			End3D(lp);	/* Did I miss something? */
+			End3D(lp);
 		}
 		(void) free((void *) life3ds);
 		life3ds = NULL;
