@@ -1,5 +1,6 @@
-#ifndef lint
-static char sccsid[] = "@(#)marquee.c	3.11 96/09/20 xlockmore";
+
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)marquee.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -28,7 +29,7 @@ extern char *messagefile;
 extern char *message;
 
 extern XFontStruct *getFont(Display * display);
-extern char *getWords(void);
+extern char *getWords(int screen);
 extern int  isRibbon(void);
 
 typedef struct {
@@ -41,12 +42,10 @@ typedef struct {
 	int         t;
 	int         startx;
 	int         nonblanks;
-	int         done;
 	int         color;
 	int         time;
 	GC          gc;
 	char       *words;
-	char       *wordbuf;
 	char        modwords[256];
 } marqueestruct;
 
@@ -65,6 +64,101 @@ font_width(XFontStruct * font, char ch)
 	return xcs.width;
 }
 
+/* returns 1 if c is a printable char, the test should work for most 8 * bit
+   charsets (e.g. latin1), if we would use isprint, we would * depend of
+   locale settings that are probably incorrect. */
+
+static int
+is_valid_char(char c)
+{
+	return (unsigned char) c >= ' ';
+}
+
+static int
+is_char_back_char(char *s)
+{
+	return is_valid_char(*s) && *(s + 1) == '\b' &&
+		*(s + 2) && is_valid_char(*(s + 2));
+}
+
+static int
+char_back_char_width(char *s)
+{
+	int         w1 = char_width[(int) (unsigned char) *s];
+	int         w2 = char_width[(int) (unsigned char) *(s + 2)];
+
+	return w2 < w1 ? w1 : w2;
+}
+
+
+/*-
+ * fix strings of the form abc^H^H^H123 to a^H1b^H2c^H3, since we only
+ * handle backspace for char, back, char correctly. We do this without
+ * duplicating the string, but I'm not sure if there are conditions
+ * when the string is actually const. (when it is def_message, no ^Hs
+ * are present, I'm not sure about resource strings)
+ */
+
+static char *
+fixup_back(char *s)
+{
+	char       *p, *p1, *p2;
+	char        tmp[1000];
+	char       *t;
+	char       *w;
+
+	/* first of all, check if we have to do anything */
+
+	if (!*s)
+		return s;
+
+	for (p = s + 1; *p; p++)
+		if (*p == '\b' && *(p + 1) == '\b')
+			break;
+	if (!*p)
+		return s;
+
+	/* now search for runs of the form char*n, back*n, char. */
+
+	for (p = s; *p; p++) {
+		for (p1 = p; *p1 && is_valid_char(*p1); p1++);
+		if (*p1 == '\b') {
+			for (p2 = p1; *p2 && *p2 == '\b'; p2++);
+
+			/* do we have `enough' chars for the backspaces? */
+
+			if (p2 - p1 > 1 && p1 - p >= p2 - p1) {
+				if (p1 - p > p2 - p1) {
+					p = p1 - (p2 - p1);
+				}
+				/* the situation is as follows:
+				   p points to the first char,
+				   p1 to the first backspace (end first char run),
+				   p2 to the first char in the 2nd run
+
+				   Question: how to do that without tmp storage?
+				 */
+				(void) strncpy(tmp, p, p1 - p);
+				t = tmp;
+				w = p;
+				while (t - tmp < p1 - p && *p2) {
+					*w++ = *t++;
+					*w++ = '\b';
+					*w++ = *p2++;
+				}
+				p = p2;
+			} else {
+				p = p2;
+			}
+		} else {
+			/* we hit some other control char, just continue at this
+			   position */
+			p = p1;
+		}
+	}
+	return s;
+}
+
 static int
 text_font_width(char *string)
 {
@@ -82,6 +176,10 @@ text_font_width(char *string)
 				t = 0;
 				break;
 			case '\b':
+				/* we handle only char, ^H, char smartly, if
+				 * we have something different, we use the
+				 * (probably wrong) assumption that we have
+				 * a monospaced font. */
 				if (t) {
 					t--;
 					x -= char_width[' '];
@@ -95,7 +193,13 @@ text_font_width(char *string)
 				break;
 			default:
 				t++;
-				x += char_width[(int) *string];
+				/* handle char, ^H, char */
+				if (is_char_back_char(string)) {
+					x += char_back_char_width(string);
+					string += 2;
+				} else {
+					x += char_width[(int) (unsigned char) *string];
+				}
 		}
 		string++;
 	}
@@ -127,8 +231,8 @@ add_blanks(marqueestruct * mp)
 		(void) strcat(mp->modwords, "  ");
 	}
 	mp->x -= 2 * char_width[' '];
-	if (mp->x <= -char_width[(int) mp->modwords[0]]) {
-		mp->x += char_width[(int) mp->modwords[0]];
+	if (mp->x <= -char_width[(int) (unsigned char) mp->modwords[0]]) {
+		mp->x += char_width[(int) (unsigned char) mp->modwords[0]];
 		(void) memcpy(mp->modwords, &(mp->modwords[1]), mp->nonblanks);
 		mp->nonblanks--;
 	}
@@ -145,8 +249,8 @@ add_letter(marqueestruct * mp, char letter)
 		(void) strcat(mp->modwords, "  ");
 	}
 	mp->x -= char_width[(int) letter];
-	if (mp->x <= -char_width[(int) mp->modwords[0]]) {
-		mp->x += char_width[(int) mp->modwords[0]];
+	if (mp->x <= -char_width[(int) (unsigned char) mp->modwords[0]]) {
+		mp->x += char_width[(int) (unsigned char) mp->modwords[0]];
 		(void) memcpy(mp->modwords, &(mp->modwords[1]), mp->t);
 		mp->modwords[mp->t] = ' ';
 		mp->t--;
@@ -171,7 +275,8 @@ init_marquee(ModeInfo * mi)
 
 	mp->win_width = MI_WIN_WIDTH(mi);
 	mp->win_height = MI_WIN_HEIGHT(mi);
-	mp->color = 0;
+	if (MI_NPIXELS(mi) > 2)
+		mp->color = NRAND(MI_NPIXELS(mi));
 	mp->time = 0;
 	mp->t = 0;
 	mp->nonblanks = 0;
@@ -179,8 +284,7 @@ init_marquee(ModeInfo * mi)
 	XClearWindow(display, MI_WINDOW(mi));
 	if (mode_font == None)
 		mode_font = getFont(display);
-	if (!mp->done) {
-		mp->done = 1;
+	if (mp->gc == NULL) {
 		gcv.font = mode_font->fid;
 		XSetFont(display, MI_GC(mi), mode_font->fid);
 		gcv.graphics_exposures = False;
@@ -197,12 +301,7 @@ init_marquee(ModeInfo * mi)
 			else
 				char_width[i] = font_width(mode_font, (char) mode_font->default_char);
 	}
-	if (mp->wordbuf)
-		(void) free((void *) mp->wordbuf);
-	mp->wordbuf = strdup(getWords());
-	if (!mp->wordbuf)
-		error("%s: out of memory.\n");
-	mp->words = mp->wordbuf;
+	mp->words = fixup_back(getWords(MI_SCREEN(mi)));
 	mp->y = 0;
 
 	if (isRibbon()) {
@@ -276,6 +375,7 @@ draw_marquee(ModeInfo * mi)
 				return;
 			case '\b':
 				if (mp->t) {
+					/* see note in text_font_width */
 					mp->t--;
 					mp->x -= char_width[' '];
 				}
@@ -310,9 +410,23 @@ draw_marquee(ModeInfo * mi)
 						mp->color = 0;
 				} else
 					XSetForeground(display, mp->gc, MI_WIN_WHITE_PIXEL(mi));
-				XDrawString(display, window, mp->gc, mp->x, mp->y + mp->ascent, ch, 1);
+				if (is_char_back_char(ch)) {
+					int         xmid = mp->x + (char_back_char_width(ch) + 1) / 2;
+
+					XDrawString(display, window, mp->gc,
+						    xmid - char_width[(int) (unsigned char) *ch] / 2,
+						  mp->y + mp->ascent, ch, 1);
+					XDrawString(display, window, mp->gc,
+						    xmid - char_width[(int) (unsigned char) *(ch + 2)] / 2,
+					      mp->y + mp->ascent, ch + 2, 1);
+					mp->x += char_back_char_width(ch);
+					mp->words += 2;
+				} else {
+					XDrawString(display, window, mp->gc,
+					   mp->x, mp->y + mp->ascent, ch, 1);
+					mp->x += char_width[(int) (unsigned char) *ch];
+				}
 				mp->t++;
-				mp->x += char_width[(int) *ch];
 		}
 		mp->words++;
 	}
@@ -327,9 +441,8 @@ release_marquee(ModeInfo * mi)
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			marqueestruct *mp = &marquees[screen];
 
-			if (!mp->gc)
+			if (mp->gc != NULL)
 				XFreeGC(MI_DISPLAY(mi), mp->gc);
-
 		}
 		(void) free((void *) marquees);
 		marquees = NULL;
