@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)maze.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)maze.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -14,7 +14,7 @@ static char sccsid[] = "@(#)maze.c	3.11 96/09/20 xlockmore";
  * Revision History:
  * 27-Feb-96: Add ModeInfo args to init and callback hooks, use new
  *		ModeInfo handle to specify long pauses (eliminate onepause).
- *		Ron Hitchens <ronh@utw.com>
+ *		Ron Hitchens <ron@idiom.com>
  * 20-Jul-95: minimum size fix (Peter Schmitzberger schmitz@coma.sbg.ac.at)
  * 17-Jun-95: removed sleep statements
  * 22-Mar-95: multidisplay fix (Caleb Epstein epstein_caleb@jpmorgan.com)
@@ -23,7 +23,7 @@ static char sccsid[] = "@(#)maze.c	3.11 96/09/20 xlockmore";
  * 4-Feb-95: patch to slow down maze (Heath Kehoe hakehoe@icaen.uiowa.edu)
  * 17-Jun-94: HP ANSI C compiler needs a type cast for gray_bits
  *            Richard Lloyd (R.K.Lloyd@csc.liv.ac.uk) 
- * 2-Sep-93: xlock version (David Bagley bagleyd@hertz.njit.edu) 
+ * 2-Sep-93: xlock version (David Bagley bagleyd@bigfoot.com) 
  * 7-Mar-93: Good ideas from xscreensaver (Jamie Zawinski jwz@netscape.com)
  * 6-Jun-85: Martin Weiss Sun Microsystems 
  */
@@ -56,15 +56,7 @@ static char sccsid[] = "@(#)maze.c	3.11 96/09/20 xlockmore";
  */
 
 #include "xlock.h"
-#ifdef VMS
-#if 0
-#include "../bitmaps/gray1.xbm"
-#else
-#include <decw$bitmaps/gray1.xbm>
-#endif
-#else
-#include <X11/bitmaps/gray1>
-#endif
+#include "bitmaps/gray1.xbm"
 
 /* aliases for vars defined in the bitmap file */
 #define ICON_WIDTH   image_width
@@ -73,8 +65,8 @@ static char sccsid[] = "@(#)maze.c	3.11 96/09/20 xlockmore";
 
 #include "maze.xbm"
 
-#define MIN_MAZE_SIZE	3
-#define MIN_SIZE        8
+#define MINGRIDSIZE	3
+#define MINSIZE        8
 
 #define WALL_TOP	0x8000
 #define WALL_RIGHT	0x4000
@@ -113,12 +105,12 @@ typedef struct {
 } paths;
 
 typedef struct {
-	int         maze_size_x, maze_size_y, maze_size, border_x, border_y;
+	int         ncols, nrows, maze_size, xb, yb;
 	int         sqnum, cur_sq_x, cur_sq_y, path_length;
 	int         start_x, start_y, start_dir, end_x, end_y, end_dir;
 	int         logo_x, logo_y;
 	int         width, height;
-	int         sq_size_x, sq_size_y, logo_size_x, logo_size_y;
+	int         xs, ys, logo_size_x, logo_size_y;
 	int         solving, current_path, stage;
 	/*int         cycles; */
 	unsigned short *maze;
@@ -159,35 +151,30 @@ static void
 set_maze_sizes(ModeInfo * mi)
 {
 	mazestruct *mp = &mazes[MI_SCREEN(mi)];
+	int         size = MI_SIZE(mi);
 
-	/* batchcount is the upper bound on the size */
-	mp->sq_size_x = MI_BATCHCOUNT(mi);
-	if (mp->sq_size_x < -MIN_SIZE) {
+	if (size < -MINSIZE) {
 		free_maze(mp);
-		mp->sq_size_x = NRAND(-mp->sq_size_x - MIN_SIZE + 1) + MIN_SIZE;
-	} else if (mp->sq_size_x < MIN_SIZE)
-		mp->sq_size_x = MIN_SIZE;
-	mp->sq_size_y = mp->sq_size_x;
-	mp->logo_size_x = logo.width / mp->sq_size_x + 1;
-	mp->logo_size_y = logo.height / mp->sq_size_y + 1;
+		mp->ys = NRAND(MIN(-size, MAX(MINSIZE, (MIN(mp->width, mp->height) - 1) /
+				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+	} else if (size < MINSIZE) {
+		if (!size)
+			mp->ys = MAX(MINSIZE, (MIN(mp->width, mp->height) - 1) / MINGRIDSIZE);
+		else
+			mp->ys = MINSIZE;
+	} else
+		mp->ys = MIN(size, MAX(MINSIZE, (MIN(mp->width, mp->height) - 1) /
+				       MINGRIDSIZE));
+	mp->xs = mp->ys;
+	mp->logo_size_x = logo.width / mp->xs + 1;
+	mp->logo_size_y = logo.height / mp->ys + 1;
 
-	mp->border_x = mp->border_y = 1;
-	mp->maze_size_x = (mp->width - mp->border_x) / mp->sq_size_x;
-	mp->maze_size_y = (mp->height - mp->border_y) / mp->sq_size_y;
+	mp->ncols = MAX((mp->width - 1) / mp->xs, MINGRIDSIZE);
+	mp->nrows = MAX((mp->height - 1) / mp->ys, MINGRIDSIZE);
 
-	if (mp->maze_size_x < MIN_MAZE_SIZE ||
-	    mp->maze_size_y < MIN_MAZE_SIZE) {
-		mp->sq_size_y = mp->sq_size_x = 10;
-		mp->maze_size_x = (mp->width - mp->border_x) / mp->sq_size_x;
-		mp->maze_size_y = (mp->height - mp->border_y) / mp->sq_size_y;
-		if (mp->maze_size_x < MIN_MAZE_SIZE ||
-		    mp->maze_size_y < MIN_MAZE_SIZE) {
-			error("%s: maze too small, exitting\n");
-		}
-	}
-	mp->border_x = (mp->width - mp->maze_size_x * mp->sq_size_x) / 2;
-	mp->border_y = (mp->height - mp->maze_size_y * mp->sq_size_y) / 2;
-	mp->maze_size = mp->maze_size_x * mp->maze_size_y;
+	mp->xb = (mp->width - mp->ncols * mp->xs) / 2;
+	mp->yb = (mp->height - mp->nrows * mp->ys) / 2;
+	mp->maze_size = mp->ncols * mp->nrows;
 	if (!mp->maze)
 		mp->maze = (unsigned short *)
 			calloc(mp->maze_size, sizeof (unsigned short));
@@ -217,29 +204,29 @@ initialize_maze(ModeInfo * mi)
 		XSetForeground(display, mp->grayGC, MI_PIXEL(mi, i));
 	}
 	/* initialize all squares */
-	for (i = 0; i < mp->maze_size_x; i++) {
-		for (j = 0; j < mp->maze_size_y; j++) {
-			mp->maze[i * mp->maze_size_y + j] = 0;
+	for (i = 0; i < mp->ncols; i++) {
+		for (j = 0; j < mp->nrows; j++) {
+			mp->maze[i * mp->nrows + j] = 0;
 		}
 	}
 
 	/* top wall */
-	for (i = 0; i < mp->maze_size_x; i++) {
-		mp->maze[i * mp->maze_size_y] |= WALL_TOP;
+	for (i = 0; i < mp->ncols; i++) {
+		mp->maze[i * mp->nrows] |= WALL_TOP;
 	}
 
 	/* right wall */
-	for (j = 0; j < mp->maze_size_y; j++) {
-		mp->maze[(mp->maze_size_x - 1) * mp->maze_size_y + j] |= WALL_RIGHT;
+	for (j = 0; j < mp->nrows; j++) {
+		mp->maze[(mp->ncols - 1) * mp->nrows + j] |= WALL_RIGHT;
 	}
 
 	/* bottom wall */
-	for (i = 0; i < mp->maze_size_x; i++) {
-		mp->maze[i * mp->maze_size_y + mp->maze_size_y - 1] |= WALL_BOTTOM;
+	for (i = 0; i < mp->ncols; i++) {
+		mp->maze[i * mp->nrows + mp->nrows - 1] |= WALL_BOTTOM;
 	}
 
 	/* left wall */
-	for (j = 0; j < mp->maze_size_y; j++) {
+	for (j = 0; j < mp->nrows; j++) {
 		mp->maze[j] |= WALL_LEFT;
 	}
 
@@ -247,25 +234,25 @@ initialize_maze(ModeInfo * mi)
 	wall = NRAND(4);
 	switch (wall) {
 		case 0:
-			i = NRAND(mp->maze_size_x);
+			i = NRAND(mp->ncols);
 			j = 0;
 			break;
 		case 1:
-			i = mp->maze_size_x - 1;
-			j = NRAND(mp->maze_size_y);
+			i = mp->ncols - 1;
+			j = NRAND(mp->nrows);
 			break;
 		case 2:
-			i = NRAND(mp->maze_size_x);
-			j = mp->maze_size_y - 1;
+			i = NRAND(mp->ncols);
+			j = mp->nrows - 1;
 			break;
 		case 3:
 			i = 0;
-			j = NRAND(mp->maze_size_y);
+			j = NRAND(mp->nrows);
 			break;
 	}
-	mp->maze[i * mp->maze_size_y + j] |= START_SQUARE;
-	mp->maze[i * mp->maze_size_y + j] |= (DOOR_IN_TOP >> wall);
-	mp->maze[i * mp->maze_size_y + j] &= ~(WALL_TOP >> wall);
+	mp->maze[i * mp->nrows + j] |= START_SQUARE;
+	mp->maze[i * mp->nrows + j] |= (DOOR_IN_TOP >> wall);
+	mp->maze[i * mp->nrows + j] &= ~(WALL_TOP >> wall);
 	mp->cur_sq_x = i;
 	mp->cur_sq_y = j;
 	mp->start_x = i;
@@ -277,38 +264,38 @@ initialize_maze(ModeInfo * mi)
 	wall = (wall + 2) % 4;
 	switch (wall) {
 		case 0:
-			i = NRAND(mp->maze_size_x);
+			i = NRAND(mp->ncols);
 			j = 0;
 			break;
 		case 1:
-			i = mp->maze_size_x - 1;
-			j = NRAND(mp->maze_size_y);
+			i = mp->ncols - 1;
+			j = NRAND(mp->nrows);
 			break;
 		case 2:
-			i = NRAND(mp->maze_size_x);
-			j = mp->maze_size_y - 1;
+			i = NRAND(mp->ncols);
+			j = mp->nrows - 1;
 			break;
 		case 3:
 			i = 0;
-			j = NRAND(mp->maze_size_y);
+			j = NRAND(mp->nrows);
 			break;
 	}
-	mp->maze[i * mp->maze_size_y + j] |= END_SQUARE;
-	mp->maze[i * mp->maze_size_y + j] |= (DOOR_OUT_TOP >> wall);
-	mp->maze[i * mp->maze_size_y + j] &= ~(WALL_TOP >> wall);
+	mp->maze[i * mp->nrows + j] |= END_SQUARE;
+	mp->maze[i * mp->nrows + j] |= (DOOR_OUT_TOP >> wall);
+	mp->maze[i * mp->nrows + j] &= ~(WALL_TOP >> wall);
 	mp->end_x = i;
 	mp->end_y = j;
 	mp->end_dir = wall;
 
 	/* set logo */
-	if ((mp->maze_size_x > mp->logo_size_x + 6) &&
-	    (mp->maze_size_y > mp->logo_size_y + 6)) {
-		mp->logo_x = NRAND(mp->maze_size_x - mp->logo_size_x - 6) + 3;
-		mp->logo_y = NRAND(mp->maze_size_y - mp->logo_size_y - 6) + 3;
+	if ((mp->ncols > mp->logo_size_x + 6) &&
+	    (mp->nrows > mp->logo_size_y + 6)) {
+		mp->logo_x = NRAND(mp->ncols - mp->logo_size_x - 6) + 3;
+		mp->logo_y = NRAND(mp->nrows - mp->logo_size_y - 6) + 3;
 
 		for (i = 0; i < mp->logo_size_x; i++)
 			for (j = 0; j < mp->logo_size_y; j++)
-				mp->maze[(mp->logo_x + i) * mp->maze_size_y + mp->logo_y + j] |=
+				mp->maze[(mp->logo_x + i) * mp->nrows + mp->logo_y + j] |=
 					DOOR_IN_TOP;
 	} else
 		mp->logo_y = mp->logo_x = -1;
@@ -324,60 +311,60 @@ choose_door(ModeInfo * mi)
 	num_candidates = 0;
 
 	/* top wall */
-	if ((!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	if ((!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_IN_TOP)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_OUT_TOP)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       WALL_TOP))) {
-		if (mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y - 1] &
+		if (mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y - 1] &
 		    DOOR_IN_ANY) {
-			mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] |= WALL_TOP;
-			mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y - 1] |=
+			mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] |= WALL_TOP;
+			mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y - 1] |=
 				WALL_BOTTOM;
 		} else
 			candidates[num_candidates++] = 0;
 	}
 	/* right wall */
-	if ((!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	if ((!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_IN_RIGHT)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_OUT_RIGHT)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       WALL_RIGHT))) {
-		if (mp->maze[(mp->cur_sq_x + 1) * mp->maze_size_y + mp->cur_sq_y] &
+		if (mp->maze[(mp->cur_sq_x + 1) * mp->nrows + mp->cur_sq_y] &
 		    DOOR_IN_ANY) {
-			mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] |= WALL_RIGHT;
-			mp->maze[(mp->cur_sq_x + 1) * mp->maze_size_y + mp->cur_sq_y] |=
+			mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] |= WALL_RIGHT;
+			mp->maze[(mp->cur_sq_x + 1) * mp->nrows + mp->cur_sq_y] |=
 				WALL_LEFT;
 		} else
 			candidates[num_candidates++] = 1;
 	}
 	/* bottom wall */
-	if ((!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	if ((!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_IN_BOTTOM)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_OUT_BOTTOM)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       WALL_BOTTOM))) {
-		if (mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y + 1] &
+		if (mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y + 1] &
 		    DOOR_IN_ANY) {
-			mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] |= WALL_BOTTOM;
-			mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y + 1] |= WALL_TOP;
+			mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] |= WALL_BOTTOM;
+			mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y + 1] |= WALL_TOP;
 		} else
 			candidates[num_candidates++] = 2;
 	}
 	/* left wall */
-	if ((!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	if ((!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_IN_LEFT)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       DOOR_OUT_LEFT)) &&
-	    (!(mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] &
+	    (!(mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
 	       WALL_LEFT))) {
-		if (mp->maze[(mp->cur_sq_x - 1) * mp->maze_size_y + mp->cur_sq_y] &
+		if (mp->maze[(mp->cur_sq_x - 1) * mp->nrows + mp->cur_sq_y] &
 		    DOOR_IN_ANY) {
-			mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] |= WALL_LEFT;
-			mp->maze[(mp->cur_sq_x - 1) * mp->maze_size_y + mp->cur_sq_y] |=
+			mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] |= WALL_LEFT;
+			mp->maze[(mp->cur_sq_x - 1) * mp->nrows + mp->cur_sq_y] |=
 				WALL_RIGHT;
 		} else
 			candidates[num_candidates++] = 3;
@@ -397,9 +384,9 @@ draw_maze_walls(ModeInfo * mi)
 	mazestruct *mp = &mazes[MI_SCREEN(mi)];
 	int         i, j, isize;
 
-	for (i = 0; i < mp->maze_size_x; i++) {
-		isize = i * mp->maze_size_y;
-		for (j = 0; j < mp->maze_size_y; j++) {
+	for (i = 0; i < mp->ncols; i++) {
+		isize = i * mp->nrows;
+		for (j = 0; j < mp->nrows; j++) {
 			/* Only need to draw half the walls... since they are shared */
 			/* top wall */
 			if (	/*(!(mp->maze[isize + j] & DOOR_IN_TOP)) &&
@@ -419,8 +406,10 @@ static int
 backup(mazestruct * mp)
 {				/* back up a move */
 	mp->sqnum--;
-	mp->cur_sq_x = mp->move_list[mp->sqnum].x;
-	mp->cur_sq_y = mp->move_list[mp->sqnum].y;
+	if (mp->sqnum >= 0) {
+		mp->cur_sq_x = mp->move_list[mp->sqnum].x;
+		mp->cur_sq_y = mp->move_list[mp->sqnum].y;
+	}
 	return (mp->sqnum);
 }				/* end of backup() */
 
@@ -439,7 +428,7 @@ create_maze_walls(ModeInfo * mi)
 				return;		/* done ... return */
 
 		/* mark the out door */
-		mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] |=
+		mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] |=
 			(DOOR_OUT_TOP >> newdoor);
 
 		switch (newdoor) {
@@ -459,11 +448,11 @@ create_maze_walls(ModeInfo * mi)
 		mp->sqnum++;
 
 		/* mark the in door */
-		mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] |=
+		mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] |=
 			(DOOR_IN_TOP >> ((newdoor + 2) % 4));
 
 		/* if end square set path length and save path */
-		if (mp->maze[mp->cur_sq_x * mp->maze_size_y + mp->cur_sq_y] & END_SQUARE) {
+		if (mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] & END_SQUARE) {
 			mp->path_length = mp->sqnum;
 			for (i = 0; i < mp->path_length; i++) {
 				mp->save_path[i].x = mp->move_list[i].x;
@@ -484,46 +473,42 @@ draw_maze_border(ModeInfo * mi)
 	mazestruct *mp = &mazes[MI_SCREEN(mi)];
 	register int i, j;
 
-	for (i = 0; i < mp->maze_size_x; i++) {
-		if (mp->maze[i * mp->maze_size_y] & WALL_TOP) {
+	for (i = 0; i < mp->ncols; i++) {
+		if (mp->maze[i * mp->nrows] & WALL_TOP) {
 			XDrawLine(display, window, gc,
-				  mp->border_x + mp->sq_size_x * i,
-				  mp->border_y,
-				  mp->border_x + mp->sq_size_x * (i + 1),
-				  mp->border_y);
+				  mp->xb + mp->xs * i, mp->yb,
+				  mp->xb + mp->xs * (i + 1), mp->yb);
 		}
-		if ((mp->maze[i * mp->maze_size_y + mp->maze_size_y - 1] & WALL_BOTTOM)) {
+		if ((mp->maze[i * mp->nrows + mp->nrows - 1] & WALL_BOTTOM)) {
 			XDrawLine(display, window, gc,
-				  mp->border_x + mp->sq_size_x * i,
-			    mp->border_y + mp->sq_size_y * (mp->maze_size_y),
-				  mp->border_x + mp->sq_size_x * (i + 1),
-			   mp->border_y + mp->sq_size_y * (mp->maze_size_y));
+				  mp->xb + mp->xs * i,
+				  mp->yb + mp->ys * (mp->nrows),
+				  mp->xb + mp->xs * (i + 1),
+				  mp->yb + mp->ys * (mp->nrows));
 		}
 	}
-	for (j = 0; j < mp->maze_size_y; j++) {
-		if (mp->maze[(mp->maze_size_x - 1) * mp->maze_size_y + j] & WALL_RIGHT) {
+	for (j = 0; j < mp->nrows; j++) {
+		if (mp->maze[(mp->ncols - 1) * mp->nrows + j] & WALL_RIGHT) {
 			XDrawLine(display, window, gc,
-			      mp->border_x + mp->sq_size_x * mp->maze_size_x,
-				  mp->border_y + mp->sq_size_y * j,
-			      mp->border_x + mp->sq_size_x * mp->maze_size_x,
-				  mp->border_y + mp->sq_size_y * (j + 1));
+				  mp->xb + mp->xs * mp->ncols,
+				  mp->yb + mp->ys * j,
+				  mp->xb + mp->xs * mp->ncols,
+				  mp->yb + mp->ys * (j + 1));
 		}
 		if (mp->maze[j] & WALL_LEFT) {
 			XDrawLine(display, window, gc,
-				  mp->border_x,
-				  mp->border_y + mp->sq_size_y * j,
-				  mp->border_x,
-				  mp->border_y + mp->sq_size_y * (j + 1));
+				  mp->xb, mp->yb + mp->ys * j,
+				  mp->xb, mp->yb + mp->ys * (j + 1));
 		}
 	}
 
 	if (mp->logo_x != -1) {
 		XPutImage(display, window, gc, &logo,
 			  0, 0,
-			  mp->border_x + mp->sq_size_x * mp->logo_x +
-		      (mp->sq_size_x * mp->logo_size_x - logo.width + 1) / 2,
-			  mp->border_y + mp->sq_size_y * mp->logo_y +
-		     (mp->sq_size_y * mp->logo_size_y - logo.height + 1) / 2,
+			  mp->xb + mp->xs * mp->logo_x +
+			  (mp->xs * mp->logo_size_x - logo.width + 1) / 2,
+			  mp->yb + mp->ys * mp->logo_y +
+			  (mp->ys * mp->logo_size_y - logo.height + 1) / 2,
 			  logo.width, logo.height);
 	}
 	draw_solid_square(mi, gc, mp->start_x, mp->start_y, mp->start_dir);
@@ -542,31 +527,31 @@ draw_wall(ModeInfo * mi, int i, int j, int dir)
 	switch (dir) {
 		case 0:
 			XDrawLine(display, window, gc,
-				  mp->border_x + mp->sq_size_x * i,
-				  mp->border_y + mp->sq_size_y * j,
-				  mp->border_x + mp->sq_size_x * (i + 1),
-				  mp->border_y + mp->sq_size_y * j);
+				  mp->xb + mp->xs * i,
+				  mp->yb + mp->ys * j,
+				  mp->xb + mp->xs * (i + 1),
+				  mp->yb + mp->ys * j);
 			break;
 		case 1:
 			XDrawLine(display, window, gc,
-				  mp->border_x + mp->sq_size_x * (i + 1),
-				  mp->border_y + mp->sq_size_y * j,
-				  mp->border_x + mp->sq_size_x * (i + 1),
-				  mp->border_y + mp->sq_size_y * (j + 1));
+				  mp->xb + mp->xs * (i + 1),
+				  mp->yb + mp->ys * j,
+				  mp->xb + mp->xs * (i + 1),
+				  mp->yb + mp->ys * (j + 1));
 			break;
 		case 2:
 			XDrawLine(display, window, gc,
-				  mp->border_x + mp->sq_size_x * i,
-				  mp->border_y + mp->sq_size_y * (j + 1),
-				  mp->border_x + mp->sq_size_x * (i + 1),
-				  mp->border_y + mp->sq_size_y * (j + 1));
+				  mp->xb + mp->xs * i,
+				  mp->yb + mp->ys * (j + 1),
+				  mp->xb + mp->xs * (i + 1),
+				  mp->yb + mp->ys * (j + 1));
 			break;
 		case 3:
 			XDrawLine(display, window, gc,
-				  mp->border_x + mp->sq_size_x * i,
-				  mp->border_y + mp->sq_size_y * j,
-				  mp->border_x + mp->sq_size_x * i,
-				  mp->border_y + mp->sq_size_y * (j + 1));
+				  mp->xb + mp->xs * i,
+				  mp->yb + mp->ys * j,
+				  mp->xb + mp->xs * i,
+				  mp->yb + mp->ys * (j + 1));
 			break;
 	}
 }				/* end of draw_wall */
@@ -583,27 +568,27 @@ draw_solid_square(ModeInfo * mi, GC gc,
 	switch (dir) {
 		case 0:
 			XFillRectangle(display, window, gc,
-				       mp->border_x + 3 + mp->sq_size_x * i,
-				       mp->border_y - 3 + mp->sq_size_y * j,
-				       mp->sq_size_x - 6, mp->sq_size_y);
+				       mp->xb + 3 + mp->xs * i,
+				       mp->yb - 3 + mp->ys * j,
+				       mp->xs - 6, mp->ys);
 			break;
 		case 1:
 			XFillRectangle(display, window, gc,
-				       mp->border_x + 3 + mp->sq_size_x * i,
-				       mp->border_y + 3 + mp->sq_size_y * j,
-				       mp->sq_size_x, mp->sq_size_y - 6);
+				       mp->xb + 3 + mp->xs * i,
+				       mp->yb + 3 + mp->ys * j,
+				       mp->xs, mp->ys - 6);
 			break;
 		case 2:
 			XFillRectangle(display, window, gc,
-				       mp->border_x + 3 + mp->sq_size_x * i,
-				       mp->border_y + 3 + mp->sq_size_y * j,
-				       mp->sq_size_x - 6, mp->sq_size_y);
+				       mp->xb + 3 + mp->xs * i,
+				       mp->yb + 3 + mp->ys * j,
+				       mp->xs - 6, mp->ys);
 			break;
 		case 3:
 			XFillRectangle(display, window, gc,
-				       mp->border_x - 3 + mp->sq_size_x * i,
-				       mp->border_y + 3 + mp->sq_size_y * j,
-				       mp->sq_size_x, mp->sq_size_y - 6);
+				       mp->xb - 3 + mp->xs * i,
+				       mp->yb + 3 + mp->ys * j,
+				       mp->xs, mp->ys - 6);
 			break;
 	}
 
@@ -617,9 +602,9 @@ solve_maze(ModeInfo * mi)
 
 	if (!mp->solving) {
 		/* plug up the surrounding wall */
-		mp->maze[mp->start_x * mp->maze_size_y + mp->start_y] |=
+		mp->maze[mp->start_x * mp->nrows + mp->start_y] |=
 			(WALL_TOP >> mp->start_dir);
-		mp->maze[mp->end_x * mp->maze_size_y + mp->end_y] |=
+		mp->maze[mp->end_x * mp->nrows + mp->end_y] |=
 			(WALL_TOP >> mp->end_dir);
 
 		/* initialize search path */
@@ -643,7 +628,7 @@ solve_maze(ModeInfo * mi)
 				  (int) (mp->path[mp->current_path].x),
 				  (int) (mp->path[mp->current_path].y),
 				  (int) (mp->path[mp->current_path].dir));
-	} else if (!(mp->maze[mp->path[mp->current_path].x * mp->maze_size_y +
+	} else if (!(mp->maze[mp->path[mp->current_path].x * mp->nrows +
 			      mp->path[mp->current_path].y] &
 		     (WALL_TOP >> mp->path[mp->current_path].dir)) &&
 		   ((mp->current_path == 0) ||
@@ -652,7 +637,7 @@ solve_maze(ModeInfo * mi)
 				       2) % 4)))) {
 		enter_square(mi, mp->current_path);
 		mp->current_path++;
-		if (mp->maze[mp->path[mp->current_path].x * mp->maze_size_y +
+		if (mp->maze[mp->path[mp->current_path].x * mp->nrows +
 			     mp->path[mp->current_path].y] & START_SQUARE) {
 			mp->solving = 0;
 			return;
@@ -806,15 +791,15 @@ refresh_maze(ModeInfo * mi)
 		mp->sqnum = 0;
 		mp->cur_sq_x = mp->start_x;
 		mp->cur_sq_y = mp->start_y;
-		mp->maze[mp->start_x * mp->maze_size_y + mp->start_y] |= START_SQUARE;
-		mp->maze[mp->start_x * mp->maze_size_y + mp->start_y] |=
+		mp->maze[mp->start_x * mp->nrows + mp->start_y] |= START_SQUARE;
+		mp->maze[mp->start_x * mp->nrows + mp->start_y] |=
 			(DOOR_IN_TOP >> mp->start_dir);
-		mp->maze[mp->start_x * mp->maze_size_y + mp->start_y] &=
+		mp->maze[mp->start_x * mp->nrows + mp->start_y] &=
 			~(WALL_TOP >> mp->start_dir);
-		mp->maze[mp->end_x * mp->maze_size_y + mp->end_y] |= END_SQUARE;
-		mp->maze[mp->end_x * mp->maze_size_y + mp->end_y] |=
+		mp->maze[mp->end_x * mp->nrows + mp->end_y] |= END_SQUARE;
+		mp->maze[mp->end_x * mp->nrows + mp->end_y] |=
 			(DOOR_OUT_TOP >> mp->end_dir);
-		mp->maze[mp->end_x * mp->maze_size_y + mp->end_y] &=
+		mp->maze[mp->end_x * mp->nrows + mp->end_y] &=
 			~(WALL_TOP >> mp->end_dir);
 		draw_maze_border(mi);
 		draw_maze_walls(mi);
