@@ -1,14 +1,14 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)mode.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)mode.c	4.02 97/04/01 xlockmore";
 
 #endif
-
 /*-
  * mode.c - Modes for xlock. 
  *
  * See xlock.c for copying information.
  *
  * Revision History:
+ * 01-Apr-97: Fixed memory leak in XStringListToTextProperty.
  * 23-Feb-96: Extensive revision to implement new mode hooks and stuff
  *		Ron Hitchens <ron@idiom.com>
  * 04-Sep-95: Moved over from mode.h (previously resource.h) with new
@@ -28,44 +28,18 @@ static const char sccsid[] = "@(#)mode.c	4.00 97/01/01 xlockmore";
 
 LockStruct  LockProcs[] =
 {
-	{"ant", init_ant, draw_ant, release_ant,
-	 refresh_ant, init_ant, NULL, &ant_opts,
-	 1000, -3, 40000, -7, 1.0,
-	 "Shows Langton's and Turk's generalized ants", 0, NULL},
-	{"ball", init_ball, draw_ball, release_ball,
-	 refresh_ball, init_ball, NULL, &ball_opts,
-	 10000, 10, 20, -100, 1.0,
-	 "Shows bouncing balls", 0, NULL},
-	{"bat", init_bat, draw_bat, release_bat,
-	 refresh_bat, init_bat, NULL, &bat_opts,
-	 100000, -8, 1, 0, 1.0,
-	 "Shows bouncing flying bats", 0, NULL},
 	{"blot", init_blot, draw_blot, release_blot,
 	 refresh_blot, init_blot, NULL, &blot_opts,
-	 100000, 6, 30, 1, 0.4,
+	 200000, 6, 30, 1, 0.4,
 	 "Shows Rorschach's ink blot test", 0, NULL},
 	{"bouboule", init_bouboule, draw_bouboule, release_bouboule,
 	 refresh_bouboule, init_bouboule, NULL, &bouboule_opts,
 	 10000, 100, 1, 15, 1.0,
 	 "Shows Mimi's bouboule of moving stars", 0, NULL},
-	{"bounce", init_bounce, draw_bounce, release_bounce,
-	 refresh_bounce, init_bounce, NULL, &bounce_opts,
-	 10000, -10, 1, 0, 1.0,
-	 "Shows bouncing footballs", 0, NULL},
-	{"braid", init_braid, draw_braid, release_braid,
-	 refresh_braid, init_braid, NULL, &braid_opts,
-	 1000, 15, 100, 1, 1.0,
-	 "Shows random braids and knots", 0, NULL},
 	{"bug", init_bug, draw_bug, release_bug,
 	 refresh_bug, init_bug, NULL, &bug_opts,
 	 75000, 10, 32767, -4, 1.0,
 	 "Shows Palmiter's bug evolution and garden of Eden", 0, NULL},
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-	{"cartoon", init_cartoon, draw_cartoon, release_cartoon,
-	 NULL, init_cartoon, NULL, &cartoon_opts,
-	 10000, 1, 1, 1, 1.0,
-	 "Shows bouncing cartoons", 0, NULL},
-#endif
 	{"clock", init_clock, draw_clock, release_clock,
 	 refresh_clock, init_clock, NULL, &clock_opts,
 	 100000, -16, 200, -200, 1.0,
@@ -80,12 +54,8 @@ LockStruct  LockProcs[] =
 	 "Shows a floating digital clock", 0, NULL},
 	{"demon", init_demon, draw_demon, release_demon,
 	 refresh_demon, init_demon, NULL, &demon_opts,
-	 50000, 16, 1000, -7, 1.0,
+	 50000, 0, 1000, -7, 1.0,
 	 "Shows Griffeath's cellular automata", 0, NULL},
-	{"drift", init_drift, draw_drift, release_drift,
-	 refresh_drift, init_drift, NULL, &drift_opts,
-	 10000, 30, 1, 1, 1.0,
-	 "Shows cosmic drifting flame fractals", 0, NULL},
 	{"eyes", init_eyes, draw_eyes, release_eyes,
 	 refresh_eyes, init_eyes, NULL, &eyes_opts,
 	 20000, -8, 5, 1, 1.0,
@@ -98,20 +68,10 @@ LockStruct  LockProcs[] =
 	 refresh_flame, init_flame, NULL, &flame_opts,
 	 750000, 20, 10000, 1, 1.0,
 	 "Shows cosmic flame fractals", 0, NULL},
-	{"forest", init_forest, draw_forest, release_forest,
-	 refresh_forest, init_forest, NULL, &forest_opts,
-	 400000, 100, 200, 1, 1.0,
-	 "Shows binary trees of a fractal forest", 0, NULL},
-	{"galaxy", init_galaxy, draw_galaxy, release_galaxy,
-	 refresh_galaxy, init_galaxy, NULL, &galaxy_opts,
-	 100, -5, 250, -2, 1.0,
-	 "Shows crashing spiral galaxies", 0, NULL},
-#ifdef USE_GL
-	{"gear", init_gear, draw_gear, release_gear,
-	 draw_gear, init_gear, NULL, &gear_opts,
-	 100, 1, 1, 1, 1.0,
-	 "Shows GL's gears", 0, NULL},
-#endif
+	{"fract", init_fract, draw_fract, release_fract,
+	 refresh_fract, init_fract, NULL, &fract_opts,
+	 200000, 1, 1, 1, 1.0,
+	 "Shows fractals", 0, NULL},
 	{"geometry", init_geometry, draw_geometry, release_geometry,
 	 refresh_geometry, init_geometry, NULL, &geometry_opts,
 	 10000, -10, 1, 1, 1.0,
@@ -124,22 +84,18 @@ LockStruct  LockProcs[] =
 	 refresh_helix, init_helix, NULL, &helix_opts,
 	 25000, 1, 100, 1, 1.0,
 	 "Shows string art", 0, NULL},
-	{"hop", init_hop, draw_hop, release_hop,
-	 refresh_hop, init_hop, NULL, &hop_opts,
-	 10000, 1000, 2500, 1, 1.0,
-	 "Shows real plane iterated fractals", 0, NULL},
 	{"hyper", init_hyper, draw_hyper, release_hyper,
 	 refresh_hyper, init_hyper, NULL, &hyper_opts,
 	 10000, 1, 300, 1, 1.0,
 	 "Shows a spinning tesseract in 4D space", 0, NULL},
+	{"ico", init_ico, draw_ico, release_ico,
+	 refresh_ico, change_ico, NULL, &ico_opts,
+	 100000, 0, 400, 0, 1.0,
+	 "Shows a bouncing polyhedra", 0, NULL},
 	{"image", init_image, draw_image, release_image,
 	 refresh_image, init_image, NULL, &image_opts,
 	 2000000, -10, 1, 1, 1.0,
 	 "Shows randomly appearing logos", 0, NULL},
-	{"julia", init_julia, draw_julia, release_julia,
-	 refresh_julia, init_julia, NULL, &julia_opts,
-	 10000, 1000, 20, 1, 1.0,
-	 "Shows the Julia set", 0, NULL},
 	{"kaleid", init_kaleid, draw_kaleid, release_kaleid,
 	 refresh_kaleid, init_kaleid, NULL, &kaleid_opts,
 	 20000, 1, 700, 1, 1.0,
@@ -164,6 +120,10 @@ LockStruct  LockProcs[] =
 	 refresh_lightning, init_lightning, NULL, &lightning_opts,
 	 10000, 1, 1, 1, 0.6,
 	 "Shows Keith's fractal lightning bolts", 0, NULL},
+	{"lisa", init_lisa, draw_lisa, release_lisa, refresh_lisa,
+	 change_lisa, NULL, &lisa_opts,
+	 25000, 1, 256, -1, 1.0,
+	 "Shows animated lisajous loops", 0, NULL},
 	{"lissie", init_lissie, draw_lissie, release_lissie,
 	 refresh_lissie, init_lissie, NULL, &lissie_opts,
 	 10000, -2, 2000, -200, 0.6,
@@ -176,22 +136,10 @@ LockStruct  LockProcs[] =
 	 refresh_marquee, init_marquee, NULL, &marquee_opts,
 	 100000, 1, 1, 1, 1.0,
 	 "Shows messages", 0, NULL},
-	{"maze", init_maze, draw_maze, release_maze,
-	 refresh_maze, init_maze, NULL, &maze_opts,
-	 1000, 1, 300, -40, 1.0,
-	 "Shows a random maze and a depth first search solution", 0, NULL},
-	{"mountain", init_mountain, draw_mountain, release_mountain,
-	 refresh_mountain, init_mountain, NULL, &mountain_opts,
-	 1000, 30, 100, 1, 1.0,
-	 "Shows Papo's mountain range", 0, NULL},
 	{"nose", init_nose, draw_nose, release_nose,
 	 refresh_nose, init_nose, NULL, &nose_opts,
 	 100000, 1, 1, 1, 1.0,
     "Shows a man with a big nose runs around spewing out messages", 0, NULL},
-	{"pacman", init_pacman, draw_pacman, release_pacman,
-	 refresh_pacman, init_pacman, NULL, &pacman_opts,
-	 100000, 10, 1, 0, 1.0,
-	 "Shows Pacman(tm)", 0, NULL},
 	{"penrose", init_penrose, draw_penrose, release_penrose,
 	 init_penrose, init_penrose, NULL, &penrose_opts,
 	 10000, 1, 1, -40, 1.0,
@@ -224,18 +172,6 @@ LockStruct  LockProcs[] =
 	 refresh_shape, init_shape, NULL, &shape_opts,
 	 10000, 100, 256, 1, 1.0,
 	 "Shows stippled rectangles, ellipses, and triangles", 0, NULL},
-	{"slip", init_slip, draw_slip, release_slip,
-	 init_slip, init_slip, NULL, &slip_opts,
-	 50000, 35, 50, 1, 1.0,
-	 "Shows slipping blits", 0, NULL},
-	{"sphere", init_sphere, draw_sphere, release_sphere,
-	 refresh_sphere, init_sphere, NULL, &sphere_opts,
-	 10000, 1, 20, 0, 1.0,
-	 "Shows a bunch of shaded spheres", 0, NULL},
-	{"spiral", init_spiral, draw_spiral, release_spiral,
-	 refresh_spiral, init_spiral, NULL, &spiral_opts,
-	 5000, -40, 350, 1, 1.0,
-	 "Shows helixes of dots", 0, NULL},
 	{"spline", init_spline, draw_spline, release_spline,
 	 refresh_spline, init_spline, NULL, &spline_opts,
 	 30000, -6, 2048, 1, 0.4,
@@ -248,26 +184,22 @@ LockStruct  LockProcs[] =
 	 refresh_swarm, init_swarm, NULL, &swarm_opts,
 	 10000, 100, 1, 1, 1.0,
 	 "Shows a swarm of bees following a wasp", 0, NULL},
-	{"swirl", init_swirl, draw_swirl, release_swirl,
-	 refresh_swirl, init_swirl, NULL, &swirl_opts,
-	 5000, 5, 1, 1, 1.0,
-	 "Shows animated swirling patterns", 0, NULL},
 	{"tri", init_tri, draw_tri, release_tri,
 	 refresh_tri, init_tri, NULL, &tri_opts,
-	 40000, 2000, 100, 1, 1.0,
+	 400000, 2000, 100, 1, 1.0,
 	 "Shows a Sierpinski's triangle", 0, NULL},
 	{"triangle", init_triangle, draw_triangle, release_triangle,
 	 refresh_triangle, init_triangle, NULL, &triangle_opts,
 	 10000, 1, 1, 1, 1.0,
 	 "Shows a triangle mountain range", 0, NULL},
+	{"tube", init_tube, draw_tube, release_tube,
+	 NULL, init_tube, NULL, &tube_opts,
+	 25000, 1, 20000, -200, 1.0,
+	 "Shows an animated tube", 0, NULL},
 	{"turtle", init_turtle, draw_turtle, release_turtle,
 	 init_turtle, init_turtle, NULL, &turtle_opts,
 	 1000000, 1, 20, 1, 1.0,
 	 "Shows turtle fractals", 0, NULL},
-	{"wator", init_wator, draw_wator, release_wator,
-	 refresh_wator, init_wator, NULL, &wator_opts,
-	 750000, 1, 32767, 0, 1.0,
-	 "Shows Dewdney's Water-Torus planet of fish and sharks", 0, NULL},
 	{"wire", init_wire, draw_wire, release_wire,
 	 refresh_wire, init_wire, NULL, &wire_opts,
 	 500000, 1000, 150, -8, 1.0,
@@ -280,26 +212,126 @@ LockStruct  LockProcs[] =
 	 refresh_worm, init_worm, NULL, &worm_opts,
 	 17000, -20, 10, -3, 1.0,
 	 "Shows wiggly worms", 0, NULL},
+/* NORMALLY HIGH CPU USAGE MODES (kind of subjective) */
+	{"ant", init_ant, draw_ant, release_ant,
+	 refresh_ant, init_ant, NULL, &ant_opts,
+	 1000, -3, 40000, -7, 1.0,
+	 "Shows Langton's and Turk's generalized ants", 0, NULL},
+	{"ball", init_ball, draw_ball, release_ball,
+	 refresh_ball, init_ball, NULL, &ball_opts,
+	 10000, 10, 20, -100, 1.0,
+	 "Shows bouncing balls", 0, NULL},
+	{"bat", init_bat, draw_bat, release_bat,
+	 refresh_bat, init_bat, NULL, &bat_opts,
+	 100000, -8, 1, 0, 1.0,
+	 "Shows bouncing flying bats", 0, NULL},
+	{"bounce", init_bounce, draw_bounce, release_bounce,
+	 refresh_bounce, init_bounce, NULL, &bounce_opts,
+	 10000, -10, 1, 0, 1.0,
+	 "Shows bouncing footballs", 0, NULL},
+	{"braid", init_braid, draw_braid, release_braid,
+	 refresh_braid, init_braid, NULL, &braid_opts,
+	 1000, 15, 100, 1, 1.0,
+	 "Shows random braids and knots", 0, NULL},
+	{"drift", init_drift, draw_drift, release_drift,
+	 refresh_drift, init_drift, NULL, &drift_opts,
+	 10000, 30, 1, 1, 1.0,
+	 "Shows cosmic drifting flame fractals", 0, NULL},
+	{"galaxy", init_galaxy, draw_galaxy, release_galaxy,
+	 refresh_galaxy, init_galaxy, NULL, &galaxy_opts,
+	 100, -5, 250, -3, 1.0,
+	 "Shows crashing spiral galaxies", 0, NULL},
+	{"forest", init_forest, draw_forest, release_forest,
+	 refresh_forest, init_forest, NULL, &forest_opts,
+	 400000, 100, 200, 1, 1.0,
+	 "Shows binary trees of a fractal forest", 0, NULL},
+	{"hop", init_hop, draw_hop, release_hop,
+	 refresh_hop, init_hop, NULL, &hop_opts,
+	 10000, 1000, 2500, 1, 1.0,
+	 "Shows real plane iterated fractals", 0, NULL},
+	{"ifs", init_ifs, draw_ifs, release_ifs,
+	 draw_ifs, init_ifs, NULL, &ifs_opts,
+	 1000, 1, 1, 1, 1.0,
+	 "Shows a modified iterated function system", 0, NULL},
+	{"julia", init_julia, draw_julia, release_julia,
+	 refresh_julia, init_julia, NULL, &julia_opts,
+	 10000, 1000, 20, 1, 1.0,
+	 "Shows the Julia set", 0, NULL},
+	{"maze", init_maze, draw_maze, release_maze,
+	 refresh_maze, init_maze, NULL, &maze_opts,
+	 1000, 1, 300, -40, 1.0,
+	 "Shows a random maze and a depth first search solution", 0, NULL},
+	{"mountain", init_mountain, draw_mountain, release_mountain,
+	 refresh_mountain, init_mountain, NULL, &mountain_opts,
+	 1000, 30, 100, 1, 1.0,
+	 "Shows Papo's mountain range", 0, NULL},
+	{"pacman", init_pacman, draw_pacman, release_pacman,
+	 refresh_pacman, init_pacman, NULL, &pacman_opts,
+	 100000, 10, 1, 0, 1.0,
+	 "Shows Pacman(tm)", 0, NULL},
+	{"slip", init_slip, draw_slip, release_slip,
+	 init_slip, init_slip, NULL, &slip_opts,
+	 50000, 35, 50, 1, 1.0,
+	 "Shows slipping blits", 0, NULL},
+	{"sphere", init_sphere, draw_sphere, release_sphere,
+	 refresh_sphere, init_sphere, NULL, &sphere_opts,
+	 10000, 1, 20, 0, 1.0,
+	 "Shows a bunch of shaded spheres", 0, NULL},
+	{"spiral", init_spiral, draw_spiral, release_spiral,
+	 refresh_spiral, init_spiral, NULL, &spiral_opts,
+	 5000, -40, 350, 1, 1.0,
+	 "Shows helixes of dots", 0, NULL},
+	{"strange", init_strange, draw_strange, release_strange,
+	 draw_strange, init_strange, NULL, &strange_opts,
+	 1000, 1, 1, 1, 1.0,
+	 "Shows strange attractors", 0, NULL},
+	{"swirl", init_swirl, draw_swirl, release_swirl,
+	 refresh_swirl, init_swirl, NULL, &swirl_opts,
+	 5000, 5, 1, 1, 1.0,
+	 "Shows animated swirling patterns", 0, NULL},
+	{"wator", init_wator, draw_wator, release_wator,
+	 refresh_wator, init_wator, NULL, &wator_opts,
+	 750000, 1, 32767, 0, 1.0,
+	 "Shows Dewdney's Water-Torus planet of fish and sharks", 0, NULL},
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+	{"cartoon", init_cartoon, draw_cartoon, release_cartoon,
+	 NULL, init_cartoon, NULL, &cartoon_opts,
+	 10000, 1, 1, 1, 1.0,
+	 "Shows bouncing cartoons", 0, NULL},
+#endif
+#ifdef USE_GL
+	{"gears", init_gears, draw_gears, release_gears,
+	 draw_gears, init_gears, NULL, &gears_opts,
+	 100, 1, 1, 1, 1.0,
+	 "Shows GL's gears", 0, NULL},
+	{"morph3d", init_morph3d, draw_morph3d, release_morph3d,
+	 draw_morph3d, change_morph3d, NULL, &morph3d_opts,
+	 100, 0, 1, 1, 1.0,
+	 "Shows GL morphing polyhedra", 0, NULL},
+	{"pipes", init_pipes, draw_pipes, release_pipes,
+	 draw_pipes, change_pipes, NULL, &pipes_opts,
+	 100, 0, 5, 500, 1.0,
+	 "Shows a selfbuilding pipe system", 0, NULL},
+	{"superquadrics", init_superquadrics, draw_superquadrics, release_superquadrics,
+	 refresh_superquadrics, init_superquadrics, NULL, &superquadrics_opts,
+	 100, 25, 40, 1, 1.0,
+	 "Shows 3D mathematical shapes", 0, NULL},
+/* TIME EXPENSIVE MODE BUT REALLY COOL */
+#ifdef USE_SPROINGIES
+	{"sproingies", init_sproingies, draw_sproingies, release_sproingies,
+	 refresh_sproingies, init_sproingies, NULL, &sproingies_opts,
+	 10, 5, 0, 0, 1.0,
+  "Shows Sproingies!  Nontoxic.  Safe for pets and small children", 0, NULL},
+#endif
+#endif
+/* NEW POSSIBLY, BUGGY MODES */
 #ifdef USE_HACKERS
 	{"fadeplot", init_fadeplot, draw_fadeplot, release_fadeplot,
 	 refresh_fadeplot, init_fadeplot, NULL, &fadeplot_opts,
 	 30000, 1500, 1, 1, 1.0,
 	 "Shows fadeplot", 0, NULL},
-#ifdef USE_GL
-	{"morph3d", init_morph3d, draw_morph3d, release_morph3d,
-	 draw_morph3d, change_morph3d, NULL, &morph3d_opts,
-	 100, 1, 1, 1, 1.0,
-	 "Shows GL morphing polyhedra", 0, NULL},
-#endif
-  {"trig", init_trig, draw_trig, release_trig,
-   NULL, init_trig, NULL, &trig_opts,
-   30000, 100, 64, 0, 1.0,
-   "Shows trigonometric patterns", 0, NULL},
-  {"tube", init_tube, draw_tube, release_tube,
-   NULL, init_tube, NULL, &tube_opts,
-   5000, 5, 20, 0, 1.0,
-   "Shows an animated tube", 0, NULL},
 #endif
+/* SPECIAL MODES */
 	{"blank", init_blank, draw_blank, release_blank,
 	 refresh_blank, init_blank, NULL, &blank_opts,
 	 3000000, 1, 1, 1, 1.0,
@@ -309,7 +341,7 @@ LockStruct  LockProcs[] =
 	 refresh_bomb, change_bomb, NULL, &bomb_opts,
 	 100000, 10, 20, 1, 1.0,
 	 "Shows a bomb and will autologout after a time", 0, NULL},
-	{"random", init_random, draw_random, NULL,
+	{"random", init_random, draw_random, release_random,
 	 refresh_random, change_random, NULL, &random_opts,
 	 1, 1, 1, 1, 1.0,
 	 "Shows a random mode from above except blank and bomb", 0, NULL},
@@ -344,12 +376,14 @@ set_window_title(ModeInfo * mi)
 	XTextProperty prop;
 	char        buf[512];
 	char       *ptr = buf;
+	unsigned int status;
 
 	(void) sprintf(buf, "%s: %s", MI_NAME(mi), MI_DESC(mi));
-/* PURIFY 4.0 on SunOS4 reports a memory leak on the next line. I am not sure
-   if its a true leak. */
-	XStringListToTextProperty(&ptr, 1, &prop);
-	XSetWMName(MI_DISPLAY(mi), MI_WINDOW(mi), &prop);
+	status = XStringListToTextProperty(&ptr, 1, &prop);
+	if (status != 0) {
+		XSetWMName(MI_DISPLAY(mi), MI_WINDOW(mi), &prop);
+		XFree((void *) prop.value);
+	}
 }
 
 /* -------------------------------------------------------------------- */
@@ -361,14 +395,16 @@ set_window_title(ModeInfo * mi)
  *      resources.
  */
 
-static void
+void
 call_release_hook(LockStruct * ls, ModeInfo * mi)
 {
 	if (ls == NULL) {
 		return;
 	}
 	MI_LOCKSTRUCT(mi) = ls;
-
+	if (MI_WIN_IS_DEBUG(mi))
+		printf("releasing last=%s ls=%s in mode\n",
+		       last_initted_mode->cmdline_arg, ls->cmdline_arg);
 	if (ls->release_hook != NULL) {
 		ls->release_hook(mi);
 	}
@@ -410,11 +446,12 @@ call_init_hook(LockStruct * ls, ModeInfo * mi)
 			ls = default_mode;
 		}
 	}
+	if (MI_WIN_IS_DEBUG(mi))
+		printf("initing ls=%s in mode\n", ls->cmdline_arg);
 	if (ls != last_initted_mode) {
 		call_release_hook(last_initted_mode, mi);
 	}
 	MI_LOCKSTRUCT(mi) = ls;
-
 	set_window_title(mi);
 
 	ls->init_hook(mi);
