@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)nose.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)nose.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -13,7 +13,7 @@ static char sccsid[] = "@(#)nose.c	3.11 96/09/20 xlockmore";
  * Revision History:
  * 27-Feb-96: Added new ModeInfo arg to init and callback hooks.  Removed
  *		references to onepause, now uses MI_PAUSE(mi) interface.
- *		Ron Hitchens <ronh@utw.com>
+ *		Ron Hitchens <ron@idiom.com>
  * 10-Oct-95: A better way of handling fortunes from a file, thanks to
  *            Jouk Jansen <joukj@alpha.chem.uva.nl>.
  * 21-Sep-95: font option added, debugged for multiscreens
@@ -67,7 +67,7 @@ extern char *messagefile;
 extern char *message;
 
 extern XFontStruct *getFont(Display * display);
-extern char *getWords(void);
+extern char *getWords(int screen);
 extern int  isRibbon(void);
 
 static XImage bimages[] =
@@ -83,7 +83,6 @@ static XImage bimages[] =
 };
 
 typedef struct {
-	int         done;
 	int         xs, ys;
 	int         width, height;
 	GC          text_fg_gc, text_bg_gc;
@@ -255,7 +254,7 @@ think(ModeInfo * mi)
 	if (LRAND() & 1)
 		walk(mi, FRONT);
 	if (LRAND() & 1) {
-		np->words = getWords();
+		np->words = getWords(MI_SCREEN(mi));
 		return 1;
 	}
 	return 0;
@@ -423,9 +422,8 @@ init_nose(ModeInfo * mi)
 			bimages[init_images].height = nose_front_height;
 			bimages[init_images].bytes_per_line = (nose_front_width + 7) / 8;
 		}
-	np->words = getWords();
-	if (!np->done) {
-		np->done = 1;
+	np->words = getWords(MI_SCREEN(mi));
+	if (np->text_fg_gc == NULL) {
 		gcv.font = mode_font->fid;
 		XSetFont(display, gc, mode_font->fid);
 		gcv.graphics_exposures = False;
@@ -482,9 +480,9 @@ release_nose(ModeInfo * mi)
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			nosestruct *np = &noses[screen];
 
-			if (!np->text_fg_gc)
+			if (np->text_fg_gc != NULL)
 				XFreeGC(MI_DISPLAY(mi), np->text_fg_gc);
-			if (!np->text_bg_gc)
+			if (np->text_bg_gc != NULL)
 				XFreeGC(MI_DISPLAY(mi), np->text_bg_gc);
 
 		}
