@@ -1,6 +1,6 @@
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)passwd.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)passwd.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -193,7 +193,7 @@ static char uai_encrypt, root_encrypt;
 struct ascid {
 	short       len;
 	char        dtype;
-	char        class;
+	char        c_class;
 	char       *addr;
 };
 
@@ -246,10 +246,10 @@ static int  krb_check_password(struct passwd *, char *);
 
 #endif
 
-#if 0
+#if defined(__cplusplus) || defined(c_plusplus)
 extern char *crypt(char *, char *);
-extern char *crypt(const char *, const char *);
 
+/* extern char *crypt(const char *, const char *); */
 #endif
 
 #if (defined( USE_XLOCKRC ) || (!defined( OSF1_ENH_SEC ) &&  !defined( HP_PASSWDETC ) && !defined( VMS )))
@@ -416,7 +416,7 @@ getCryptedUserPasswd(void)
 
 	username.len = strlen(user);
 	username.dtype = 0;
-	username.class = 0;
+	username.c_class = 0;
 	username.addr = user;
 
 	sys$getuai(0, 0, &username, &il, 0, 0, 0);
@@ -513,7 +513,7 @@ getCryptedRootPasswd(void)
 
 	rootuser.len = strlen(root);
 	rootuser.dtype = 0;
-	rootuser.class = 0;
+	rootuser.c_class = 0;
 	rootuser.addr = root;
 
 	sys$getuai(0, 0, &rootuser, &il, 0, 0, 0);
@@ -615,7 +615,7 @@ checkPasswd(char *buffer)
 
 	password.len = strlen(buffer);
 	password.dtype = 0;
-	password.class = 0;
+	password.c_class = 0;
 	password.addr = buffer;
 
 	str$upcase(&password, &password);
