@@ -144,6 +144,8 @@ typedef unsigned char vertex_type_c;
  * node.  A vertex count is maintained to make it easier to pick
  * vertices randomly.
  */
+typedef struct forced_node forced_node_c;
+
 typedef struct fringe_node {
 	struct fringe_node *prev;
 	struct fringe_node *next;
@@ -184,11 +186,11 @@ typedef struct {
  * applicable rule.  forced_sides are specified using S_LEFT and
  * S_RIGHT as if looking at the untiled region from the vertex.
  */
-typedef struct forced_node {
+struct forced_node {
 	fringe_node_c *vertex;
 	unsigned    forced_sides;
 	struct forced_node *next;
-} forced_node_c;
+};
 
 typedef struct {
 	forced_node_c *first;
