@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)petal.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)petal.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -29,7 +29,6 @@ static char sccsid[] = "@(#)petal.c	3.11 96/09/20 xlockmore";
  */
 
 #include "xlock.h"
-#include <math.h>
 #define TWOPI (2.0*M_PI)
 
 /* 
