@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)puzzle.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)puzzle.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -25,31 +25,30 @@ static char sccsid[] = "@(#)puzzle.c	3.11 96/09/20 xlockmore";
  * if you have a picture on your background.
  */
 
-#include <time.h>
-#include <math.h>
 #include "xlock.h"
-#include "ras.h"
+#include <time.h>
 
 #define PUZZLE_WIDTH   image_width
 #define PUZZLE_HEIGHT    image_height
 #define PUZZLE_BITS    image_bits
-#define PUZZLE_NAME    image_name
-
+#define NOWAY 255
 #include "puzzle.xbm"
 
-#ifdef HAS_XPM
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+#define PUZZLE_NAME    image_name
 #include "puzzle.xpm"
-#else
-static char **image_name = NULL;
-
 #endif
 
 ModeSpecOpt puzzle_opts =
 {0, NULL, 0, NULL, NULL};
 
 extern void getImage(ModeInfo * mi, XImage ** logo,
-		     int width, int height, unsigned char *bits, char **name,
-		     int *graphics_format, Bool * newcolormap);
+		     int width, int height, unsigned char *bits,
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+		     char **name,
+#endif
+		     int *graphics_format, Colormap * newcolormap,
+		     unsigned long *blackpix, unsigned long *whitepix);
 extern void destroyImage(XImage * logo, int graphics_format);
 
 typedef struct {
@@ -58,7 +57,7 @@ typedef struct {
 	XPoint      count, boxsize, windowsize;
 	XPoint      usablewindow, offsetwindow;
 	XPoint      randompos, randpos;
-	unsigned long black;
+	unsigned long black, white;
 	int         row, col, nextrow, nextcol, nextbox;
 	int         incrementOfMove;
 	int         lastbox;
@@ -81,10 +80,9 @@ typedef struct {
 	int         ascent, fontWidth, fontHeight;
 #else
 	XImage     *logo;
-	Bool        newcolormap;
 	GC          backGC;
-	int         graphics_format;
 	Colormap    cm;
+	int         graphics_format;
 #endif
 } puzzlestruct;
 
@@ -195,7 +193,7 @@ setupmove(ModeInfo * mi)
 	switch (pp->nextbox) {
 		case 0:
 			if ((pp->row == 0) || (pp->lastbox == 2))
-				pp->nextbox = 99;
+				pp->nextbox = NOWAY;
 			else {
 				pp->nextrow = pp->row - 1;
 				pp->nextcol = pp->col;
@@ -203,7 +201,7 @@ setupmove(ModeInfo * mi)
 			break;
 		case 1:
 			if ((pp->col == pp->count.x - 1) || (pp->lastbox == 3))
-				pp->nextbox = 99;
+				pp->nextbox = NOWAY;
 			else {
 				pp->nextrow = pp->row;
 				pp->nextcol = pp->col + 1;
@@ -211,7 +209,7 @@ setupmove(ModeInfo * mi)
 			break;
 		case 2:
 			if ((pp->row == pp->count.y - 1) || (pp->lastbox == 0))
-				pp->nextbox = 99;
+				pp->nextbox = NOWAY;
 			else {
 				pp->nextrow = pp->row + 1;
 				pp->nextcol = pp->col;
@@ -219,18 +217,18 @@ setupmove(ModeInfo * mi)
 			break;
 		case 3:
 			if ((pp->col == 0) || (pp->lastbox == 1))
-				pp->nextbox = 99;
+				pp->nextbox = NOWAY;
 			else {
 				pp->nextrow = pp->row;
 				pp->nextcol = pp->col - 1;
 			}
 			break;
 		default:
-			pp->nextbox = 99;
+			pp->nextbox = NOWAY;
 			break;
 	}
 
-	if (pp->nextbox != 99) {
+	if (pp->nextbox != NOWAY) {
 		pp->lastbox = pp->nextbox;
 		return True;
 	} else
@@ -249,28 +247,51 @@ setupmovedelta(ModeInfo * mi)
 	/* There is a big problem here when used with -inwindow , possible solutions
 	 * seem to expose more levels of complication when implemented.
 	 */
-	if (pp->graphics_format < IS_XPM)
+	/* UnmapNotify VisibilityPartiallyObscured)) */
+	if (pp->box) {
+		XDestroyImage(pp->box);
+		pp->box = NULL;
+	} {
 		/* This will bomb out easily if using -inwindow or -debug .
 		 * X Error of failed request:  BadMatch (invalid parameter attributes)
-		 * Taking that into account I could not figure out how to get XSubImage
-		 * to work here.
 		 */
-		pp->box = XGetImage(display, window,
+		XWindowAttributes wa;
+
+		(void) XGetWindowAttributes(display, window, &wa);
+
+#if 0
+		if (!(wa.all_event_masks & NoExpose))
+#endif
+#if 0
+			do {
+				(void) XGetWindowAttributes(display, window, &wa);
+			}
+			while (wa.map_state != IsViewable);
+#endif
+		if (wa.map_state == IsViewable)
+/* Next line bombs out with BadMatch if moved off screen. */
+			pp->box = XGetImage(display, window,
 			   pp->nextcol * pp->boxsize.x + pp->randompos.x + 1,
 			   pp->nextrow * pp->boxsize.y + pp->randompos.y + 1,
-				    pp->boxsize.x - 2, pp->boxsize.y - 2,
-				    AllPlanes, ZPixmap);
-	else
+					pp->boxsize.x - 2, pp->boxsize.y - 2,
+					    AllPlanes, ZPixmap);
+	}
+#if 0
+	/* Current state of the puzzle has to be known for this to work
+	 * (pp->nextrow) and (pp->nextcol) are the correct starting values, but
+	 * as the puzzle progresses it is not correct.
+	 */
+	if (pp->graphics_format >= IS_XPM)
 		/* This can only be used with a fully initialized image where logo is
 		 * initialized directly or indirectly with XCreateImage .
 		 * It also has the disadvantage that it uses XGetPixel and XPutPixel
 		 * internally, so it could be slow.
 		 */
 		pp->box = XSubImage(pp->logo,
-				    pp->nextcol * pp->boxsize.x + 1,
-				    pp->nextrow * pp->boxsize.y + 1,
+				    (pp->nextrow) * pp->boxsize.x + 1,
+				    (pp->nextcol) * pp->boxsize.y + 1,
 				    pp->boxsize.x - 2, pp->boxsize.y - 2);
-
+#endif
 	if (pp->nextcol > pp->col) {
 		pp->cfactor = -1;
 		pp->cbs = pp->boxsize.x;
@@ -326,17 +347,19 @@ wrapupmovedelta(ModeInfo * mi)
 {
 	puzzlestruct *pp = &puzzs[MI_SCREEN(mi)];
 
-	XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box, 0, 0,
-		  pp->col * pp->boxsize.x + pp->randompos.x + 1,
-		  pp->row * pp->boxsize.y + pp->randompos.y + 1,
-		  pp->boxsize.x - 2, pp->boxsize.y - 2);
-	XFlush(MI_DISPLAY(mi));
+	if (pp->box) {
+		XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box, 0, 0,
+			  pp->col * pp->boxsize.x + pp->randompos.x + 1,
+			  pp->row * pp->boxsize.y + pp->randompos.y + 1,
+			  pp->boxsize.x - 2, pp->boxsize.y - 2);
+		XFlush(MI_DISPLAY(mi));
 
-	pp->row = pp->nextrow;
-	pp->col = pp->nextcol;
+		pp->row = pp->nextrow;
+		pp->col = pp->nextcol;
 
-	XDestroyImage(pp->box);
-	pp->box = NULL;
+		XDestroyImage(pp->box);
+		pp->box = NULL;
+	}
 }
 
 static int
@@ -349,11 +372,12 @@ moveboxdelta(ModeInfo * mi)
 	pp->Lp * pp->rfactor + pp->randompos.y;
 
 	if (pp->Lp <= pp->lengthOfMove) {
-		XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box,
-		 0, 0, cf + 1, rf + 1, pp->boxsize.x - 2, pp->boxsize.y - 2);
-		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC,
-		cf + pp->cbs - 1, rf + pp->rbs - 1, pp->cbw + 2, pp->rbw + 2);
-
+		if (pp->box) {
+			XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box,
+				  0, 0, cf + 1, rf + 1, pp->boxsize.x - 2, pp->boxsize.y - 2);
+			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC,
+				       cf + pp->cbs - 1, rf + pp->rbs - 1, pp->cbw + 2, pp->rbw + 2);
+		}
 		if ((pp->Lp + pp->incrementOfMove > pp->lengthOfMove) &&
 		    (pp->Lp != pp->lengthOfMove))
 			pp->Lp = pp->lengthOfMove - pp->incrementOfMove;
@@ -402,22 +426,24 @@ init_puzzle(ModeInfo * mi)
 		XDestroyImage(pp->box);
 		pp->box = NULL;
 	}
-	if (!pp->logo || pp->graphics_format <= 0) {
+	if (!pp->logo)
 		getImage(mi, &pp->logo, PUZZLE_WIDTH, PUZZLE_HEIGHT, PUZZLE_BITS,
-			 PUZZLE_NAME, &pp->graphics_format, &pp->newcolormap);
-		if (pp->newcolormap) {
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+			 PUZZLE_NAME,
+#endif
+			 &pp->graphics_format, &pp->cm,
+			 &pp->black, &pp->white);
+	if (pp->cm != None) {
+		if (pp->backGC == None) {
 			XGCValues   xgcv;
 
-			pp->cm = XCreateColormap(display, window, MI_VISUAL(mi), AllocNone);
-			SetImageColors(display, pp->cm);
 			setColormap(display, window, pp->cm, MI_WIN_IS_INWINDOW(mi));
-			xgcv.background = pp->black = GetBlack();
+			xgcv.background = pp->black;
 			pp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
-		} else {
-			pp->cm = None;
-			pp->black = MI_WIN_BLACK_PIXEL(mi);
-			pp->backGC = MI_GC(mi);
 		}
+	} else {
+		pp->black = MI_WIN_BLACK_PIXEL(mi);
+		pp->backGC = MI_GC(mi);
 	}
 	XSetForeground(display, pp->backGC, pp->black);
 	XFillRectangle(display, window, pp->backGC, 0, 0,
@@ -541,13 +567,14 @@ release_puzzle(ModeInfo * mi)
 
 			if (pp->fixbuff != NULL)
 				(void) free((void *) pp->fixbuff);
-			if (pp->newcolormap) {
-				if (!pp->backGC)
+			if (pp->cm != None) {
+				if (pp->backGC != None)
 					XFreeGC(MI_DISPLAY(mi), pp->backGC);
-				if (pp->cm != None)
-					XFreeColormap(MI_DISPLAY(mi), pp->cm);
+				XFreeColormap(MI_DISPLAY(mi), pp->cm);
 			}
 			destroyImage(pp->logo, pp->graphics_format);
+			if (pp->box)
+				XDestroyImage(pp->box);
 		}
 		(void) free((void *) puzzs);
 		puzzs = NULL;
