@@ -1,6 +1,5 @@
-
-#ifndef lint
-static char sccsid[] = "@(#)ras.c	3.9 96/05/25 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)ras.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -18,10 +17,8 @@ static char sccsid[] = "@(#)ras.c	3.9 96/05/25 xlockmore";
  * 22-May-95: Written.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
 #include "xlock.h"
+#include <time.h>
 #include "ras.h"
 
 char       *imagefile;
@@ -125,6 +122,7 @@ GetBlack(void)
 	return ras.entry[find_nearest(0, 0, 0)];
 }
 
+#if 0
 unsigned long
 GetForeground(Display * display, unsigned long pixel)
 {
@@ -148,6 +146,7 @@ GetBackground(Display * display, unsigned long pixel)
 	return ras.entry[find_nearest(
 			 bgcol.red >> 8, bgcol.green >> 8, bgcol.blue >> 8)];
 }
+#endif
 
 static unsigned long
 get_long(int n)
@@ -166,7 +165,7 @@ RasterFileToImage(Display * display, char *filename, XImage ** image
 	int         read_width;
 	FILE       *file;
 
-	if (!(file = fopen(filename, "rb"))) {
+	if ((file = fopen(filename, "rb")) == NULL) {
 		/*(void) fprintf(stderr, "could not read file \"%s\"\n", filename); */
 		return RasterOpenFailed;
 	}
