@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)resource.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)resource.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -13,6 +13,10 @@ static const char sccsid[] = "@(#)resource.c	4.00 97/01/01 xlockmore";
  * Revision History:
  *
  * Changes of David Bagley (bagleyd@bigfoot.com)
+ * 01-Apr-97: Tom Schmidt <tschmidt@micron.com>
+ *            Fixed memory leak.  Made -visual option a hacker mode for now.
+ * 20-Mar-97: Tom Schmidt <tschmidt@micron.com>
+ *            Added -visual option.
  *  3-Apr-96: Jouk Jansen <joukj@alpha.chem.uva.nl>.
  *            Supply for wildcards for filenames for random selection
  * 18-Mar-96: Ron Hitchens <ron@idiom.com>
@@ -60,6 +64,16 @@ static const char sccsid[] = "@(#)resource.c	4.00 97/01/01 xlockmore";
 #include "xlock.h"
 #include "mode.h"
 #include "version.h"
+#if VMS
+#if ( __VMS_VER < 70000000 )
+#ifdef __DECC
+#define gethostname decc$gethostname
+#define gethostbyname decc$gethostbyname
+#endif
+#else
+#include <socket.h>
+#endif
+#endif
 #include <netdb.h>
 #include <ctype.h>
 
@@ -85,14 +99,8 @@ extern char *getenv(const char *);
 #include <ssdef>
 #include <stsdef>
 #include <types.h>
-/* #include <socket.h> */
 #include <starlet.h>
 
-/*#if ( __VMS_VER < 70000000 )
-#define gethostname decc$gethostname
-#define gethostbyname decc$gethostbyname
-#endif*/
-
 #define DEF_FILESEARCHPATH "DECW$SYSTEM_DEFAULTS:DECW$%N.DAT%S"
 #define BUFSIZE 132
 #define DECW$C_WS_DSP_TRANSPORT 2	/*  taken from wsdriver.lis */
@@ -101,7 +109,7 @@ extern char *getenv(const char *);
 struct descriptor_t {		/* descriptor structure         */
 	unsigned short len;
 	unsigned char type;
-	unsigned char class;
+	unsigned char c_class;
 	char       *ptr;
 };
 typedef struct descriptor_t dsc;
@@ -197,17 +205,23 @@ static int  descr();
 #endif /* !DEF_PLAY && !USE_NAS */
 #endif /* !USE_RPLAY */
 
-#ifdef USE_AUTO_LOGOUT
-#define DEF_LOGOUT	"30"	/* minutes until auto-logout */
+#if defined( USE_AUTO_LOGOUT ) && !defined( DEF_AUTO_LOGOUT )
+#if ( USE_AUTO_LOGOUT <= 0 )
+#define DEF_AUTO_LOGOUT "120"	/* User Default, can be overridden */
+#else
+#define DEF_AUTO_LOGOUT "0"	/* User Default, can be overridden */
+#endif
 #endif
 
-#ifdef USE_LOGOUT_BUTTON
-#ifdef ALLOW_NO_LOGOUT_BUTTON
-#define DEF_BTN   	"5"	/* minutes until logout button appears */
+#if defined( USE_BUTTON_LOGOUT )
+#if !defined( DEF_BUTTON_LOGOUT )
+#if ( USE_BUTTON_LOGOUT <= 0 )
+#define DEF_BUTTON_LOGOUT "5"	/* User Default, can be overridden */
+#else
+#define DEF_BUTTON_LOGOUT "0"	/* User Default, can be overridden */
 #endif
-#ifndef LO_BUTTON_TIME
-#define LO_BUTTON_TIME 5
 #endif
+
 #define DEF_BTN_LABEL	"Logout"	/* string that appears in logout button */
 
 /* this string appears immediately below logout button */
@@ -274,6 +288,8 @@ static XrmOptionDescRec genTable[] =
 	{"+echokeys", ".echokeys", XrmoptionNoArg, (caddr_t) "off"},
 	{"-enablesaver", ".enablesaver", XrmoptionNoArg, (caddr_t) "on"},
 	{"+enablesaver", ".enablesaver", XrmoptionNoArg, (caddr_t) "off"},
+	{"-fullrandom", ".fullrandom", XrmoptionNoArg, (caddr_t) "on"},
+	{"+fullrandom", ".fullrandom", XrmoptionNoArg, (caddr_t) "off"},
 	{"-grabmouse", ".grabmouse", XrmoptionNoArg, (caddr_t) "on"},
 	{"+grabmouse", ".grabmouse", XrmoptionNoArg, (caddr_t) "off"},
 	{"-install", ".install", XrmoptionNoArg, (caddr_t) "on"},
@@ -303,6 +319,9 @@ static XrmOptionDescRec genTable[] =
 	{"-geometry", ".geometry", XrmoptionSepArg, (caddr_t) NULL},
 	{"-icongeometry", ".icongeometry", XrmoptionSepArg, (caddr_t) NULL},
 
+	{"-wireframe", ".wireframe", XrmoptionNoArg, (caddr_t) "on"},
+	{"+wireframe", ".wireframe", XrmoptionNoArg, (caddr_t) "off"},
+
 	{"-use3d", ".use3d", XrmoptionNoArg, (caddr_t) "on"},
 	{"+use3d", ".use3d", XrmoptionNoArg, (caddr_t) "off"},
 	{"-delta3d", ".delta3d", XrmoptionSepArg, (caddr_t) NULL},
@@ -326,9 +345,9 @@ static XrmOptionDescRec genTable[] =
 	{"-cpasswd", ".cpasswd", XrmoptionSepArg, (caddr_t) NULL},
 #endif
 #ifdef USE_AUTO_LOGOUT
-	{"-forceLogout", ".forceLogout", XrmoptionSepArg, (caddr_t) NULL},
+	{"-logoutAuto", ".logoutAuto", XrmoptionSepArg, (caddr_t) NULL},
 #endif
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 	{"-logoutButton", ".logoutButton", XrmoptionSepArg, (caddr_t) NULL},
 {"-logoutButtonLabel", ".logoutButtonLabel", XrmoptionSepArg, (caddr_t) NULL},
  {"-logoutButtonHelp", ".logoutButtonHelp", XrmoptionSepArg, (caddr_t) NULL},
@@ -363,6 +382,9 @@ static XrmOptionDescRec modeTable[] =
 static XrmOptionDescRec cmdlineTable[] =
 {
 	{"-display", ".display", XrmoptionSepArg, (caddr_t) NULL},
+#ifdef USE_HACKERS
+	{"-visual", ".visual", XrmoptionSepArg, (caddr_t) NULL},
+#endif
 	{"-nolock", ".nolock", XrmoptionNoArg, (caddr_t) "on"},
 	{"+nolock", ".nolock", XrmoptionNoArg, (caddr_t) "off"},
 	{"-remote", ".remote", XrmoptionNoArg, (caddr_t) "on"},
@@ -387,10 +409,13 @@ static XrmOptionDescRec nameTable[] =
 
 static OptionStruct opDesc[] =
 {
-	{"-help", "print out this message"},
-	{"-version", "print version number (if >= 4.00)"},
+	{"-help", "print out this message to standard output"},
+	{"-version", "print version number (if >= 4.00) to standard output"},
 	{"-resources", "print default resource file to standard output"},
 	{"-display displayname", "X server to contact"},
+#ifdef USE_HACKERS
+	{"-visual visualname", "X visual to use"},
+#endif
 {"-name resourcename", "class name to use for resources (default is XLock)"},
 	{"-delay usecs", "microsecond delay between screen updates"},
 	{"-batchcount num", "number of things per batch"},
@@ -413,6 +438,7 @@ static OptionStruct opDesc[] =
 	{"-/+enablesaver", "turn on/off enable X server screen saver"},
 	{"-/+grabmouse", "turn on/off grabbing of mouse and keyboard"},
 	{"-/+install", "whether to use private colormap if needed (yes/no)"},
+	{"-/+fullrandom", "turn on/off full random choice of mode-options"},
 	{"-/+sound", "whether to use sound if configured for it (yes/no)"},
 	{"-/+timeelapsed", "turn on/off clock"},
 	{"-/+usefirst", "turn on/off using the first char typed in password"},
@@ -434,6 +460,8 @@ static OptionStruct opDesc[] =
 	{"-geometry geom", "geometry for non-full screen lock"},
    {"-icongeometry geom", "geometry for password window (location ignored)"},
 
+	{"-/+wireframe", "turn on/off wireframe"},
+
 	{"-/+use3d", "turn on/off 3d view"},
    {"-delta3d value", "space between the center of your 2 eyes for 3d mode"},
 	{"-none3d color", "color to be used for null in 3d mode"},
@@ -456,12 +484,10 @@ static OptionStruct opDesc[] =
 	{"-cpasswd crypted-password", "text string of encrypted password"},
 #endif
 #ifdef USE_AUTO_LOGOUT
-	{"-forceLogout minutes", "number of minutes until auto logout"},
-#endif
-#ifdef USE_LOGOUT_BUTTON
-#ifdef ALLOW_NO_LOGOUT_BUTTON
-  {"-logoutButton minutes", "number of minutes until logout button appears"},
+	{"-logoutAuto minutes", "number of minutes until auto logout (not more than forced auto logout time)"},
 #endif
+#ifdef USE_BUTTON_LOGOUT
+	{"-logoutButton minutes", "number of minutes until logout button appears (not more than forced button time)"},
     {"-logoutButtonLabel string", "text string to use inside logout button"},
    {"-logoutButtonHelp string", "text string to use for logout button help"},
 	{"-logoutFailedString string", "text string to use for failed logout attempts"},
@@ -502,6 +528,7 @@ Bool        allowroot;
 Bool        debug;
 Bool        echokeys;
 Bool        enablesaver;
+Bool        fullrandom;
 Bool        grabmouse;
 Bool        install;
 Bool        sound;
@@ -509,6 +536,9 @@ Bool        timeelapsed;
 Bool        usefirst;
 Bool        verbose;
 
+static char *visualname;
+int         VisualClassWanted;
+
 int         nicelevel;
 int         lockdelay;
 int         timeout;
@@ -525,6 +555,8 @@ char       *text_invalid;
 char       *geometry;
 char       *icongeometry;
 
+Bool        wireframe;
+
 Bool        use3d;
 float       delta3d;
 char       *none3d;
@@ -537,19 +569,13 @@ char       *cpasswd;
 
 #endif
 #ifdef USE_AUTO_LOGOUT
-int         forceLogout;
+int         logoutAuto;
 
 #endif
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 int         enable_button = 1;
 
-#ifdef ALLOW_NO_LOGOUT_BUTTON
 int         logoutButton;
-
-#else
-int         logoutButton = LO_BUTTON_TIME;
-
-#endif
 char       *logoutButtonLabel;
 char       *logoutButtonHelp;
 char       *logoutFailedString;
@@ -578,12 +604,14 @@ static argtype genvars[] =
 	{(caddr_t *) & debug, "debug", "Debug", "off", t_Bool},
 	{(caddr_t *) & echokeys, "echokeys", "EchoKeys", "off", t_Bool},
     {(caddr_t *) & enablesaver, "enablesaver", "EnableSaver", "off", t_Bool},
+	{(caddr_t *) & fullrandom, "fullrandom", "FullRandom", "off", t_Bool},
 	{(caddr_t *) & grabmouse, "grabmouse", "GrabMouse", "on", t_Bool},
-	{(caddr_t *) & install, "install", "Install", "off", t_Bool},
+	{(caddr_t *) & install, "install", "Install", "on", t_Bool},
 	{(caddr_t *) & sound, "sound", "Sound", "off", t_Bool},
     {(caddr_t *) & timeelapsed, "timeelapsed", "TimeElapsed", "off", t_Bool},
 	{(caddr_t *) & usefirst, "usefirst", "Usefirst", "off", t_Bool},
 	{(caddr_t *) & verbose, "verbose", "Verbose", "off", t_Bool},
+	{(caddr_t *) & visualname, "visual", "Visual", "", t_String},
 
 	{(caddr_t *) & nicelevel, "nice", "Nice", DEF_NICE, t_Int},
    {(caddr_t *) & lockdelay, "lockdelay", "LockDelay", DEF_LOCKDELAY, t_Int},
@@ -600,6 +628,8 @@ static argtype genvars[] =
     {(caddr_t *) & geometry, "geometry", "Geometry", DEF_GEOMETRY, t_String},
 	{(caddr_t *) & icongeometry, "icongeometry", "IconGeometry", DEF_ICONGEOMETRY, t_String},
 
+	{(caddr_t *) & wireframe, "wireframe", "WireFrame", "off", t_Bool},
+
 	{(caddr_t *) & use3d, "use3d", "Use3D", "off", t_Bool},
 	{(caddr_t *) & delta3d, "delta3d", "Delta3D", DEF_DELTA3D, t_Float},
 	{(caddr_t *) & none3d, "none3d", "None3D", DEF_NONE3D, t_String},
@@ -619,12 +649,10 @@ static argtype genvars[] =
 	{(caddr_t *) & cpasswd, "cpasswd", "cpasswd", "", t_String},
 #endif
 #ifdef USE_AUTO_LOGOUT
-{(caddr_t *) & forceLogout, "forceLogout", "ForceLogout", DEF_LOGOUT, t_Int},
-#endif
-#ifdef USE_LOGOUT_BUTTON
-#ifdef ALLOW_NO_LOGOUT_BUTTON
-{(caddr_t *) & logoutButton, "logoutButton", "LogoutButton", DEF_BTN, t_Int},
+	{(caddr_t *) & logoutAuto, "logoutAuto", "logoutAuto", DEF_AUTO_LOGOUT, t_Int},
 #endif
+#ifdef USE_BUTTON_LOGOUT
+	{(caddr_t *) & logoutButton, "logoutButton", "LogoutButton", DEF_BUTTON_LOGOUT, t_Int},
 	{(caddr_t *) & logoutButtonLabel, "logoutButtonLabel",
 	 "LogoutButtonLabel", DEF_BTN_LABEL, t_String},
 	{(caddr_t *) & logoutButtonHelp, "logoutButtonHelp",
@@ -739,26 +767,24 @@ Help(void)
 {
 	int         i;
 
-	(void) fprintf(stderr, "usage:\n        %s [-options ...]\n\n",
-		       ProgramName);
-	(void) fprintf(stderr, "where options include:\n");
+	(void) printf("usage:\n        %s [-options ...]\n\n", ProgramName);
+	(void) printf("where options include:\n");
 	for (i = 0; i < (int) opDescEntries; i++) {
-		(void) fprintf(stderr, "    %-28s %s\n",
-			       opDesc[i].opt, opDesc[i].desc);
+		(void) printf("    %-28s %s\n", opDesc[i].opt, opDesc[i].desc);
 	}
 
-	(void) fprintf(stderr, "    %-28s %s\n", "-mode mode", "animation mode");
-	(void) fprintf(stderr, "    where mode is one of:\n");
+	(void) printf("    %-28s %s\n", "-mode mode", "animation mode");
+	(void) printf("    where mode is one of:\n");
 	for (i = 0; i < numprocs; i++) {
 		int         j;
 
-		(void) fprintf(stderr, "          %-23s %s\n",
-			       LockProcs[i].cmdline_arg, LockProcs[i].desc);
+		(void) printf("          %-23s %s\n",
+			      LockProcs[i].cmdline_arg, LockProcs[i].desc);
 		for (j = 0; j < LockProcs[i].msopt->numvarsdesc; j++)
-			(void) fprintf(stderr, "              %-23s %s\n",
-				       LockProcs[i].msopt->desc[j].opt, LockProcs[i].msopt->desc[j].desc);
+			(void) printf("              %-23s %s\n",
+				      LockProcs[i].msopt->desc[j].opt, LockProcs[i].msopt->desc[j].desc);
 	}
-	(void) putc('\n', stderr);
+	(void) printf("\n");
 
 	exit(0);
 }
@@ -766,14 +792,14 @@ Help(void)
 static void
 Version(void)
 {
-	(void) fprintf(stderr, "XLock version %s\n", VERSION);
+	(void) printf("XLock version %s\n", VERSION);
 	exit(0);
 }
 
 static void
 DumpResources(void)
 {
-	int         i;
+	int         i, j;
 
 	(void) printf("%s.mode: %s\n", classname, DEF_MODE);
 
@@ -792,6 +818,10 @@ DumpResources(void)
 			      "size", LockProcs[i].def_size);
 		(void) printf("%s.%s.%s: %g\n", classname, LockProcs[i].cmdline_arg,
 			      "saturation", LockProcs[i].def_saturation);
+		for (j = 0; j < LockProcs[i].msopt->numvarsdesc; j++)
+			(void) printf("%s.%s.%s: %s\n", classname, LockProcs[i].cmdline_arg,
+				      LockProcs[i].msopt->vars[j].name,
+				      LockProcs[i].msopt->vars[j].def);
 	}
 	exit(0);
 }
@@ -846,6 +876,7 @@ GetResource(XrmDatabase database, char *parentname, char *parentclassname,
 	switch (valueType) {
 		case t_String:
 			{
+/* PURIFY 3.0a on SunOS4 reports a 1 byte memory leak on the following line. */
 				char       *s = (char *) malloc(len + 1);
 
 				if (s == (char *) NULL)
@@ -879,10 +910,43 @@ parsefilepath(char *xfilesearchpath, char *TypeName, char *ClassName)
 	char       *appdefaults;
 	char       *src;
 	char       *dst;
+	int         i, maxlen;
+
+	i = maxlen = 0;
+	for (src = xfilesearchpath; *src != '\0'; src++) {
+		if (*src == '%') {
+			src++;
+			switch (*src) {
+				case '%':
+				case ':':
+					i++;
+					break;
+				case 'T':
+					i += strlen(TypeName);
+					break;
+				case 'N':
+					i += strlen(ClassName);
+					break;
+				default:
+					break;
+			}
+#ifdef VMS
+		} else if (*src == '#') {	/* Colons required in VMS use # */
+#else
+		} else if (*src == ':') {
+#endif
+			if (i > maxlen)
+				maxlen = i;
+			i = 0;
+		}
+		else
+			i++;
+	}
+	if (i > maxlen)
+		maxlen = i;
 
 	/* appdefaults will be at most this long */
-	appdefaults = (char *) malloc(strlen(xfilesearchpath) +
-				   strlen(TypeName) + strlen(ClassName) + 1);
+	appdefaults = (char *) malloc(maxlen + 1);
 
 	src = xfilesearchpath;
 	dst = appdefaults;
@@ -1217,7 +1281,7 @@ getResources(int argc, char **argv)
 	}
 	userDB = parsefilepath(userpath, "app-defaults", classname);
 
-	if (!userfile)
+	if (userfile)
 		(void) free((void *) userfile);
 
 	(void) XrmMergeDatabases(applicationDB, &RDB);
@@ -1346,13 +1410,36 @@ getResources(int argc, char **argv)
 		}
 	}
 
-#if defined( USE_AUTO_LOGOUT ) || defined( USE_LOGOUT_BUTTON )
+#if defined( USE_AUTO_LOGOUT ) || defined( USE_BUTTON_LOGOUT )
 	if (fullLock()) {
 #ifdef USE_AUTO_LOGOUT
-		forceLogout = 0;
+		logoutAuto = 0;
 #endif
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 		enable_button = 0;
+#endif
+	} else {
+#ifdef USE_AUTO_LOGOUT
+#if ( USE_AUTO_LOGOUT > 0 )	/* Could be USER defined if 0 */
+		if (logoutAuto > USE_AUTO_LOGOUT)
+			logoutAuto = USE_AUTO_LOGOUT;
+		else if (logoutAuto <= 0)	/* Handle 0 as a special case */
+			logoutAuto = USE_AUTO_LOGOUT;
+#else
+		if (logoutAuto <= 0)	/* Handle 0 as a special case */
+			(void) sscanf(DEF_AUTO_LOGOUT, "%d", &logoutAuto);
+#endif
+#endif
+#ifdef USE_BUTTON_LOGOUT
+#if ( USE_BUTTON_LOGOUT > 0 )	/* Could be USER defined if 0 */
+		if (logoutButton > USE_BUTTON_LOGOUT)
+			logoutButton = USE_BUTTON_LOGOUT;
+		else if (logoutButton <= 0)	/* Handle 0 as a special case */
+			logoutButton = USE_BUTTON_LOGOUT;
+#else
+		if (logoutButton <= 0)	/* Handle 0 as a special case */
+			(void) sscanf(DEF_BUTTON_LOGOUT, "%d", &logoutButton);
+#endif
 #endif
 	}
 #endif
@@ -1375,6 +1462,16 @@ getResources(int argc, char **argv)
 		for (i = 0; i < (int) NMODEARGS; i++)
 			printvar(modename, modevars[i]);
 	}
+	if (*visualname == '\0') {
+		VisualClassWanted = -1;
+		if (verbose)
+			(void) fprintf(stderr, "Using default visual class\n");
+	} else {
+		VisualClassWanted = visualClassFromName(visualname);
+		if (verbose)
+			(void) fprintf(stderr, "Using visual class %s\n",
+				       nameOfVisualClass(VisualClassWanted));
+	}
 #if HAVE_DIRENT_H
 	/* Evaluate imagefile */
 	if (strcmp(imagefile, DEF_IMAGEFILE)) {
