@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)shape.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)shape.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -15,7 +15,7 @@ static char sccsid[] = "@(#)shape.c	3.11 96/09/20 xlockmore";
  * 27-Jun-95: added ellipses
  * 27-Feb-95: patch for VMS
  * 29-Sep-94: multidisplay bug fix (epstein_caleb@jpmorgan.com)
- * 15-Jul-94: xlock version (David Bagley bagleyd@hertz.njit.edu)
+ * 15-Jul-94: xlock version (David Bagley bagleyd@bigfoot.com)
  * 1992:     xscreensaver version (Jamie Zawinski jwz@netscape.com)
  */
 
@@ -32,59 +32,21 @@ static char sccsid[] = "@(#)shape.c	3.11 96/09/20 xlockmore";
  */
 
 #include	"xlock.h"
-
-/* if you get errors in this file on VMS #define CORRUPT_VMS_BITMAPS */
-#ifdef CORRUPT_VMS_BITMAPS
-#define NBITS 7
-#else
 #define NBITS 12
-#endif
 
-#ifdef VMS
-#if 0
-#include "../bitmaps/stipple.xbm"
-#include "../bitmaps/cross_weave.xbm"
-#include "../bitmaps/dimple1.xbm"
-#include "../bitmaps/dimple3.xbm"
-#include "../bitmaps/flipped_gray.xbm"
-#include "../bitmaps/gray1.xbm"
-#include "../bitmaps/gray3.xbm"
-#include "../bitmaps/hlines2.xbm"
-#include "../bitmaps/light_gray.xbm"
-#include "../bitmaps/root_weave.xbm"
-#include "../bitmaps/vlines2.xbm"
-#include "../bitmaps/vlines3.xbm"
-#else
-#include <decw$bitmaps/cross_weave.xbm>
-#include <decw$bitmaps/dimple1.xbm>
-#include <decw$bitmaps/dimple3.xbm>
-#include <decw$bitmaps/flipped_gray.xbm>
-#include <decw$bitmaps/gray1.xbm>
-#include <decw$bitmaps/gray3.xbm>
-#include <decw$bitmaps/vlines3.xbm>
-#ifndef CORRUPT_VMS_BITMAPS
-#include <decw$bitmaps/stipple.xbm>
-#include <decw$bitmaps/hlines2.xbm>
-#include <decw$bitmaps/light_gray.xbm>
-#include <decw$bitmaps/root_weave.xbm>
-#include <decw$bitmaps/vlines2.xbm>
-#endif
-#endif
-#else
-#include <X11/bitmaps/stipple>
-#include <X11/bitmaps/cross_weave>
-#include <X11/bitmaps/dimple1>
-#include <X11/bitmaps/dimple3>
-#include <X11/bitmaps/flipped_gray>
-#include <X11/bitmaps/gray1>
-#include <X11/bitmaps/gray3>
-#include <X11/bitmaps/hlines2>
-#include <X11/bitmaps/light_gray>
-#include <X11/bitmaps/root_weave>
-#include <X11/bitmaps/vlines2>
-#include <X11/bitmaps/vlines3>
-#endif
-#define BITS(n,w,h)\
+#include "bitmaps/stipple.xbm"
+#include "bitmaps/cross_weave.xbm"
+#include "bitmaps/dimple1.xbm"
+#include "bitmaps/dimple3.xbm"
+#include "bitmaps/flipped_gray.xbm"
+#include "bitmaps/gray1.xbm"
+#include "bitmaps/gray3.xbm"
+#include "bitmaps/hlines2.xbm"
+#include "bitmaps/light_gray.xbm"
+#include "bitmaps/root_weave.xbm"
+#include "bitmaps/vlines2.xbm"
+#include "bitmaps/vlines3.xbm"
+#define SHAPEBITS(n,w,h)\
   sp->pixmaps[sp->init_bits++]=\
 		XCreateBitmapFromData(display,window,(char *)n,w,h)
 
@@ -133,20 +95,18 @@ init_shape(ModeInfo * mi)
 		XSetFillStyle(display, sp->stippledgc, FillOpaqueStippled);
 	}
 	if (!sp->init_bits) {
-		BITS(cross_weave_bits, cross_weave_width, cross_weave_height);
-		BITS(dimple1_bits, dimple1_width, dimple1_height);
-		BITS(dimple3_bits, dimple3_width, dimple3_height);
-		BITS(flipped_gray_bits, flipped_gray_width, flipped_gray_height);
-		BITS(gray1_bits, gray1_width, gray1_height);
-		BITS(gray3_bits, gray3_width, gray3_height);
-		BITS(vlines3_bits, vlines3_width, vlines3_height);
-#ifndef CORRUPT_VMS_BITMAPS
-		BITS(stipple_bits, stipple_width, stipple_height);
-		BITS(hlines2_bits, hlines2_width, hlines2_height);
-		BITS(light_gray_bits, light_gray_width, light_gray_height);
-		BITS(root_weave_bits, root_weave_width, root_weave_height);
-		BITS(vlines2_bits, vlines2_width, vlines2_height);
-#endif
+		SHAPEBITS(cross_weave_bits, cross_weave_width, cross_weave_height);
+		SHAPEBITS(dimple1_bits, dimple1_width, dimple1_height);
+		SHAPEBITS(dimple3_bits, dimple3_width, dimple3_height);
+		SHAPEBITS(flipped_gray_bits, flipped_gray_width, flipped_gray_height);
+		SHAPEBITS(gray1_bits, gray1_width, gray1_height);
+		SHAPEBITS(gray3_bits, gray3_width, gray3_height);
+		SHAPEBITS(vlines3_bits, vlines3_width, vlines3_height);
+		SHAPEBITS(stipple_bits, stipple_width, stipple_height);
+		SHAPEBITS(hlines2_bits, hlines2_width, hlines2_height);
+		SHAPEBITS(light_gray_bits, light_gray_width, light_gray_height);
+		SHAPEBITS(root_weave_bits, root_weave_width, root_weave_height);
+		SHAPEBITS(vlines2_bits, vlines2_width, vlines2_height);
 	}
 	XClearWindow(display, window);
 }
