@@ -1,6 +1,6 @@
 
-#ifndef lint
-static char sccsid[] = "@(#)slip.c	3.11 96/09/20 xlockmore";
+#if !defined( lint ) && !defined( SABER )
+static const char sccsid[] = "@(#)slip.c	4.00 97/01/01 xlockmore";
 
 #endif
 
@@ -14,7 +14,6 @@ static char sccsid[] = "@(#)slip.c	3.11 96/09/20 xlockmore";
  */
 
 #include "xlock.h"
-#include <math.h>
 
 ModeSpecOpt slip_opts =
 {0, NULL, 0, NULL, NULL};
@@ -85,12 +84,21 @@ prepare_screen(ModeInfo * mi, slipstruct * s)
 			n = 2000;
 	}
 
-	XSetForeground(display, gc, MI_PIXEL(mi, halfrandom(MI_NPIXELS(mi))));
+	if (MI_NPIXELS(mi) > 2)
+		XSetForeground(display, gc, MI_PIXEL(mi, halfrandom(MI_NPIXELS(mi))));
+	else if (halfrandom(2))
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	else
+		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
 
 	for (i = 0; i < n; i++) {
 		if (not_solid)
-			XSetForeground(display, gc,
-				   MI_PIXEL(mi, halfrandom(MI_NPIXELS(mi))));
+			if (MI_NPIXELS(mi) > 2)
+				XSetForeground(display, gc, MI_PIXEL(mi, halfrandom(MI_NPIXELS(mi))));
+			else if (halfrandom(2))
+				XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+			else
+				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
 		XFillRectangle(display, MI_WINDOW(mi), gc,
 			       halfrandom(s->width - w),
 			       halfrandom(s->height - w),
