@@ -109,6 +109,7 @@ typedef struct swirl_data {
 	int         shift;	/* colourmap shift */
 	int         dshift;	/* colourmap shift while drawing */
 	XColor      fgcol, bgcol;	/* foreground and background colour specs */
+	Bool       off_screen;
 } SWIRL    , *SWIRL_P;
 
 #define SWIRLCOLOURS 13
@@ -149,8 +150,8 @@ initialise_swirl(ModeInfo * mi, SWIRL_P swirl)
 
 	swirl->width = 0;	/* width and height of window */
 	swirl->height = 0;
-	swirl->depth = 0;
-	swirl->rdepth = 0;
+	swirl->depth = 1;
+	swirl->rdepth = 1;
 	swirl->visual = NULL;
 	swirl->resolution = MIN_RES + 1;	/* current resolution */
 	swirl->max_resolution = MAX_RES;	/* maximum resolution */
@@ -564,11 +565,11 @@ basic_map(SWIRL_P swirl, XColor * values)
 
 	/* allocate colours in proportion to the lengths of the sides */
 	interpolate(swirl, &value, &pixel,
-		    (int) ((double) swirl->dcolours * ((double) L1 / (double) L)), c, c + 1);
+		    (int) ((double) swirl->dcolours * ((double) L1 / (double) L)) + 1, c, c + 1);
 	interpolate(swirl, &value, &pixel,
-		    (int) ((double) swirl->dcolours * ((double) L2 / (double) L)), c + 1, c + 2);
+		    (int) ((double) swirl->dcolours * ((double) L2 / (double) L)) + 1, c + 1, c + 2);
 	interpolate(swirl, &value, &pixel,
-		    (int) ((double) swirl->dcolours * ((double) L3 / (double) L)), c + 2, c);
+		    (int) ((double) swirl->dcolours * ((double) L3 / (double) L)) + 1, c + 2, c);
 
 	/* fill up any remaining slots (due to rounding) */
 	while ((int) pixel < swirl->colours) {
@@ -639,49 +640,6 @@ pre_rotate(SWIRL_P swirl, XColor * values)
 
 /****************************************************************/
 
-/* 
- * default_visual_info
- *
- * Gets a XVisualInfo structure that refers to a given visual or the default
- * visual.
- *
- * -      mi is the ModeInfo
- * -      visual is the visual to look up NULL => look up the default visual
- * -      default_info is set to point to the member of the returned list
- *          that corresponds to the default visual.
- *
- * Returns a list of XVisualInfo structures or NULL on failure. Free the list
- *   with XFree.
- */
-static XVisualInfo *
-default_visual_info(ModeInfo * mi, Visual * visual, XVisualInfo ** default_info)
-{
-	Display    *display = MI_DISPLAY(mi);
-	XVisualInfo *info_list;
-	int         i, n;
-
-	/* get a complete list of visuals */
-	info_list = XGetVisualInfo(display, VisualNoMask, NULL, &n);
-	if ((info_list == NULL) || (n == 0))
-		return (NULL);
-
-	/* do we need to get the default visual? */
-	if (visual == NULL)
-		visual = DefaultVisual(display, MI_SCREEN(mi));
-
-	/* search through the list for the default visual */
-	*default_info = info_list;
-	for (i = 0; i < n; i++, (*default_info)++)
-		if ((*default_info)->visual == visual &&
-		    (*default_info)->screen == MI_SCREEN(mi)) {
-			break;
-		}
-	/* return the list (and the default info) */
-	return (info_list);
-}
-
-/****************************************************************/
-
 /* 
  * create_colourmap
  *
@@ -694,62 +652,21 @@ static void
 create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 {
 	Display    *display = MI_DISPLAY(mi);
-	XVisualInfo *list, *default_info;
 	int         preserve;
 	int         n_rotations;
 	int         i;
+	Bool        truecolor;
+  unsigned long redmask, greenmask, bluemask;
 
-	/* get information about the default visual */
-	list = default_visual_info(mi, swirl->visual, &default_info);
-
-	/* how many colours are there altogether? */
-	swirl->colours = MI_NPIXELS(mi);
-	if (swirl->colours < 2)
-		swirl->colours = 2;
-	if (swirl->colours > default_info->colormap_size) {
-		swirl->colours = default_info->colormap_size;
-	}
-	/* how many colours should we preserve (out of white, black, fg, bg)? */
-	if (((swirl->bg == swirl->black) || (swirl->bg == swirl->white)) &&
-	    ((swirl->fg == swirl->black) || (swirl->fg == swirl->white)))
-		preserve = 2;
-	else if ((swirl->bg == swirl->black) || (swirl->fg == swirl->black) ||
-		 (swirl->bg == swirl->white) || (swirl->fg == swirl->white) ||
-		 (swirl->bg == swirl->fg))
-		preserve = 3;
-	else
-		preserve = 4;
+	swirl->fixed_colourmap = !setupColormap(mi, &(swirl->colours),
+    &truecolor, &redmask, &greenmask, &bluemask);
+	preserve = preserveColors(swirl->fg, swirl->bg, swirl->white, swirl->black);
 
 	/* how many colours should we animate? */
 	swirl->dcolours = (swirl->colours > preserve + 1) ?
 		swirl->colours - preserve : swirl->colours;
 
-	/* are we in monochrome mode? */
-	swirl->monochrome = (swirl->colours <= 2);
-
-	/* is the colourmap fixed (we'll leave it alone in mono too) */
-
-#if defined(__cplusplus) || defined(c_plusplus)
-	swirl->fixed_colourmap = ((default_info->c_class == StaticGray) ||
-				  (default_info->c_class == StaticColor) ||
-				  (default_info->c_class == TrueColor) ||
-	swirl->monochrome || MI_WIN_IS_INROOT(mi) || !MI_WIN_IS_INSTALL(mi));
-#else
-	swirl->fixed_colourmap = ((default_info->class == StaticGray) ||
-				  (default_info->class == StaticColor) ||
-				  (default_info->class == TrueColor) ||
-	swirl->monochrome || MI_WIN_IS_INROOT(mi) || !MI_WIN_IS_INSTALL(mi));
-#endif
-
-#ifdef FORCEFIXEDCOLORS
-	/* pretending a fixed colourmap */
-	swirl->fixed_colourmap = True;
-#endif
-	/* free the list */
-	XFree((char *) list);
-
-	/* if monochrome, we've done all we can */
-	if (swirl->monochrome)
+	if (MI_NPIXELS(mi) < 2)
 		return;
 
 	/* how fast to shift the colourmap? */
@@ -780,32 +697,27 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 		/* store the colours in the colour map */
 		XStoreColors(display, swirl->cmap, swirl->rgb_values, swirl->colours);
 	} else {
-#if defined(__cplusplus) || defined(c_plusplus)
-		if (default_info->c_class == TrueColor)
-#else
-		if (default_info->class == TrueColor)
-#endif
-		{
+		if (truecolor) {
 			int         rsh, gsh, bsh;
 			unsigned long int t;
 
-			t = default_info->red_mask;
+			t = redmask;
 			for (i = 0; (int) t > 0; i++, t >>= 1);
 			rsh = 16 - i;
-			t = default_info->green_mask;
+			t = greenmask;
 			for (i = 0; (int) t > 0; i++, t >>= 1);
 			gsh = 16 - i;
-			t = default_info->blue_mask;
+			t = bluemask;
 			for (i = 0; (int) t > 0; i++, t >>= 1);
 			bsh = 16 - i;
 			for (i = 0; i < swirl->colours; i++)
 				swirl->rgb_values[i].pixel =
 					((rsh > 0 ? (swirl->rgb_values[i].red) >> rsh :
-					  (swirl->rgb_values[i].red) << (-rsh)) & default_info->red_mask) |
+					  (swirl->rgb_values[i].red) << (-rsh)) & redmask) |
 					((gsh > 0 ? (swirl->rgb_values[i].green) >> gsh :
-					  (swirl->rgb_values[i].green) << (-gsh)) & default_info->green_mask) |
+					  (swirl->rgb_values[i].green) << (-gsh)) & greenmask) |
 					((bsh > 0 ? (swirl->rgb_values[i].blue) >> bsh :
-					  (swirl->rgb_values[i].blue) << (-bsh)) & default_info->blue_mask);
+					  (swirl->rgb_values[i].blue) << (-bsh)) & bluemask);
 		} else {
 			/* lookup the colours in the fixed colour map */
 			for (i = 0; i < swirl->colours; i++)
@@ -1015,7 +927,11 @@ do_point(SWIRL_P swirl, int i, int j)
 					add = (int) (dcolours / (1.0 + 0.01 * abs(knot->m) * dist));
 					break;
 				case WHEEL:
-					theta = (atan2(dy, dx) + M_PI) / M_PI;
+					/* Avoid atan2: DOMAIN error message */
+					if (dy == 0.0 && dx == 0.0)
+						theta = 1.0;
+					else
+						theta = (atan2(dy, dx) + M_PI) / M_PI;
 					if (theta < 1.0)
 						add = (int) (dcolours * theta +
 						  sin(0.1 * knot->m * dist) *
@@ -1030,11 +946,20 @@ do_point(SWIRL_P swirl, int i, int j)
 					  fabs(cos(0.002 * knot->m * dist)));
 					break;
 				case RAY:
-					add = (int) (dcolours * fabs(sin(2.0 * atan2(dy, dx))));
+					/* Avoid atan2: DOMAIN error message */
+					if (dy == 0.0 && dx == 0.0)
+						add = 0;
+					else
+						add = (int) (dcolours * fabs(sin(2.0 * atan2(dy, dx))));
+
 					break;
 				case HOOK:
-					add = (int) (rads * atan2(dy, dx) +
-					   0.05 * (abs(knot->m) - 1) * dist);
+					/* Avoid atan2: DOMAIN error message */
+					if (dy == 0.0 && dx == 0.0)
+						add = (int) (0.05 * (abs(knot->m) - 1) * dist);
+					else
+						add = (int) (rads * atan2(dy, dx) +
+							     0.05 * (abs(knot->m) - 1) * dist);
 					break;
 			}
 		/* for a +ve mass add on the contribution else take it off */
@@ -1069,7 +994,7 @@ do_point(SWIRL_P swirl, int i, int j)
 	value = value % swirl->colours;
 
 	/* lookup the pixel value if necessary */
-	if (swirl->fixed_colourmap && !swirl->monochrome)
+	if (swirl->fixed_colourmap && swirl->dcolours > 2)
 		value = swirl->rgb_values[value].pixel;
 
 	/* return it */
@@ -1132,12 +1057,15 @@ draw_point(ModeInfo * mi, SWIRL_P swirl)
 		/* interleave blocks at half r */
 		draw_block(swirl->ximage, x, y, r2, do_point(swirl, x, y));
 		draw_block(swirl->ximage, x + r2, y, r2, do_point(swirl, x + r2, y));
-		draw_block(swirl->ximage, x + r2, y + r2, r2, do_point(swirl, x + r2, y + r2));
+		draw_block(swirl->ximage, x + r2, y + r2, r2, do_point(swirl,
+			x + r2, y + r2));
 		draw_block(swirl->ximage, x, y + r2, r2, do_point(swirl, x, y + r2));
 	} else
 		draw_block(swirl->ximage, x, y, r, do_point(swirl, x, y));
 
 	/* update the screen */
+/* PURIFY 4.0.1 on SunOS4 and on Solaris 2 reports a 256 byte memory leak on * 
+   the next line. */
 	XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi), swirl->ximage,
 		  x, y, x, y, r, r);
 }
@@ -1152,8 +1080,6 @@ draw_point(ModeInfo * mi, SWIRL_P swirl)
 static void
 next_point(SWIRL_P swirl)
 {
-	static Bool off_screen;
-
 	/* more to do in this direction? */
 	if (swirl->dir_done < swirl->dir_todo) {
 		/* move in the current direction */
@@ -1188,11 +1114,11 @@ next_point(SWIRL_P swirl)
 					swirl->y += (swirl->dir_todo * swirl->r);
 
 					/* check for finish */
-					if (off_screen)
+					if (swirl->off_screen)
 						swirl->drawing = False;
-					off_screen = True;
+					swirl->off_screen = True;
 				} else
-					off_screen = False;
+					swirl->off_screen = False;
 				break;
 			case DRAW_DOWN:
 				swirl->direction = DRAW_LEFT;
@@ -1203,11 +1129,11 @@ next_point(SWIRL_P swirl)
 					swirl->x -= (swirl->dir_todo * swirl->r);
 
 					/* check for finish */
-					if (off_screen)
+					if (swirl->off_screen)
 						swirl->drawing = False;
-					off_screen = True;
+					swirl->off_screen = True;
 				} else
-					off_screen = False;
+					swirl->off_screen = False;
 				break;
 			case DRAW_LEFT:
 				swirl->direction = DRAW_UP;
@@ -1217,11 +1143,11 @@ next_point(SWIRL_P swirl)
 					swirl->y -= (swirl->dir_todo * swirl->r);
 
 					/* check for finish */
-					if (off_screen)
+					if (swirl->off_screen)
 						swirl->drawing = False;
-					off_screen = True;
+					swirl->off_screen = True;
 				} else
-					off_screen = False;
+					swirl->off_screen = False;
 				break;
 			case DRAW_UP:
 				swirl->direction = DRAW_RIGHT;
@@ -1232,11 +1158,11 @@ next_point(SWIRL_P swirl)
 					swirl->x += (swirl->dir_todo * swirl->r);
 
 					/* check for finish */
-					if (off_screen)
+					if (swirl->off_screen)
 						swirl->drawing = False;
-					off_screen = True;
+					swirl->off_screen = True;
 				} else
-					off_screen = False;
+					swirl->off_screen = False;
 				break;
 		}
 	}
