@@ -1,6 +1,6 @@
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)utils.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)utils.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -47,6 +47,8 @@ static const char sccsid[] = "@(#)utils.c	4.00 97/01/01 xlockmore";
 #include <sys/select.h>
 #endif
 #endif
+#else
+#include <starlet.h>
 #endif
 #if defined( SYSV ) || defined( SVR4 )
 #ifdef LESS_THAN_AIX3_2
@@ -101,16 +103,16 @@ usleep(unsigned long usec)
  * returns the number of seconds since 01-Jan-70.
  * This is used to control rate and timeout in many of the animations.
  */
-long
+unsigned long
 seconds(void)
 {
 #if HAVE_GETTIMEOFDAY
 	struct timeval now;
 
 	(void) gettimeofday(&now, NULL);
-	return now.tv_sec;
+	return (unsigned long) now.tv_sec;
 #else
-	return (long) time(NULL);
+	return (unsigned long) time((time_t *) 0);
 #endif
 }
 
@@ -131,7 +133,7 @@ strdup(char *str)
 {
 	register char *ptr;
 
-	ptr = malloc(strlen(str) + 1);
+	ptr = (char *) malloc(strlen(str) + 1);
 	(void) strcpy(ptr, str);
 	return ptr;
 }
@@ -396,10 +398,10 @@ getFont(Display * display)
 	return mode_font;
 }
 
+#if HAVE_DIRENT_H
 static void
 randomImage(char *im_file)
 {
-#if HAVE_DIRENT_H
 	extern char directory_r[DIRBUF];
 	extern struct dirent **image_list;
 	extern int  num_list;
@@ -410,9 +412,10 @@ randomImage(char *im_file)
 		(void) strcpy(im_file, directory_r);
 		(void) strcat(im_file, image_list[num]->d_name);
 	}
-#endif
 }
 
+#endif
+
 #if HAVE_DIRENT_H
 
 extern Bool debug;
@@ -655,7 +658,9 @@ getImage(ModeInfo * mi, XImage ** logo,
 
 	*graphics_format = 0;
 
+#if HAVE_DIRENT_H
 	randomImage(imagefile);
+#endif
 
 	if (strlen(imagefile))
 		if (readable(imagefile)) {
@@ -781,3 +786,329 @@ XEraseImage(Display * display, Window window, GC gc,
 			XFillRectangle(display, window, gc, xlast, ylast, xsize, ysize);
 	}
 }
+
+static struct visual_class_name {
+	int         class;
+	char       *name;
+} VisualClassName[] = {
+
+	{
+		StaticGray, "StaticGray"
+	},
+	{
+		GrayScale, "GrayScale"
+	},
+	{
+		StaticColor, "StaticColor"
+	},
+	{
+		PseudoColor, "PseudoColor"
+	},
+	{
+		TrueColor, "TrueColor"
+	},
+	{
+		DirectColor, "DirectColor"
+	},
+	{
+		-1, NULL
+	},
+};
+
+int
+visualClassFromName(char *name)
+{
+	int         a;
+	char       *s1, *s2;
+	int         class = -1;
+
+	for (a = 0; VisualClassName[a].name; a++) {
+		for (s1 = VisualClassName[a].name, s2 = name; *s1 && *s2; s1++, s2++)
+			if ((isupper(*s1) ? tolower(*s1) : *s1) !=
+			    (isupper(*s2) ? tolower(*s2) : *s2))
+				break;
+
+		if ((*s1 == '\0') || (*s2 == '\0')) {
+
+			if (class != -1) {
+				(void) fprintf(stderr,
+					       "%s does not uniquely describe a visual class (ignored)\n", name);
+				return (-1);
+			}
+			class = VisualClassName[a].class;
+		}
+	}
+	if (class == -1)
+		(void) fprintf(stderr, "%s is not a visual class (ignored)\n", name);
+	return (class);
+}
+
+char
+           *
+nameOfVisualClass(int class)
+{
+	int         a;
+
+	for (a = 0; VisualClassName[a].name; a++)
+		if (VisualClassName[a].class == class)
+			return (VisualClassName[a].name);
+	return ("[Unknown Visual Class]");
+}
+
+#if defined(__cplusplus) || defined(c_plusplus)
+#define DEFAULT_VIS(v)    v.c_class = DefaultVisual(display, screen)->c_class;
+#define WANTED_VIS(v)   v.c_class = VisualClassWanted;
+#define CHECK_WANTED_VIS(v)      ((v)->c_class == VisualClassWanted)
+#define NAME_VIS(v) nameOfVisualClass(v->c_class)
+#else
+#define DEFAULT_VIS(v)    v.class = DefaultVisual(display, screen)->class;
+#define WANTED_VIS(v)   v.class = VisualClassWanted;
+#define CHECK_WANTED_VIS(v)      ((v)->class == VisualClassWanted)
+#define NAME_VIS(v) nameOfVisualClass(v->class)
+#endif
+
+void
+showVisualInfo(XVisualInfo * Vis)
+{
+	(void) fprintf(stderr, "Visual info: ");
+	(void) fprintf(stderr, "screen %d, ", Vis->screen);
+	(void) fprintf(stderr, "class %s, ", NAME_VIS(Vis));
+	(void) fprintf(stderr, "depth %d\n", Vis->depth);
+}
+
+/*-
+ * default_visual_info
+ *
+ * Gets a XVisualInfo structure that refers to a given visual or the default
+ * visual.
+ *
+ * -      mi is the ModeInfo
+ * -      visual is the visual to look up NULL => look up the default visual
+ * -      default_info is set to point to the member of the returned list
+ *          that corresponds to the default visual.
+ *
+ * Returns a list of XVisualInfo structures or NULL on failure. Free the list
+ *   with XFree.
+ */
+static XVisualInfo *
+defaultVisualInfo(ModeInfo * mi, Visual * visual, XVisualInfo ** default_info)
+{
+	Display    *display = MI_DISPLAY(mi);
+	int         screen = MI_SCREEN(mi);
+	XVisualInfo *info_list, vTemplate;
+	int         i, n;
+	extern int  VisualClassWanted;
+
+	/* get a complete list of visuals */
+	/*info_list = XGetVisualInfo(display, VisualNoMask, NULL, &n); */
+	vTemplate.screen = screen;
+	vTemplate.depth = MI_WIN_DEPTH(mi);
+	if (VisualClassWanted == -1) {
+		DEFAULT_VIS(vTemplate);
+	} else {
+		WANTED_VIS(vTemplate);
+	}
+	info_list = XGetVisualInfo(display,
+			VisualScreenMask | VisualDepthMask | VisualClassMask,
+				   &vTemplate, &n);
+	if (VisualClassWanted != -1 && n == 0) {
+		/* Wanted visual not found so use default */
+		DEFAULT_VIS(vTemplate);
+		info_list = XGetVisualInfo(display,
+			VisualScreenMask | VisualDepthMask | VisualClassMask,
+					   &vTemplate, &n);
+	}
+/*-
+  WANTED_VIS(vTemplate);
+   if (VisualClassWanted == -1)
+    info_list = XGetVisualInfo(display, VisualScreenMask | VisualDepthMask,
+      &vTemplate, &n);
+   else
+     info_list = XGetVisUalInfo(display,
+       VisualScreenMask | VisualDepthMask | VisualClassMask,
+       &vTemplateWant, &n);
+ */
+	if ((info_list == NULL) || (n == 0)) {
+		if (MI_WIN_IS_VERBOSE(mi))
+			(void) fprintf(stderr, "Could not get any Visuals, numvisuals = %d.\n",
+				       n);
+		return (NULL);
+	}
+	/* do we need to get the default visual? */
+	if (visual == NULL)
+		visual = DefaultVisual(display, screen);
+
+	*default_info = info_list;
+	if (VisualClassWanted == -1) {
+		/* search through the list for the default visual */
+		for (i = 0; i < n; i++, (*default_info)++)
+			if ((*default_info)->visual == visual &&
+			    (*default_info)->screen == screen)
+				break;
+	} else {
+		/* search through the list for the requested visual
+		 * with the greatest color depth */
+		for (i = 0; i < n; i++, (*default_info)++)
+			if ((*default_info)->depth >= 24 &&
+			    (*default_info)->screen == screen &&
+			    CHECK_WANTED_VIS(*default_info))
+				break;
+		if (i == n) {
+			*default_info = info_list;
+			for (i = 0; i < n; i++, (*default_info)++)
+				if ((*default_info)->depth == 16 &&
+				    (*default_info)->screen == screen &&
+				    CHECK_WANTED_VIS(*default_info))
+					break;
+		}
+		if (i == n) {
+			*default_info = info_list;
+			for (i = 0; i < n; i++, (*default_info)++)
+				if ((*default_info)->depth == 8 &&
+				    (*default_info)->screen == screen &&
+				    CHECK_WANTED_VIS(*default_info))
+					break;
+		}
+		if (i == n) {
+			*default_info = info_list;
+			for (i = 0; i < n; i++, (*default_info)++)
+				if ((*default_info)->depth == 4 &&
+				    (*default_info)->screen == screen &&
+				    CHECK_WANTED_VIS(*default_info))
+					break;
+		}
+		if (i == n) {
+			*default_info = info_list;
+			for (i = 0; i < n; i++, (*default_info)++)
+				if ((*default_info)->screen == screen &&
+				    CHECK_WANTED_VIS(*default_info))
+					break;
+		}
+	}
+	if (MI_WIN_IS_VERBOSE(mi)) {
+		(void) fprintf(stderr, "Visual info: ");
+		(void) fprintf(stderr, "screen %d, ", (*default_info)->screen);
+		(void) fprintf(stderr, "class %s, ", NAME_VIS((*default_info)));
+		(void) fprintf(stderr, "depth %d\n", (*default_info)->depth);
+	}
+	/* return the list (and the default info) */
+	return (info_list);
+}
+
+/*-
+ * setupColormap
+ *
+ * Create a read/write colourmap to use
+ *
+ */
+
+Bool
+setupColormap(ModeInfo * mi, int *colors, Bool * truecolor,
+   unsigned long *redmask, unsigned long *bluemask, unsigned long *greenmask)
+{
+	XVisualInfo *list, *default_info;
+
+	/* get information about the default visual */
+	list = defaultVisualInfo(mi, MI_VISUAL(mi), &default_info);
+
+	/* how many colours are there altogether? */
+	*colors = MI_NPIXELS(mi);
+	if (*colors > default_info->colormap_size) {
+		*colors = default_info->colormap_size;
+	}
+	if (*colors < 2)
+		*colors = 2;
+
+#if defined(__cplusplus) || defined(c_plusplus)
+	*truecolor = (default_info->c_class == TrueColor);
+#else
+	*truecolor = (default_info->class == TrueColor);
+#endif
+
+	*redmask = default_info->red_mask;
+	*greenmask = default_info->green_mask;
+	*bluemask = default_info->blue_mask;
+
+#ifdef FORCEFIXEDCOLORS
+	/* pretending a fixed colourmap */
+	XFree((char *list);
+	      return False;
+#else
+	{
+		Bool        fixed;
+
+		fixed = (
+#if defined(__cplusplus) || defined(c_plusplus)
+				(default_info->c_class != StaticGray) &&
+				(default_info->c_class != StaticColor) &&
+				(default_info->c_class != TrueColor) &&
+#else
+				(default_info->class != StaticGray) &&
+				(default_info->class != StaticColor) &&
+				(default_info->class != TrueColor) &&
+#endif
+				(MI_NPIXELS(mi) > 2) && !MI_WIN_IS_INROOT(mi) && MI_WIN_IS_INSTALL(mi));
+		XFree((char *) list);
+		return fixed;
+	}
+#endif
+}
+
+/*-
+ * useableColors
+ */
+int
+preserveColors(unsigned long fg, unsigned long bg,
+	       unsigned long white, unsigned long black)
+{
+	/* how many colours should we preserve (out of white, black, fg, bg)? */
+	if (((bg == black) || (bg == white)) && ((fg == black) || (fg == white)))
+		return 2;
+	else if ((bg == black) || (fg == black) ||
+		 (bg == white) || (fg == white) || (bg == fg))
+		return 3;
+	else
+		return 4;
+}
+
+#ifdef USE_GL
+
+#include <GL/gl.h>
+#include <GL/glx.h>
+
+static XVisualInfo *glVis[MAXSCREENS];
+
+int
+getVisual(ModeInfo * mi, XVisualInfo * wantVis, int mono)
+{
+	Display    *display = MI_DISPLAY(mi);
+	int         screen;
+	static int  first;
+
+	if (first) {
+		for (screen = 0; screen < MAXSCREENS; screen++)
+			glVis[screen] = NULL;
+	}
+	screen = MI_SCREEN(mi);
+
+	if (!glVis[screen]) {
+		if (mono) {
+			/* Monochrome display - use color index mode */
+			int         attribList[] =
+			{GLX_DOUBLEBUFFER, None};
+
+			glVis[screen] = glXChooseVisual(display, screen, attribList);
+		} else {
+			int         attribList[] =
+			{GLX_RGBA, GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, 1, None};
+
+			glVis[screen] = glXChooseVisual(display, screen, attribList);
+		}
+	}
+	/* check if GL can render into root window. */
+	if ((!glVis[screen]) || (glVis[screen]->visual != wantVis->visual)) {
+		return (0);
+	}
+	return (1);		/* success */
+}
+#endif
