@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)xlock.c	4.01 97/02/24 xlockmore";
+static const char sccsid[] = "@(#)xlock.c	4.02 97/04/01 xlockmore";
 
 #endif
 
@@ -219,12 +219,13 @@ extern int  usleep(unsigned int);
 
 #endif
 
-#if defined( USE_AUTO_LOGOUT ) || defined( USE_LOGOUT_BUTTON )
+#if defined( USE_AUTO_LOGOUT ) || defined( USE_BUTTON_LOGOUT )
 extern void logoutUser(void);
 
 #endif
 
-#if 0				/* #if !defined( AIXV3 ) && !defined( __hpux ) && !defined( __bsdi__ ) */
+#if 0
+	/* #if !defined( AIXV3 ) && !defined( __hpux ) && !defined( __bsdi__ ) */
 extern int  select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
 
 #if !defined( __hpux ) && !defined( apollo )
@@ -255,6 +256,7 @@ extern Bool allowroot;
 extern Bool debug;
 extern Bool echokeys;
 extern Bool enablesaver;
+extern Bool fullrandom;
 extern Bool grabmouse;
 extern Bool install;
 extern Bool timeelapsed;
@@ -263,6 +265,7 @@ extern Bool verbose;
 extern int  nicelevel;
 extern int  lockdelay;
 extern int  timeout;
+extern Bool wireframe;
 extern Bool use3d;
 
 extern char *fontname;
@@ -291,13 +294,13 @@ extern Bool sound;
 
 #endif
 #ifdef USE_AUTO_LOGOUT
-extern int  forceLogout;
+extern int  logoutAuto;
 
 #endif
 
-#ifdef USE_LOGOUT_BUTTON
-extern int  enable_button;
+#ifdef USE_BUTTON_LOGOUT
 extern int  logoutButton;
+extern int  enable_button;
 extern char *logoutButtonLabel;
 extern char *logoutButtonHelp;
 extern char *logoutFailedString;
@@ -316,7 +319,7 @@ static int  screens;		/* number of screens */
 static Window win[MAXSCREENS];	/* window used to cover screen */
 static Window icon[MAXSCREENS];	/* window used during password typein */
 
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 static Window button[MAXSCREENS];
 
 #endif
@@ -347,7 +350,7 @@ static int  sstimeout;		/* screen saver parameters */
 static int  ssinterval;
 static int  ssblanking;
 static int  ssexposures;
-static long start_time;
+static unsigned long start_time;
 static char *message[MESSAGELINES + 1];		/* Message is stored here */
 
 /* GEOMETRY STUFF */
@@ -355,7 +358,7 @@ static int  sizeconfiguremask;
 static XWindowChanges fullsizeconfigure, minisizeconfigure;
 static int  fullscreen = False;
 
-#if defined( USE_LOGOUT_BUTTON ) || defined( USE_AUTO_LOGOUT )
+#if defined( USE_AUTO_LOGOUT ) || defined( USE_BUTTON_LOGOUT )
 static int  tried_logout = 0;
 
 #endif
@@ -554,7 +557,8 @@ mode_info(int scrn, Window window, int iconic)
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_USE3D, use3d &&
 			    !(mono || CellsOfScreen(MI_SCREENPTR(mi)) <= 2));
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_VERBOSE, verbose);
-
+		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_FULLRANDOM, fullrandom);
+		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_WIREFRAME, wireframe);
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_INFO_INITTED, True);
 	}
 	if (MI_WINDOW(mi) != window) {
@@ -675,31 +679,30 @@ putText(Window w, GC gc, char *string, int bold, int left, int *px, int *py)
 static void
 statusUpdate(int isnew, int scr)
 {
-	int         left, x, y, now, len;
+	int         left, x, y, len;
 	char        buf[1024];
 	XWindowAttributes xgwa;
 	static int  last_time;
 
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 	int         ysave;
 	static      made_button, last_tried_lo = 0;
 
-#endif /* USE_LOGOUT_BUTTON */
+#endif /* USE_BUTTON_LOGOUT */
 
-	now = seconds();
-	len = (now - start_time) / 60;
+	len = (int) (seconds() - start_time) / 60;
 
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 	if (tried_logout && !last_tried_lo) {
 		last_tried_lo = 1;
 		isnew = 1;
 #ifdef USE_AUTO_LOGOUT
-		forceLogout = 0;
+		logoutAuto = 0;
 #endif
 	}
 	if (isnew)
 		made_button = 0;
-#endif /* USE_LOGOUT_BUTTON */
+#endif /* USE_BUTTON_LOGOUT */
 
 	if (isnew || last_time != len)
 		last_time = len;
@@ -739,7 +742,7 @@ statusUpdate(int isnew, int scr)
 				       len / 60, len % 60);
 		putText(win[scr], textgc[scr], buf, False, left, &x, &y);
 	}
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 	if (enable_button) {
 		if (logoutButton > len) {
 			int         tmp = logoutButton - len;
@@ -771,7 +774,7 @@ statusUpdate(int isnew, int scr)
 #endif
 					       tmp / 60, tmp % 60);
 			putText(win[scr], textgc[scr], buf, False, left, &x, &y);
-		} else if (!made_button || (isnew && tried_logout)) {
+		} else {
 			putText(win[scr], textgc[scr],
 #ifdef NL
 				"Schop me eruit                                               \n",
@@ -783,40 +786,43 @@ statusUpdate(int isnew, int scr)
 #endif
 #endif
 				False, left, &x, &y);
-			made_button = 1;
-			ysave = y;
-			y = xgwa.height * 0.6;
+			if (!made_button || (isnew && tried_logout)) {
+				made_button = 1;
+				ysave = y;
 
-			XUnmapWindow(dsp, button[scr]);
-			XSetForeground(dsp, Scr[scr].gc, bgcol[scr]);
-			XFillRectangle(dsp, win[scr], Scr[scr].gc, left, y,
-				       xgwa.width - left, xgwa.height - y);
-			XSetForeground(dsp, Scr[scr].gc, fgcol[scr]);
-
-			if (tried_logout) {
-				putText(win[scr], textgc[scr], logoutFailedString,
-					True, left, &x, &y);
-			} else {
-				XMoveWindow(dsp, button[scr], left, y);
-				XMapWindow(dsp, button[scr]);
-				XRaiseWindow(dsp, button[scr]);
-				(void) sprintf(buf, " %s ", logoutButtonLabel);
-				XSetForeground(dsp, Scr[scr].gc, WhitePixel(dsp, scr));
-				XDrawString(dsp, button[scr], Scr[scr].gc, 0, font->ascent + 1,
-					    buf, strlen(buf));
+			  y += font->ascent + font->descent + 8; /* Leave a gap for validating */
+				XUnmapWindow(dsp, button[scr]);
+				XSetForeground(dsp, Scr[scr].gc, bgcol[scr]);
+				XFillRectangle(dsp, win[scr], Scr[scr].gc, left, y,
+					       xgwa.width - left,
+				     5 + 2 * (font->ascent + font->descent));
 				XSetForeground(dsp, Scr[scr].gc, fgcol[scr]);
-				y += 5 + 2 * font->ascent + font->descent;
-				putText(win[scr], textgc[scr], logoutButtonHelp,
-					False, left, &x, &y);
+
+				if (tried_logout) {
+					putText(win[scr], textgc[scr], logoutFailedString,
+						True, left, &x, &y);
+				} else {
+					XMoveWindow(dsp, button[scr], left, y);
+					XMapWindow(dsp, button[scr]);
+					XRaiseWindow(dsp, button[scr]);
+					(void) sprintf(buf, " %s ", logoutButtonLabel);
+					XSetForeground(dsp, Scr[scr].gc, WhitePixel(dsp, scr));
+					XDrawString(dsp, button[scr], Scr[scr].gc, 0, font->ascent + 1,
+						    buf, strlen(buf));
+					XSetForeground(dsp, Scr[scr].gc, fgcol[scr]);
+					y += 5 + 2 * font->ascent + font->descent;
+					putText(win[scr], textgc[scr], logoutButtonHelp,
+						False, left, &x, &y);
+				}
+				y = ysave;
+				x = left;
 			}
-			y = ysave;
-			x = left;
 		}
 	}
-#endif /* USE_LOGOUT_BUTTON */
+#endif /* USE_BUTTON_LOGOUT */
 #ifdef USE_AUTO_LOGOUT
-	if (forceLogout) {
-		int         tmp = forceLogout - len;
+	if (logoutAuto) {
+		int         tmp = logoutAuto - len;
 
 		if (tmp < 60)
 			(void) sprintf(buf, "%d minute%s until auto-logout.    \n",
@@ -833,12 +839,12 @@ statusUpdate(int isnew, int scr)
 static void
 checkLogout(void)
 {
-	if (nolock || tried_logout || !forceLogout)
+	if (nolock || tried_logout || !logoutAuto)
 		return;
 
-	if (seconds() > forceLogout * 60 + start_time) {
+	if (logoutAuto * 60 < (int) (seconds() - start_time)) {
 		tried_logout = 1;
-		forceLogout = 0;
+		logoutAuto = 0;
 		logoutUser();
 	}
 }
@@ -886,7 +892,7 @@ runMainLoop(int maxtime, int iconscreen)
 	struct timeval sleep_time, first, repeat;
 	struct timeval elapsed, tmp;
 	fd_set      reads;
-	long        started;
+	unsigned long started;
 
 	first.tv_sec = first.tv_usec = 0;
 	elapsed.tv_sec = elapsed.tv_usec = 0;
@@ -924,14 +930,20 @@ runMainLoop(int maxtime, int iconscreen)
 		FD_ZERO(&reads);
 		FD_SET(fd, &reads);
 
-#ifdef DCE_PASSWD
+#if DCE_PASSWD
 		r = _select_sys(fd + 1,
 			(fd_set *) & reads, (fd_set *) NULL, (fd_set *) NULL,
 				(struct timeval *) &sleep_time);
+#else
+#if defined(__cplusplus) || defined(c_plusplus)
+		r = select(fd + 1,
+			(fd_set *) & reads, (fd_set *) NULL, (fd_set *) NULL,
+			   (struct timeval *) &sleep_time);
 #else
 		r = select(fd + 1,
 			   (void *) &reads, (void *) NULL, (void *) NULL,
 			   (struct timeval *) &sleep_time);
+#endif
 #endif
 
 		if (r == 1)
@@ -958,7 +970,8 @@ runMainLoop(int maxtime, int iconscreen)
 			if (cbwin != None) {
 				mi = mode_info(screen, cbwin, iconic);
 				call_callback_hook((LockStruct *) NULL, mi);
-				onepause = MI_PAUSE(mi);
+				if (MI_PAUSE(mi) > onepause)	/* Better on multiscreens to pause */
+					onepause = MI_PAUSE(mi);
 			}
 		}
 
@@ -968,7 +981,7 @@ runMainLoop(int maxtime, int iconscreen)
 		if (QLength(dsp)) {
 			return 0;
 		}
-		if (maxtime && ((seconds() - started) > maxtime)) {
+		if (maxtime && ((int) (seconds() - started) > maxtime)) {
 			return -1;
 		}
 		/* if (mindelay) usleep(mindelay); */
@@ -1028,7 +1041,7 @@ ReadXString(char *s, int slen)
 				break;
 #endif
 
-			if (seconds() - lasteventtime > (unsigned long) timeout) {
+			if (timeout < (int) (seconds() - lasteventtime)) {
 				screen = thisscreen;
 				return 1;
 			}
@@ -1102,7 +1115,7 @@ ReadXString(char *s, int slen)
 				if (((XButtonEvent *) & event)->window == icon[screen]) {
 					return 1;
 				}
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 				if (((XButtonEvent *) & event)->window == button[screen]) {
 					XSetFunction(dsp, Scr[screen].gc, GXxor);
 					XSetForeground(dsp, Scr[screen].gc, fgcol[screen]);
@@ -1220,11 +1233,11 @@ getPassword(void)
 	timey = y;
 
 #ifdef USE_AUTO_LOGOUT
-	if (forceLogout) {
+	if (logoutAuto) {
 		y += font->ascent + font->descent + 2;
 	}
 #endif
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 	if (enable_button) {
 		y += font->ascent + font->descent + 2;
 	}
@@ -1318,7 +1331,7 @@ getPassword(void)
 	}
 	XChangeGrabbedCursor(mycursor);
 	XUnmapWindow(dsp, icon[screen]);
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 	XUnmapWindow(dsp, button[screen]);
 #endif
 	if (!fullscreen)
@@ -1436,7 +1449,7 @@ justDisplay(void)
 		}
 
 		if (!nolock && lockdelay &&
-		    (lockdelay <= (seconds() - start_time))) {
+		    (lockdelay <= (int) (seconds() - start_time))) {
 			timetodie = True;
 			not_done = False;
 		}
@@ -1488,18 +1501,17 @@ syslogStop(void)
 {
 	struct passwd *pw;
 	struct group *gr;
-	int         seconds, minutes;
+	int         secs, mins;
 
-	seconds = time((time_t *) 0) - start_time;
-	minutes = seconds / 60;
-	seconds %= 60;
+	secs = (int) (seconds() - start_time);
+	mins = secs / 60;
+	secs %= 60;
 
 	pw = getpwuid(getuid());
 	gr = getgrgid(getgid());
 
 	syslog(LOG_INFO, "Stop: %s, %s, %s, %dm %ds",
-	       pw->pw_name, gr->gr_name, XDisplayString(dsp),
-	       minutes, seconds);
+	       pw->pw_name, gr->gr_name, XDisplayString(dsp), mins, secs);
 	closelog();
 }
 
@@ -1577,11 +1589,11 @@ read_message()
 	FILE       *msgf;
 	char        buf[121];
 	char       *home = getenv("HOME");
-  char       *buffer;
-  int         i, cr;
+	char       *buffer;
+	int         i, cr;
 
-  if (!home)
-    home = "";
+	if (!home)
+		home = "";
 
 #if ( __VMS_VER >= 70000000 )
 	buffer = (char *) malloc(255);
@@ -1667,22 +1679,43 @@ main(int argc, char **argv)
 	XColor      nullcolor;
 	char      **msgp;
 	int         tmp;
+#ifdef HAVE_SETEUID
+	uid_t       euid;
+	gid_t       egid;
+#endif
 
 #ifdef OSF1_ENH_SEC
 	set_auth_parameters(argc, argv);
 #endif
 
+#ifdef HAVE_SETEUID
+	/* save effective uid and gid for later */
+	euid = geteuid();
+	egid = getegid();
+
+	/* revoke root privs temporarily, to get the correct .Xauthority */
+	(void) setegid(getgid());
+	(void) seteuid(getuid());
+#endif
+
 	ProgramName = strrchr(argv[0], '/');
 	if (ProgramName)
 		ProgramName++;
 	else
 		ProgramName = argv[0];
 
-	start_time = time((time_t *) 0);
-	SRAND(start_time);	/* random mode needs the seed set. */
+	start_time = seconds();
+	SRAND((long) start_time);	/* random mode needs the seed set. */
+	/*SRAND(getpid()); */
 
 	getResources(argc, argv);
 
+#ifdef HAVE_SETEUID
+	/* become root to get the password */
+	(void) setegid(egid);
+	(void) seteuid(euid);
+#endif
+
 	initPasswd();
 
 	/* revoke root privs, if there were any */
@@ -1962,7 +1995,7 @@ main(int argc, char **argv)
 						 InputOutput, CopyFromParent,
 						     CIMASK, &xswa);
 
-#ifdef USE_LOGOUT_BUTTON
+#ifdef USE_BUTTON_LOGOUT
 		{
 			char        buf[1024];
 			int         w, h;
