@@ -43,11 +43,15 @@
 #else /* HAVE_CONFIG_H */
 
 /* THIS MAY SOON BE DEFUNCT, SHOULD WORK NOW THOUGH FOR IMAKE */
+#define inline
 #ifdef AIXV3
 #define HAVE_SYS_SELECT_H 1
 #else
 #define HAVE_SYS_TIME_H 1
 #endif
+#if !defined( __hpux ) && !defined( apollo ) && !defined( VMS )
+#define HAVE_SETEUID
+#endif
 #define HAVE_FCNTL_H 1
 #define HAVE_LIMITS_H 1
 #define HAVE_SYSLOG_H 1
@@ -159,6 +163,7 @@
 #include <unixlib.h>
 #endif
 #include <sys/types.h>
+#if 0
 #ifndef uid_t
 #define uid_t int
 #endif
@@ -171,6 +176,7 @@
 #ifndef size_t
 #define size_t unsigned
 #endif
+#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -268,7 +274,18 @@ extern void reserveColors(ModeInfo * mi, Colormap cmap,
 extern void fixColormap(Display * display, Window window,
 			int screen, float saturation,
 	  Bool mono, Bool install, Bool inroot, Bool inwindow, Bool verbose);
-extern long seconds(void);
+extern int preserveColors(unsigned long fg, unsigned long bg,
+  unsigned long white, unsigned long black);
+extern Bool setupColormap(ModeInfo * mi, int *colors, Bool * truecolor,
+ unsigned long *redmask, unsigned long *bluemask, unsigned long *greenmask);
+extern int  visualClassFromName(char *name);
+extern char *nameOfVisualClass(int class);
+extern void showVisualInfo(XVisualInfo * wantVis);
+#ifdef USE_GL
+extern int getVisual(ModeInfo * mi, XVisualInfo * wantVis, int mono);
+#endif
+
+extern unsigned long seconds(void);
 extern void finish(void);
 extern void error(char *s1,...);
 extern void warning(char *s1,...);
@@ -278,6 +295,48 @@ extern char *strdup(char *);
 
 #endif
 
+#if 0
+/* Sun's standard and GCC's header files leave out prototypes for all sorts of 
+   functions. */
+extern int  tolower(int);
+extern int  toupper(int);
+extern FILE *tmpfile(void);
+extern int  fclose(FILE *);
+extern char *fgets(char *, int, FILE *);
+extern int  fgetc(FILE *);
+extern int  fflush(FILE *);
+extern int  fprintf(FILE *, const char *,...);
+extern int  fputc(char, FILE *);
+extern int  fputs(const char *, FILE *);
+extern size_t fread(void *, size_t, size_t, FILE *);
+extern int  fscanf(FILE *, const char *,...);
+extern int  fgetpos(FILE *, long *);
+extern int  fseek(FILE *, long, int);
+extern int  fsetpos(FILE *, const long *);
+extern long ftell(FILE *);
+extern size_t fwrite(const void *, size_t, size_t, FILE *);
+extern char *gets(char *);
+extern int  pclose(FILE *);
+extern void perror(const char *);
+extern int  printf(const char *,...);
+extern int  puts(const char *);
+extern int  remove(const char *);
+extern int  rename(const char *, const char *);
+extern int  rewind(FILE *);
+extern int  scanf(const char *,...);
+extern int  sscanf(const char *, const char *,...);
+extern void setbuf(FILE *, char *);
+extern int  setvbuf(FILE *, char *, int, size_t);
+extern int  ungetc(int, FILE *);
+extern int  vprintf(const char *, void *);
+extern int  vfprintf(FILE *, const char *, void *);
+extern char *vsprintf(char *, const char *, void *);
+extern int  gethostname(char *, int);
+
+#endif
+
+
+
 #ifdef LESS_THAN_AIX3_2
 #undef NULL
 #define NULL 0
